/*
Code is partly based on pseudo code out of the paper:
"There Is No 16-Clue Sudoku: Solving the Sudoku Minimum Number of Clues Problem via Hitting Set Enumeration".
It is written by Gary McGuire, Bastian Tugemann, and Gilles Civario in 2014 and
published in Experimental Mathematics, 23:190217
*/
#include "Blueprints.h"
#include <ostream>
int UnavPat::GetSize() const {
	int size = 0;
	for ( int i = 0;  i < digits;  i++ )
		size += groups[ i ].size;
	return size;
}
std::vector<byte> UnavPat::toByteVector() {
	std::vector<byte> vec;
	for ( int i = 0;  i < digits;  i++ ) {
		for (int j = 0; j < groups[i].size; ++j) {
			vec.push_back(groups[i].data[j]);
		}
	}
	return vec;
}

bool UnavPat::shrink(int n) {
	UnavPat temp = *this;
	// delete rows if possible
	for (int bandIndex = 0; bandIndex < 3; bandIndex++) {
		int rowsToDelete = 3 - n;
		for (int rowIndex = 0; rowsToDelete > 0 && rowIndex < 3; rowIndex++) {
			// if row is empty
			bool containsElementInRow = false;
			for (int i = 0; i < digits; i++) {
				for (int j = 0; j < temp.groups[i].size; ++j) {
					if (temp.groups[i].data[j] / 10 == (bandIndex * 3 + 1) + rowIndex) {
						containsElementInRow = true;
						goto end;
					}
				}
			}
		end:if (!containsElementInRow) {
				// for every next row move elements up
				for (int followingRow = (bandIndex * 3 + 1) + rowIndex; followingRow <= 9; followingRow++) {
					for (int i = 0; i < digits; i++) {
						for (int j = 0; j < temp.groups[i].size; ++j) {
							if (temp.groups[i].data[j] / 10 == followingRow) {
								groups[i].data[j] = groups[i].data[j] - 10;
							}
						}
					}
				}
				rowsToDelete--;
			}
		}
		if (rowsToDelete > 0) {
			return false;
		}
	}

	// delete cols if possible
	for (int stackIndex = 0; stackIndex < 3; stackIndex++) {
		int colsToDelete = 3 - n;
		for (int colIndex = 0; colsToDelete > 0 && colIndex < 3; colIndex++) {
			// if col is empty
			bool containsElementInCol = false;
			for (int i = 0; i < digits; i++) {
				for (int j = 0; j < temp.groups[i].size; ++j) {
					if (temp.groups[i].data[j] % 10 == (stackIndex * 3 + 1) + colIndex) {
						containsElementInCol = true;
						goto end2;
					}
				}
			}
		end2:if (!containsElementInCol) {
				// for every next col move elements up
				for (int followingCol = (stackIndex * 3 + 1) + colIndex; followingCol <= 9; followingCol++) {
					for (int i = 0; i < digits; i++) {
						for (int j = 0; j < temp.groups[i].size; ++j) {
							if (temp.groups[i].data[j] % 10 == followingCol) {
								groups[i].data[j] = groups[i].data[j] - 1;
							}
						}
					}
				}
				colsToDelete--;
			}
		}
		if (colsToDelete > 0) {
			return false;
		}
	}
	return true;
}

//bool UnavPat::shrinkOld() {
//	for ( int i = 0;  i < digits;  i++ ) {
//		for (int j = 0; j < groups[i].size; ++j) {
//			if ((groups[i].data[j] / 10) % 3 == 0 || (groups[i].data[j] % 10) % 3 == 0) {
//				return false;
//			}
//			else {
//				groups[i].data[j] = (groups[i].data[j] / 10 - (groups[i].data[j] / 10) / 4) * 10 +
//					groups[i].data[j] % 10 - (groups[i].data[j] % 10) / 4;
//			}
//		}
//	}
//	return true;
//}

std::ostream& operator<<(std::ostream &strm, const UnavPat &bp) {
	for ( int i = 0;  i < bp.digits;  i++ ) {
		for (int j = 0; j < bp.groups[i].size; ++j) {
			strm << bp.groups[i].data[j] << ",";
		}
		strm << std::endl;
	}
  return strm;
}
