/*
Code is partly based on pseudo code out of the paper:
"There Is No 16-Clue Sudoku: Solving the Sudoku Minimum Number of Clues Problem via Hitting Set Enumeration".
It is written by Gary McGuire, Bastian Tugemann, and Gilles Civario in 2014 and
published in Experimental Mathematics, 23:190217
*/

#ifndef SUDOKUGRID_H_
#define SUDOKUGRID_H_
#include <vector>
#include <stack>
#include <tuple>
#include "Blueprints.h"
typedef unsigned char byte;

unsigned int factorial(unsigned int n);

class SudokuGrid {
public:
	SudokuGrid(int size);
	SudokuGrid(const SudokuGrid& copy, const std::vector<byte>& cells);
	SudokuGrid(int size, const std::vector<byte>& cells, bool transformed = false);
	SudokuGrid(std::vector<byte> digits);
	SudokuGrid(int size, UnavPat pattern);
	std::vector<std::vector<byte>> grid;
	int n;

	bool operator ==(const SudokuGrid &b) const;
	SudokuGrid(const SudokuGrid& copy);
	SudokuGrid& transpose();
	SudokuGrid& swapRows(int row1, int row2);
	SudokuGrid& swapCols(int col1, int col2);
	SudokuGrid& swapBands(int band1, int band2);
	SudokuGrid& swapStacks(int stack1, int stack2);
	SudokuGrid& permuteCols(int stackindex, int index, bool reversed, int columnindex = 0, std::stack<std::tuple<int, int> > stack = std::stack<std::tuple<int, int> >());
	SudokuGrid& permuteRows(int bandindex, int index, bool reversed, int rowindex = 0, std::stack<std::tuple<int, int> > stack = std::stack<std::tuple<int, int> >());
	SudokuGrid& permuteStacks(int index, bool reversed, int stackindex = 0, std::stack<std::tuple<int, int> > stack = std::stack<std::tuple<int, int> >());
	SudokuGrid& permuteBands(int index, bool reversed, int bandindex = 0, std::stack<std::tuple<int, int> > stack = std::stack<std::tuple<int, int> >());
	SudokuGrid& permuteBandsNonRec(int index, bool reversed, int bandindex = 0, std::stack<std::tuple<int, int> > stack = std::stack<std::tuple<int, int> >());

	std::vector<byte> getNotZeroCells();
};






#endif /* SUDOKUGRID_H_ */
