/*
Code is partly based on pseudo code out of the paper:
"There Is No 16-Clue Sudoku: Solving the Sudoku Minimum Number of Clues Problem via Hitting Set Enumeration".
It is written by Gary McGuire, Bastian Tugemann, and Gilles Civario in 2014 and
published in Experimental Mathematics, 23:190217

The blueprint datastructures in particular are out of the program "checker".
Thanks to Gary McGuire for letting me use them in my code.
"checker" can be found on the following site: http://www.math.ie/checker.html
I accessed the site in June 2016.
*/

#ifndef BLUEPRINTS_H_
#define BLUEPRINTS_H_
#include <ostream>
#include <vector>
typedef unsigned char byte;

struct SmallSet
   {
   int size;
   int data[ 9 ];
   };

struct UnavPat
   {
   int digits;             // # of different digits occurring
   SmallSet groups[ 9 ];   // each group consists of those cells containing the same digit
   // we only use groups[ 0 ], ..., groups[ digits - 1 ]
   int GetSize() const;
   std::vector<byte> toByteVector();
   bool shrink(int n);
   };

std::ostream& operator<<(std::ostream &strm, const UnavPat &bp);

UnavPat const unavpat1x2[] =
   {
   {2, {{2,{11,24}}, {2,{14,21}} } },
   {3, {{2,{11,24}}, {2,{14,31}}, {2,{21,34}} } },
   {5, {{2,{11,24}}, {2,{13,36}}, {2,{14,23}}, {2,{16,21}}, {2,{26,33}} } },
   {5, {{2,{11,24}}, {2,{13,35}}, {2,{14,23}}, {2,{15,21}}, {2,{25,33}} } },
   {6, {{2,{11,24}}, {2,{12,36}}, {2,{14,22}}, {2,{16,31}}, {2,{21,35}}, {2,{25,32}} } },
   {6, {{2,{11,24}}, {2,{12,35}}, {2,{14,22}}, {2,{15,31}}, {2,{21,36}}, {2,{26,32}} } },
   {6, {{2,{11,24}}, {2,{13,26}}, {2,{14,33}}, {2,{16,31}}, {2,{21,34}}, {2,{23,36}} } },
   {6, {{2,{11,24}}, {2,{13,25}}, {2,{14,33}}, {2,{15,31}}, {2,{21,34}}, {2,{23,35}} } },
   };
int const cntpat1x2 = 8;

UnavPat const unavpat1x3[] =
   {
   {3, {{2,{11,24}}, {2,{14,27}}, {2,{17,21}} } },
   {2, {{3,{11,24,37}}, {3,{17,21,34}} } },
   {4, {{2,{11,24}}, {2,{12,27}}, {2,{14,22}}, {2,{17,21}} } },
   {4, {{2,{11,24}}, {2,{14,37}}, {2,{17,21}}, {2,{27,34}} } },
   {5, {{2,{11,24}}, {2,{12,29}}, {2,{14,27}}, {2,{17,22}}, {2,{19,21}} } },
   {5, {{2,{11,24}}, {2,{12,28}}, {2,{14,27}}, {2,{17,22}}, {2,{18,21}} } },
   {5, {{2,{11,24}}, {2,{14,37}}, {2,{17,21}}, {2,{23,34}}, {2,{27,33}} } },
   {4, {{2,{11,24}}, {2,{13,36}}, {3,{14,27,33}}, {3,{16,21,37}} } },
   {4, {{2,{11,24}}, {2,{13,35}}, {3,{14,27,33}}, {3,{15,21,37}} } },
   {5, {{2,{11,24}}, {2,{14,27}}, {2,{16,21}}, {2,{17,36}}, {2,{26,37}} } },
   {5, {{2,{11,24}}, {2,{14,27}}, {2,{15,21}}, {2,{17,35}}, {2,{25,37}} } },
   {4, {{2,{11,24}}, {3,{14,23,37}}, {2,{15,21}}, {3,{17,25,33}} } },
   {4, {{2,{11,24}}, {3,{14,23,37}}, {2,{16,21}}, {3,{17,26,33}} } },
   {5, {{2,{11,24}}, {2,{13,35}}, {3,{14,27,33}}, {2,{15,21}}, {2,{25,37}} } },
   {5, {{2,{11,24}}, {2,{13,36}}, {3,{14,27,33}}, {2,{16,21}}, {2,{26,37}} } },
   {5, {{3,{11,24,38}}, {2,{16,21}}, {2,{18,36}}, {2,{26,37}}, {2,{27,34}} } },
   {5, {{3,{11,24,39}}, {2,{16,21}}, {2,{19,36}}, {2,{26,37}}, {2,{27,34}} } },
   {5, {{3,{11,24,38}}, {2,{15,21}}, {2,{18,35}}, {2,{25,37}}, {2,{27,34}} } },
   {5, {{3,{11,24,39}}, {2,{15,21}}, {2,{19,35}}, {2,{25,37}}, {2,{27,34}} } },
   {6, {{2,{11,24}}, {2,{13,26}}, {2,{14,27}}, {2,{16,28}}, {2,{17,23}}, {2,{18,21}} } },
   {6, {{2,{11,24}}, {2,{13,26}}, {2,{14,27}}, {2,{16,29}}, {2,{17,23}}, {2,{19,21}} } },
   {6, {{2,{11,24}}, {2,{13,25}}, {2,{14,27}}, {2,{15,28}}, {2,{17,23}}, {2,{18,21}} } },
   {6, {{2,{11,24}}, {2,{13,25}}, {2,{14,27}}, {2,{15,29}}, {2,{17,23}}, {2,{19,21}} } },
   {5, {{3,{11,24,37}}, {2,{12,35}}, {3,{14,27,32}}, {2,{15,38}}, {2,{18,31}} } },
   {5, {{3,{11,24,37}}, {2,{12,35}}, {3,{14,27,32}}, {2,{15,39}}, {2,{19,31}} } },
   {5, {{3,{11,24,37}}, {2,{12,36}}, {3,{14,27,32}}, {2,{16,38}}, {2,{18,31}} } },
   {5, {{3,{11,24,37}}, {2,{12,36}}, {3,{14,27,32}}, {2,{16,39}}, {2,{19,31}} } },
   {6, {{2,{11,24}}, {2,{12,37}}, {2,{14,32}}, {2,{17,21}}, {2,{23,34}}, {2,{27,33}} } },
   {5, {{2,{11,24}}, {2,{13,38}}, {3,{14,27,33}}, {3,{18,21,35}}, {2,{25,37}} } },
   {5, {{2,{11,24}}, {2,{13,39}}, {3,{14,27,33}}, {3,{19,21,35}}, {2,{25,37}} } },
   {5, {{2,{11,24}}, {2,{13,38}}, {3,{14,27,33}}, {3,{18,21,36}}, {2,{26,37}} } },
   {5, {{2,{11,24}}, {2,{13,39}}, {3,{14,27,33}}, {3,{19,21,36}}, {2,{26,37}} } },
   {6, {{2,{11,24}}, {2,{13,25}}, {2,{14,33}}, {2,{15,27}}, {2,{17,21}}, {2,{23,34}} } },
   {6, {{2,{11,24}}, {2,{13,26}}, {2,{14,33}}, {2,{16,27}}, {2,{17,21}}, {2,{23,34}} } },
   {5, {{3,{11,24,37}}, {3,{12,27,35}}, {2,{14,22}}, {2,{15,38}}, {2,{18,31}} } },
   {5, {{3,{11,24,37}}, {3,{12,27,35}}, {2,{14,22}}, {2,{15,39}}, {2,{19,31}} } },
   {5, {{3,{11,24,37}}, {3,{12,27,36}}, {2,{14,22}}, {2,{16,38}}, {2,{18,31}} } },
   {5, {{3,{11,24,37}}, {3,{12,27,36}}, {2,{14,22}}, {2,{16,39}}, {2,{19,31}} } },
   {5, {{3,{11,24,37}}, {3,{12,25,38}}, {2,{14,22}}, {2,{18,31}}, {2,{27,35}} } },
   {5, {{3,{11,24,37}}, {3,{12,25,39}}, {2,{14,22}}, {2,{19,31}}, {2,{27,35}} } },
   {5, {{3,{11,24,37}}, {3,{12,26,38}}, {2,{14,22}}, {2,{18,31}}, {2,{27,36}} } },
   {5, {{3,{11,24,37}}, {3,{12,26,39}}, {2,{14,22}}, {2,{19,31}}, {2,{27,36}} } },
   {6, {{2,{11,24}}, {2,{13,27}}, {2,{14,23}}, {2,{16,29}}, {2,{17,26}}, {2,{19,21}} } },
   {6, {{2,{11,24}}, {2,{13,27}}, {2,{14,23}}, {2,{16,28}}, {2,{17,26}}, {2,{18,21}} } },
   {6, {{2,{11,24}}, {2,{13,27}}, {2,{14,23}}, {2,{15,29}}, {2,{17,25}}, {2,{19,21}} } },
   {6, {{2,{11,24}}, {2,{13,27}}, {2,{14,23}}, {2,{15,28}}, {2,{17,25}}, {2,{18,21}} } },
   {4, {{3,{11,24,38}}, {3,{13,26,37}}, {3,{17,23,34}}, {3,{18,21,36}} } },
   {4, {{3,{11,24,39}}, {3,{13,26,37}}, {3,{17,23,34}}, {3,{19,21,36}} } },
   {4, {{3,{11,24,38}}, {3,{13,25,37}}, {3,{17,23,34}}, {3,{18,21,35}} } },
   {4, {{3,{11,24,39}}, {3,{13,25,37}}, {3,{17,23,34}}, {3,{19,21,35}} } },
   {5, {{3,{11,24,39}}, {2,{12,37}}, {3,{14,29,32}}, {2,{15,31}}, {2,{17,35}} } },
   {5, {{3,{11,24,38}}, {2,{12,37}}, {3,{14,28,32}}, {2,{15,31}}, {2,{17,35}} } },
   {5, {{3,{11,24,39}}, {2,{12,37}}, {3,{14,29,32}}, {2,{16,31}}, {2,{17,36}} } },
   {5, {{3,{11,24,38}}, {2,{12,37}}, {3,{14,28,32}}, {2,{16,31}}, {2,{17,36}} } },
   {5, {{3,{11,24,37}}, {3,{14,23,38}}, {2,{18,21}}, {2,{27,33}}, {2,{28,31}} } },
   {5, {{3,{11,24,37}}, {3,{14,23,39}}, {2,{19,21}}, {2,{27,33}}, {2,{29,31}} } },
   {6, {{2,{11,24}}, {2,{12,27}}, {2,{14,32}}, {2,{15,21}}, {2,{17,25}}, {2,{22,34}} } },
   {6, {{2,{11,24}}, {2,{12,27}}, {2,{14,32}}, {2,{16,21}}, {2,{17,26}}, {2,{22,34}} } },
   {4, {{3,{11,24,37}}, {3,{12,28,35}}, {3,{15,21,38}}, {3,{17,22,34}} } },
   {4, {{3,{11,24,37}}, {3,{12,29,35}}, {3,{15,21,39}}, {3,{17,22,34}} } },
   {4, {{3,{11,24,37}}, {3,{12,28,36}}, {3,{16,21,38}}, {3,{17,22,34}} } },
   {4, {{3,{11,24,37}}, {3,{12,29,36}}, {3,{16,21,39}}, {3,{17,22,34}} } },
   {5, {{3,{11,24,37}}, {2,{12,38}}, {3,{17,22,34}}, {2,{18,21}}, {2,{28,32}} } },
   {5, {{3,{11,24,37}}, {2,{12,39}}, {3,{17,22,34}}, {2,{19,21}}, {2,{29,32}} } },
   {6, {{2,{11,24}}, {2,{14,37}}, {2,{15,31}}, {2,{17,25}}, {2,{21,35}}, {2,{27,34}} } },
   {6, {{2,{11,24}}, {2,{14,37}}, {2,{16,31}}, {2,{17,26}}, {2,{21,36}}, {2,{27,34}} } },
   {5, {{2,{11,24}}, {2,{14,31}}, {3,{16,21,37}}, {3,{17,26,33}}, {2,{23,34}} } },
   {5, {{2,{11,24}}, {2,{14,31}}, {3,{15,21,37}}, {3,{17,25,33}}, {2,{23,34}} } },
   {6, {{2,{11,24}}, {2,{14,37}}, {2,{17,31}}, {2,{21,39}}, {2,{23,34}}, {2,{29,33}} } },
   {6, {{2,{11,24}}, {2,{14,37}}, {2,{17,31}}, {2,{21,38}}, {2,{23,34}}, {2,{28,33}} } },
   {6, {{2,{11,24}}, {2,{14,27}}, {2,{16,39}}, {2,{17,36}}, {2,{19,31}}, {2,{21,37}} } },
   {6, {{2,{11,24}}, {2,{14,27}}, {2,{16,38}}, {2,{17,36}}, {2,{18,31}}, {2,{21,37}} } },
   {6, {{2,{11,24}}, {2,{14,27}}, {2,{15,39}}, {2,{17,35}}, {2,{19,31}}, {2,{21,37}} } },
   {6, {{2,{11,24}}, {2,{14,27}}, {2,{15,38}}, {2,{17,35}}, {2,{18,31}}, {2,{21,37}} } },
   };
int const cntpat1x3 = 74;

UnavPat const unavpat2x2[ 4 ][ 250 ] =
   {
   {
   {2, {{3,{11,24,43}}, {3,{14,23,41}}}},
   {3, {{3,{11,24,42}}, {3,{14,23,41}}, {2,{22,43}} } },
   {2, {{4,{11,24,46,63}}, {4,{16,23,41,64}} } },
   {2, {{4,{11,24,45,63}}, {4,{15,23,41,64}} } },
   {2, {{4,{11,24,46,53}}, {4,{16,23,41,54}} } },
   {2, {{4,{11,24,45,53}}, {4,{15,23,41,54}} } },
   {2, {{4,{11,24,45,63}}, {4,{14,23,41,65}} } },
   {2, {{4,{11,24,46,63}}, {4,{14,23,41,66}} } },
   {2, {{4,{11,24,45,53}}, {4,{14,23,41,55}} } },
   {2, {{4,{11,24,46,53}}, {4,{14,23,41,56}} } },
   {3, {{4,{11,24,45,53}}, {4,{14,23,41,65}}, {2,{55,63}} } },
   {3, {{4,{11,24,46,53}}, {4,{14,23,41,66}}, {2,{56,63}} } },
   {3, {{4,{11,24,45,63}}, {4,{14,23,41,55}}, {2,{65,53}} } },
   {3, {{4,{11,24,46,63}}, {4,{14,23,41,56}}, {2,{66,53}} } },
   {3, {{4,{11,24,43,56}}, {3,{16,31,54}}, {3,{23,34,41}} } },
   {3, {{4,{11,24,43,55}}, {3,{15,31,54}}, {3,{23,34,41}} } },
   {3, {{4,{11,24,43,66}}, {3,{16,31,64}}, {3,{23,34,41}} } },
   {3, {{4,{11,24,43,65}}, {3,{15,31,64}}, {3,{23,34,41}} } },
   {3, {{3,{11,24,63}}, {4,{14,23,41,66}}, {3,{26,44,61}} } },
   {3, {{3,{11,24,63}}, {4,{14,23,41,65}}, {3,{25,44,61}} } },
   {3, {{3,{11,24,53}}, {4,{14,23,41,56}}, {3,{26,44,51}} } },
   {3, {{3,{11,24,53}}, {4,{14,23,41,55}}, {3,{25,44,51}} } },
   {3, {{4,{11,24,55,63}}, {4,{15,23,41,64}}, {2,{45,51}} } },
   {3, {{4,{11,24,56,63}}, {4,{16,23,41,64}}, {2,{46,51}} } },
   {3, {{4,{11,24,65,53}}, {4,{15,23,41,54}}, {2,{45,61}} } },
   {3, {{4,{11,24,66,53}}, {4,{16,23,41,54}}, {2,{46,61}} } },
   {3, {{4,{11,24,46,63}}, {4,{14,23,41,56}}, {3,{16,53,64}} } },
   {3, {{4,{11,24,45,63}}, {4,{14,23,41,55}}, {3,{15,53,64}} } },
   {3, {{4,{11,24,46,53}}, {4,{14,23,41,66}}, {3,{16,63,54}} } },
   {3, {{4,{11,24,45,53}}, {4,{14,23,41,65}}, {3,{15,63,54}} } },
   {3, {{4,{11,24,43,56}}, {3,{16,31,55}}, {4,{23,35,41,54}} } },
   {3, {{4,{11,24,43,55}}, {3,{15,31,56}}, {4,{23,36,41,54}} } },
   {3, {{4,{11,24,43,66}}, {3,{16,31,65}}, {4,{23,35,41,64}} } },
   {3, {{4,{11,24,43,65}}, {3,{15,31,66}}, {4,{23,36,41,64}} } },
   {5, {{3,{11,24,43}}, {2,{12,36}}, {2,{14,22}}, {3,{16,23,41}}, {2,{26,32}} } },
   {5, {{3,{11,24,43}}, {2,{12,35}}, {2,{14,22}}, {3,{15,23,41}}, {2,{25,32}} } },
   {5, {{3,{11,24,43}}, {2,{12,25}}, {2,{14,22}}, {2,{15,31}}, {3,{23,35,41}} } },
   {5, {{3,{11,24,43}}, {2,{12,26}}, {2,{14,22}}, {2,{16,31}}, {3,{23,36,41}} } },
   {5, {{3,{11,24,43}}, {2,{14,31}}, {2,{22,34}}, {3,{23,35,41}}, {2,{25,32}} } },
   {5, {{3,{11,24,43}}, {2,{14,31}}, {2,{22,34}}, {3,{23,36,41}}, {2,{26,32}} } },
   {5, {{4,{11,24,43,65}}, {2,{15,21}}, {2,{23,41}}, {2,{25,44}}, {2,{45,61}} } },
   {5, {{4,{11,24,43,66}}, {2,{16,21}}, {2,{23,41}}, {2,{26,44}}, {2,{46,61}} } },
   {5, {{4,{11,24,43,55}}, {2,{15,21}}, {2,{23,41}}, {2,{25,44}}, {2,{45,51}} } },
   {5, {{4,{11,24,43,56}}, {2,{16,21}}, {2,{23,41}}, {2,{26,44}}, {2,{46,51}} } },
   {5, {{3,{11,24,42}}, {3,{14,23,41}}, {2,{22,53}}, {2,{43,55}}, {2,{45,52}} } },
   {5, {{3,{11,24,42}}, {3,{14,23,41}}, {2,{22,53}}, {2,{43,56}}, {2,{46,52}} } },
   {5, {{3,{11,24,42}}, {3,{14,23,41}}, {2,{22,63}}, {2,{43,65}}, {2,{45,62}} } },
   {5, {{3,{11,24,42}}, {3,{14,23,41}}, {2,{22,63}}, {2,{43,66}}, {2,{46,62}} } },
   {4, {{4,{11,24,53,66}}, {2,{16,31}}, {4,{23,36,41,54}}, {2,{46,61}} } },
   {4, {{4,{11,24,53,65}}, {2,{15,31}}, {4,{23,35,41,54}}, {2,{45,61}} } },
   {4, {{4,{11,24,63,56}}, {2,{16,31}}, {4,{23,36,41,64}}, {2,{46,51}} } },
   {4, {{4,{11,24,63,55}}, {2,{15,31}}, {4,{23,35,41,64}}, {2,{45,51}} } },
   {4, {{4,{11,24,45,63}}, {2,{15,31}}, {4,{23,35,41,54}}, {2,{53,64}} } },
   {4, {{4,{11,24,46,63}}, {2,{16,31}}, {4,{23,36,41,54}}, {2,{53,64}} } },
   {4, {{4,{11,24,45,53}}, {2,{15,31}}, {4,{23,35,41,64}}, {2,{63,54}} } },
   {4, {{4,{11,24,46,53}}, {2,{16,31}}, {4,{23,36,41,64}}, {2,{63,54}} } },
   {4, {{4,{11,24,43,66}}, {3,{16,31,65}}, {3,{23,35,41}}, {2,{25,64}} } },
   {4, {{4,{11,24,43,65}}, {3,{15,31,66}}, {3,{23,36,41}}, {2,{26,64}} } },
   {4, {{4,{11,24,43,56}}, {3,{16,31,55}}, {3,{23,35,41}}, {2,{25,54}} } },
   {4, {{4,{11,24,43,55}}, {3,{15,31,56}}, {3,{23,36,41}}, {2,{26,54}} } },
   {4, {{3,{11,24,43}}, {3,{14,46,63}}, {4,{15,23,41,64}}, {2,{16,45}} } },
   {4, {{3,{11,24,43}}, {3,{14,45,63}}, {4,{16,23,41,64}}, {2,{15,46}} } },
   {4, {{3,{11,24,43}}, {3,{14,46,53}}, {4,{15,23,41,54}}, {2,{16,45}} } },
   {4, {{3,{11,24,43}}, {3,{14,45,53}}, {4,{16,23,41,54}}, {2,{15,46}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,31,53,64}}, {4,{23,34,41,56}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,31,53,64}}, {4,{23,34,41,55}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,31,63,54}}, {4,{23,34,41,66}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,31,63,54}}, {4,{23,34,41,65}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,33,51,64}}, {4,{23,34,41,55}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,33,51,64}}, {4,{23,34,41,56}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,33,61,54}}, {4,{23,34,41,65}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,33,61,54}}, {4,{23,34,41,66}} } },
   {4, {{3,{11,24,43}}, {2,{14,45}}, {4,{15,23,41,66}}, {3,{26,44,63}} } },
   {4, {{3,{11,24,43}}, {2,{14,46}}, {4,{16,23,41,65}}, {3,{25,44,63}} } },
   {4, {{3,{11,24,43}}, {2,{14,45}}, {4,{15,23,41,56}}, {3,{26,44,53}} } },
   {4, {{3,{11,24,43}}, {2,{14,46}}, {4,{16,23,41,55}}, {3,{25,44,53}} } },
   {4, {{4,{11,24,45,63}}, {4,{15,23,41,54}}, {2,{43,64}}, {2,{44,53}} } },
   {4, {{4,{11,24,46,63}}, {4,{16,23,41,54}}, {2,{43,64}}, {2,{44,53}} } },
   {4, {{4,{11,24,45,53}}, {4,{15,23,41,64}}, {2,{43,54}}, {2,{44,63}} } },
   {4, {{4,{11,24,46,53}}, {4,{16,23,41,64}}, {2,{43,54}}, {2,{44,63}} } },
   {4, {{3,{11,24,43}}, {3,{14,46,53}}, {4,{16,23,41,64}}, {2,{54,63}} } },
   {4, {{3,{11,24,43}}, {3,{14,45,53}}, {4,{15,23,41,64}}, {2,{54,63}} } },
   {4, {{3,{11,24,43}}, {3,{14,46,63}}, {4,{16,23,41,54}}, {2,{64,53}} } },
   {4, {{3,{11,24,43}}, {3,{14,45,63}}, {4,{15,23,41,54}}, {2,{64,53}} } },
   {4, {{3,{11,24,43}}, {3,{14,45,53}}, {3,{15,21,54}}, {3,{23,41,55}} } },
   {4, {{3,{11,24,43}}, {3,{14,46,53}}, {3,{16,21,54}}, {3,{23,41,56}} } },
   {4, {{3,{11,24,43}}, {3,{14,45,63}}, {3,{15,21,64}}, {3,{23,41,65}} } },
   {4, {{3,{11,24,43}}, {3,{14,46,63}}, {3,{16,21,64}}, {3,{23,41,66}} } },
   {5, {{3,{11,24,43}}, {2,{14,65}}, {3,{15,23,41}}, {2,{25,66}}, {2,{26,64}} } },
   {5, {{3,{11,24,43}}, {2,{14,66}}, {3,{16,23,41}}, {2,{26,65}}, {2,{25,64}} } },
   {5, {{3,{11,24,43}}, {2,{14,55}}, {3,{15,23,41}}, {2,{25,56}}, {2,{26,54}} } },
   {5, {{3,{11,24,43}}, {2,{14,56}}, {3,{16,23,41}}, {2,{26,55}}, {2,{25,54}} } },
   {4, {{4,{11,24,43,55}}, {3,{15,31,44}}, {3,{23,34,41}}, {2,{45,51}} } },
   {4, {{4,{11,24,43,56}}, {3,{16,31,44}}, {3,{23,34,41}}, {2,{46,51}} } },
   {4, {{4,{11,24,43,65}}, {3,{15,31,44}}, {3,{23,34,41}}, {2,{45,61}} } },
   {4, {{4,{11,24,43,66}}, {3,{16,31,44}}, {3,{23,34,41}}, {2,{46,61}} } },
   {4, {{3,{11,24,63}}, {4,{14,23,41,66}}, {3,{26,44,51}}, {2,{56,61}} } },
   {4, {{3,{11,24,63}}, {4,{14,23,41,65}}, {3,{25,44,51}}, {2,{55,61}} } },
   {4, {{3,{11,24,53}}, {4,{14,23,41,56}}, {3,{26,44,61}}, {2,{66,51}} } },
   {4, {{3,{11,24,53}}, {4,{14,23,41,55}}, {3,{25,44,61}}, {2,{65,51}} } },
   {4, {{4,{11,24,46,62}}, {2,{14,31}}, {3,{22,43,66}}, {3,{23,34,41}} } },
   {4, {{4,{11,24,45,62}}, {2,{14,31}}, {3,{22,43,65}}, {3,{23,34,41}} } },
   {4, {{4,{11,24,46,52}}, {2,{14,31}}, {3,{22,43,56}}, {3,{23,34,41}} } },
   {4, {{4,{11,24,45,52}}, {2,{14,31}}, {3,{22,43,55}}, {3,{23,34,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,22,61}}, {3,{23,36,41}}, {3,{26,31,62}} } },
   {4, {{3,{11,24,43}}, {3,{14,22,61}}, {3,{23,35,41}}, {3,{25,31,62}} } },
   {4, {{3,{11,24,43}}, {3,{14,22,51}}, {3,{23,36,41}}, {3,{26,31,52}} } },
   {4, {{3,{11,24,43}}, {3,{14,22,51}}, {3,{23,35,41}}, {3,{25,31,52}} } },
   {4, {{3,{11,24,53}}, {3,{14,46,51}}, {4,{16,23,41,64}}, {2,{54,63}} } },
   {4, {{3,{11,24,53}}, {3,{14,45,51}}, {4,{15,23,41,64}}, {2,{54,63}} } },
   {4, {{3,{11,24,63}}, {3,{14,46,61}}, {4,{16,23,41,54}}, {2,{64,53}} } },
   {4, {{3,{11,24,63}}, {3,{14,45,61}}, {4,{15,23,41,54}}, {2,{64,53}} } },
   {5, {{2,{11,24}}, {2,{13,26}}, {3,{14,31,43}}, {2,{16,21}}, {3,{23,34,41}} } },
   {5, {{2,{11,24}}, {2,{13,25}}, {3,{14,31,43}}, {2,{15,21}}, {3,{23,34,41}} } },
   {4, {{4,{11,24,46,62}}, {4,{14,23,41,66}}, {2,{21,63}}, {2,{22,61}} } },
   {4, {{4,{11,24,45,62}}, {4,{14,23,41,65}}, {2,{21,63}}, {2,{22,61}} } },
   {4, {{4,{11,24,46,52}}, {4,{14,23,41,56}}, {2,{21,53}}, {2,{22,51}} } },
   {4, {{4,{11,24,45,52}}, {4,{14,23,41,55}}, {2,{21,53}}, {2,{22,51}} } },
   {4, {{4,{11,24,43,66}}, {4,{16,23,41,54}}, {2,{44,53}}, {2,{46,61}} } },
   {4, {{4,{11,24,43,65}}, {4,{15,23,41,54}}, {2,{44,53}}, {2,{45,61}} } },
   {4, {{4,{11,24,43,56}}, {4,{16,23,41,64}}, {2,{44,63}}, {2,{46,51}} } },
   {4, {{4,{11,24,43,55}}, {4,{15,23,41,64}}, {2,{44,63}}, {2,{45,51}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{25,43,56}}, {3,{26,45,51}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{26,43,55}}, {3,{25,46,51}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{25,43,66}}, {3,{26,45,61}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{26,43,65}}, {3,{25,46,61}} } },
   {4, {{3,{11,24,63}}, {3,{14,46,61}}, {3,{16,23,41}}, {3,{26,43,64}} } },
   {4, {{3,{11,24,63}}, {3,{14,45,61}}, {3,{15,23,41}}, {3,{25,43,64}} } },
   {4, {{3,{11,24,53}}, {3,{14,46,51}}, {3,{16,23,41}}, {3,{26,43,54}} } },
   {4, {{3,{11,24,53}}, {3,{14,45,51}}, {3,{15,23,41}}, {3,{25,43,54}} } },
   {3, {{4,{11,24,46,62}}, {4,{14,22,53,66}}, {4,{16,23,41,54}} } },
   {3, {{4,{11,24,45,62}}, {4,{14,22,53,65}}, {4,{15,23,41,54}} } },
   {3, {{4,{11,24,46,52}}, {4,{14,22,63,56}}, {4,{16,23,41,64}} } },
   {3, {{4,{11,24,45,52}}, {4,{14,22,63,55}}, {4,{15,23,41,64}} } },
   },
   {
   {4, {{2,{11,24}}, {2,{12,41}}, {2,{14,22}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {2,{13,41}}, {3,{14,32,43}}, {3,{21,34,42}} } },
   {5, {{2,{11,24}}, {2,{14,31}}, {2,{21,42}}, {2,{22,34}}, {2,{32,41}} } },
   {4, {{3,{11,24,56}}, {2,{12,41}}, {3,{16,22,54}}, {2,{21,42}} } },
   {4, {{3,{11,24,55}}, {2,{12,41}}, {3,{15,22,54}}, {2,{21,42}} } },
   {4, {{3,{11,24,66}}, {2,{12,41}}, {3,{16,22,64}}, {2,{21,42}} } },
   {4, {{3,{11,24,65}}, {2,{12,41}}, {3,{15,22,64}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {3,{12,44,51}}, {2,{14,22}}, {3,{21,42,54}} } },
   {4, {{2,{11,24}}, {3,{12,44,61}}, {2,{14,22}}, {3,{21,42,64}} } },
   {5, {{2,{11,24}}, {2,{13,41}}, {2,{14,33}}, {3,{21,34,42}}, {2,{32,43}} } },
   {4, {{3,{11,24,56}}, {4,{15,21,42,54}}, {2,{16,45}}, {2,{46,52}} } },
   {4, {{3,{11,24,55}}, {4,{16,21,42,54}}, {2,{15,46}}, {2,{45,52}} } },
   {4, {{3,{11,24,66}}, {4,{15,21,42,64}}, {2,{16,45}}, {2,{46,62}} } },
   {4, {{3,{11,24,65}}, {4,{16,21,42,64}}, {2,{15,46}}, {2,{45,62}} } },
   {4, {{2,{11,24}}, {3,{14,45,52}}, {2,{15,31}}, {4,{21,35,42,54}} } },
   {4, {{2,{11,24}}, {3,{14,46,52}}, {2,{16,31}}, {4,{21,36,42,54}} } },
   {4, {{2,{11,24}}, {3,{14,45,62}}, {2,{15,31}}, {4,{21,35,42,64}} } },
   {4, {{2,{11,24}}, {3,{14,46,62}}, {2,{16,31}}, {4,{21,36,42,64}} } },
   {4, {{3,{11,24,45}}, {2,{12,41}}, {4,{15,21,42,54}}, {2,{44,51}} } },
   {4, {{3,{11,24,46}}, {2,{12,41}}, {4,{16,21,42,54}}, {2,{44,51}} } },
   {4, {{3,{11,24,45}}, {2,{12,41}}, {4,{15,21,42,64}}, {2,{44,61}} } },
   {4, {{3,{11,24,46}}, {2,{12,41}}, {4,{16,21,42,64}}, {2,{44,61}} } },
   {5, {{2,{11,24}}, {2,{14,46}}, {3,{16,32,41}}, {2,{21,42}}, {3,{22,36,44}} } },
   {5, {{2,{11,24}}, {2,{14,45}}, {3,{15,32,41}}, {2,{21,42}}, {3,{22,35,44}} } },
   {4, {{2,{11,24}}, {4,{14,31,46,52}}, {4,{21,36,42,64}}, {2,{54,62}} } },
   {4, {{2,{11,24}}, {4,{14,31,45,52}}, {4,{21,35,42,64}}, {2,{54,62}} } },
   {4, {{2,{11,24}}, {4,{14,31,46,62}}, {4,{21,36,42,54}}, {2,{64,52}} } },
   {4, {{2,{11,24}}, {4,{14,31,45,62}}, {4,{21,35,42,54}}, {2,{64,52}} } },
   {4, {{3,{11,24,45}}, {4,{15,21,42,66}}, {3,{26,54,62}}, {2,{44,52}} } },
   {4, {{3,{11,24,46}}, {4,{16,21,42,65}}, {3,{25,54,62}}, {2,{44,52}} } },
   {4, {{3,{11,24,45}}, {4,{15,21,42,56}}, {3,{26,64,52}}, {2,{44,62}} } },
   {4, {{3,{11,24,46}}, {4,{16,21,42,55}}, {3,{25,64,52}}, {2,{44,62}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {3,{14,33,52}}, {2,{21,42}}, {3,{22,34,53}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {3,{14,33,62}}, {2,{21,42}}, {3,{22,34,63}} } },
   {5, {{3,{11,24,45}}, {2,{12,41}}, {3,{15,32,44}}, {2,{21,42}}, {2,{22,34}} } },
   {5, {{3,{11,24,46}}, {2,{12,41}}, {3,{16,32,44}}, {2,{21,42}}, {2,{22,34}} } },
   {6, {{2,{11,24}}, {2,{14,31}}, {2,{21,42}}, {2,{22,34}}, {2,{32,43}}, {2,{33,41}} } },
   {5, {{3,{11,24,66}}, {2,{12,41}}, {2,{16,32}}, {3,{21,34,42}}, {2,{36,64}} } },
   {5, {{3,{11,24,65}}, {2,{12,41}}, {2,{15,32}}, {3,{21,34,42}}, {2,{35,64}} } },
   {5, {{3,{11,24,56}}, {2,{12,41}}, {2,{16,32}}, {3,{21,34,42}}, {2,{36,54}} } },
   {5, {{3,{11,24,55}}, {2,{12,41}}, {2,{15,32}}, {3,{21,34,42}}, {2,{35,54}} } },
   {3, {{4,{11,24,55,62}}, {4,{15,31,46,52}}, {4,{21,36,42,64}} } },
   {3, {{4,{11,24,56,62}}, {4,{16,31,45,52}}, {4,{21,35,42,64}} } },
   {3, {{4,{11,24,65,52}}, {4,{15,31,46,62}}, {4,{21,36,42,54}} } },
   {3, {{4,{11,24,66,52}}, {4,{16,31,45,62}}, {4,{21,35,42,54}} } },
   {4, {{2,{11,24}}, {4,{14,31,45,52}}, {4,{21,36,42,54}}, {2,{35,46}} } },
   {4, {{2,{11,24}}, {4,{14,31,46,52}}, {4,{21,35,42,54}}, {2,{36,45}} } },
   {4, {{2,{11,24}}, {4,{14,31,45,62}}, {4,{21,36,42,64}}, {2,{35,46}} } },
   {4, {{2,{11,24}}, {4,{14,31,46,62}}, {4,{21,35,42,64}}, {2,{36,45}} } },
   {4, {{2,{11,24}}, {2,{13,51}}, {4,{14,32,45,53}}, {4,{21,34,42,55}} } },
   {4, {{2,{11,24}}, {2,{13,51}}, {4,{14,32,46,53}}, {4,{21,34,42,56}} } },
   {4, {{2,{11,24}}, {2,{13,61}}, {4,{14,32,45,63}}, {4,{21,34,42,65}} } },
   {4, {{2,{11,24}}, {2,{13,61}}, {4,{14,32,46,63}}, {4,{21,34,42,66}} } },
   {3, {{4,{11,24,45,62}}, {4,{15,32,51,64}}, {4,{21,34,42,55}} } },
   {3, {{4,{11,24,46,62}}, {4,{16,32,51,64}}, {4,{21,34,42,56}} } },
   {3, {{4,{11,24,45,52}}, {4,{15,32,61,54}}, {4,{21,34,42,65}} } },
   {3, {{4,{11,24,46,52}}, {4,{16,32,61,54}}, {4,{21,34,42,66}} } },
   {6, {{2,{11,24}}, {2,{13,41}}, {2,{14,33}}, {2,{21,42}}, {2,{22,34}}, {2,{32,43}} } },
   {6, {{2,{11,24}}, {2,{12,51}}, {2,{14,22}}, {2,{21,42}}, {2,{41,54}}, {2,{44,52}} } },
   {6, {{2,{11,24}}, {2,{12,61}}, {2,{14,22}}, {2,{21,42}}, {2,{41,64}}, {2,{44,62}} } },
   {5, {{2,{11,24}}, {3,{14,31,56}}, {2,{21,42}}, {3,{22,36,54}}, {2,{32,41}} } },
   {5, {{2,{11,24}}, {3,{14,31,55}}, {2,{21,42}}, {3,{22,35,54}}, {2,{32,41}} } },
   {5, {{2,{11,24}}, {3,{14,31,66}}, {2,{21,42}}, {3,{22,36,64}}, {2,{32,41}} } },
   {5, {{2,{11,24}}, {3,{14,31,65}}, {2,{21,42}}, {3,{22,35,64}}, {2,{32,41}} } },
   {6, {{2,{11,24}}, {2,{12,35}}, {2,{14,22}}, {2,{15,31}}, {2,{21,42}}, {2,{32,41}} } },
   {6, {{2,{11,24}}, {2,{12,36}}, {2,{14,22}}, {2,{16,31}}, {2,{21,42}}, {2,{32,41}} } },
   {4, {{3,{11,24,66}}, {4,{15,21,42,64}}, {3,{16,55,62}}, {2,{45,52}} } },
   {4, {{3,{11,24,65}}, {4,{16,21,42,64}}, {3,{15,56,62}}, {2,{46,52}} } },
   {4, {{3,{11,24,56}}, {4,{15,21,42,54}}, {3,{16,65,52}}, {2,{45,62}} } },
   {4, {{3,{11,24,55}}, {4,{16,21,42,54}}, {3,{15,66,52}}, {2,{46,62}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,65}}, {2,{15,22}}, {2,{21,42}}, {2,{25,64}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,66}}, {2,{16,22}}, {2,{21,42}}, {2,{26,64}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,55}}, {2,{15,22}}, {2,{21,42}}, {2,{25,54}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,56}}, {2,{16,22}}, {2,{21,42}}, {2,{26,54}} } },
   {5, {{3,{11,24,56}}, {2,{12,41}}, {3,{15,22,54}}, {2,{16,55}}, {2,{21,42}} } },
   {5, {{3,{11,24,55}}, {2,{12,41}}, {3,{16,22,54}}, {2,{15,56}}, {2,{21,42}} } },
   {5, {{3,{11,24,66}}, {2,{12,41}}, {3,{15,22,64}}, {2,{16,65}}, {2,{21,42}} } },
   {5, {{3,{11,24,65}}, {2,{12,41}}, {3,{16,22,64}}, {2,{15,66}}, {2,{21,42}} } },
   {5, {{2,{11,24}}, {2,{13,61}}, {4,{14,22,46,63}}, {2,{21,42}}, {2,{41,66}} } },
   {5, {{2,{11,24}}, {2,{13,61}}, {4,{14,22,45,63}}, {2,{21,42}}, {2,{41,65}} } },
   {5, {{2,{11,24}}, {2,{13,51}}, {4,{14,22,46,53}}, {2,{21,42}}, {2,{41,56}} } },
   {5, {{2,{11,24}}, {2,{13,51}}, {4,{14,22,45,53}}, {2,{21,42}}, {2,{41,55}} } },
   {4, {{2,{11,24}}, {3,{14,52,66}}, {4,{16,21,42,54}}, {3,{26,44,62}} } },
   {4, {{2,{11,24}}, {3,{14,52,65}}, {4,{15,21,42,54}}, {3,{25,44,62}} } },
   {4, {{2,{11,24}}, {3,{14,62,56}}, {4,{16,21,42,64}}, {3,{26,44,52}} } },
   {4, {{2,{11,24}}, {3,{14,62,55}}, {4,{15,21,42,64}}, {3,{25,44,52}} } },
   {4, {{3,{11,24,46}}, {3,{12,44,61}}, {3,{16,22,64}}, {3,{21,42,66}} } },
   {4, {{3,{11,24,45}}, {3,{12,44,61}}, {3,{15,22,64}}, {3,{21,42,65}} } },
   {4, {{3,{11,24,46}}, {3,{12,44,51}}, {3,{16,22,54}}, {3,{21,42,56}} } },
   {4, {{3,{11,24,45}}, {3,{12,44,51}}, {3,{15,22,54}}, {3,{21,42,55}} } },
   {4, {{2,{11,24}}, {3,{14,33,61}}, {3,{21,42,65}}, {4,{22,34,45,63}} } },
   {4, {{2,{11,24}}, {3,{14,33,61}}, {3,{21,42,66}}, {4,{22,34,46,63}} } },
   {4, {{2,{11,24}}, {3,{14,33,51}}, {3,{21,42,55}}, {4,{22,34,45,53}} } },
   {4, {{2,{11,24}}, {3,{14,33,51}}, {3,{21,42,56}}, {4,{22,34,46,53}} } },
   {4, {{4,{11,24,46,53}}, {4,{16,21,42,54}}, {2,{32,43}}, {2,{33,52}} } },
   {4, {{4,{11,24,45,53}}, {4,{15,21,42,54}}, {2,{32,43}}, {2,{33,52}} } },
   {4, {{4,{11,24,46,63}}, {4,{16,21,42,64}}, {2,{32,43}}, {2,{33,62}} } },
   {4, {{4,{11,24,45,63}}, {4,{15,21,42,64}}, {2,{32,43}}, {2,{33,62}} } },
   {4, {{4,{11,24,43,56}}, {2,{12,41}}, {4,{16,22,44,53}}, {2,{21,42}} } },
   {4, {{4,{11,24,43,55}}, {2,{12,41}}, {4,{15,22,44,53}}, {2,{21,42}} } },
   {4, {{4,{11,24,43,66}}, {2,{12,41}}, {4,{16,22,44,63}}, {2,{21,42}} } },
   {4, {{4,{11,24,43,65}}, {2,{12,41}}, {4,{15,22,44,63}}, {2,{21,42}} } },
   {5, {{3,{11,24,56}}, {2,{16,31}}, {2,{21,42}}, {3,{22,36,54}}, {2,{32,41}} } },
   {5, {{3,{11,24,55}}, {2,{15,31}}, {2,{21,42}}, {3,{22,35,54}}, {2,{32,41}} } },
   {5, {{3,{11,24,66}}, {2,{16,31}}, {2,{21,42}}, {3,{22,36,64}}, {2,{32,41}} } },
   {5, {{3,{11,24,65}}, {2,{15,31}}, {2,{21,42}}, {3,{22,35,64}}, {2,{32,41}} } },
   {5, {{2,{11,24}}, {3,{12,44,61}}, {2,{14,22}}, {3,{21,42,54}}, {2,{51,64}} } },
   {5, {{2,{11,24}}, {3,{12,44,51}}, {2,{14,22}}, {3,{21,42,64}}, {2,{61,54}} } },
   {5, {{3,{11,24,65}}, {3,{15,32,41}}, {2,{21,42}}, {2,{22,34}}, {2,{35,64}} } },
   {5, {{3,{11,24,66}}, {3,{16,32,41}}, {2,{21,42}}, {2,{22,34}}, {2,{36,64}} } },
   {5, {{3,{11,24,55}}, {3,{15,32,41}}, {2,{21,42}}, {2,{22,34}}, {2,{35,54}} } },
   {5, {{3,{11,24,56}}, {3,{16,32,41}}, {2,{21,42}}, {2,{22,34}}, {2,{36,54}} } },
   {4, {{3,{11,24,56}}, {3,{12,45,61}}, {3,{16,22,54}}, {3,{21,42,65}} } },
   {4, {{3,{11,24,55}}, {3,{12,46,61}}, {3,{15,22,54}}, {3,{21,42,66}} } },
   {4, {{3,{11,24,66}}, {3,{12,45,51}}, {3,{16,22,64}}, {3,{21,42,55}} } },
   {4, {{3,{11,24,65}}, {3,{12,46,51}}, {3,{15,22,64}}, {3,{21,42,56}} } },
   },
   {
   {2, {{3,{11,24,43}}, {3,{14,23,41}}}},
   {3, {{3,{11,24,42}}, {3,{14,23,41}}, {2,{22,43}} } },
   {2, {{4,{11,24,46,63}}, {4,{16,23,41,64}} } },
   {2, {{4,{11,24,45,63}}, {4,{15,23,41,64}} } },
   {2, {{4,{11,24,46,53}}, {4,{16,23,41,54}} } },
   {2, {{4,{11,24,45,53}}, {4,{15,23,41,54}} } },
   {2, {{4,{11,24,45,63}}, {4,{14,23,41,65}} } },
   {2, {{4,{11,24,46,63}}, {4,{14,23,41,66}} } },
   {2, {{4,{11,24,45,53}}, {4,{14,23,41,55}} } },
   {2, {{4,{11,24,46,53}}, {4,{14,23,41,56}} } },
   {3, {{4,{11,24,45,53}}, {4,{14,23,41,65}}, {2,{55,63}} } },
   {3, {{4,{11,24,46,53}}, {4,{14,23,41,66}}, {2,{56,63}} } },
   {3, {{4,{11,24,45,63}}, {4,{14,23,41,55}}, {2,{65,53}} } },
   {3, {{4,{11,24,46,63}}, {4,{14,23,41,56}}, {2,{66,53}} } },
   {3, {{4,{11,24,43,56}}, {3,{16,31,54}}, {3,{23,34,41}} } },
   {3, {{4,{11,24,43,55}}, {3,{15,31,54}}, {3,{23,34,41}} } },
   {3, {{4,{11,24,43,66}}, {3,{16,31,64}}, {3,{23,34,41}} } },
   {3, {{4,{11,24,43,65}}, {3,{15,31,64}}, {3,{23,34,41}} } },
   {3, {{3,{11,24,63}}, {4,{14,23,41,66}}, {3,{26,44,61}} } },
   {3, {{3,{11,24,63}}, {4,{14,23,41,65}}, {3,{25,44,61}} } },
   {3, {{3,{11,24,53}}, {4,{14,23,41,56}}, {3,{26,44,51}} } },
   {3, {{3,{11,24,53}}, {4,{14,23,41,55}}, {3,{25,44,51}} } },
   {3, {{4,{11,24,55,63}}, {4,{15,23,41,64}}, {2,{45,51}} } },
   {3, {{4,{11,24,56,63}}, {4,{16,23,41,64}}, {2,{46,51}} } },
   {3, {{4,{11,24,65,53}}, {4,{15,23,41,54}}, {2,{45,61}} } },
   {3, {{4,{11,24,66,53}}, {4,{16,23,41,54}}, {2,{46,61}} } },
   {3, {{4,{11,24,46,63}}, {4,{14,23,41,56}}, {3,{16,53,64}} } },
   {3, {{4,{11,24,45,63}}, {4,{14,23,41,55}}, {3,{15,53,64}} } },
   {3, {{4,{11,24,46,53}}, {4,{14,23,41,66}}, {3,{16,63,54}} } },
   {3, {{4,{11,24,45,53}}, {4,{14,23,41,65}}, {3,{15,63,54}} } },
   {3, {{4,{11,24,43,56}}, {3,{16,31,55}}, {4,{23,35,41,54}} } },
   {3, {{4,{11,24,43,55}}, {3,{15,31,56}}, {4,{23,36,41,54}} } },
   {3, {{4,{11,24,43,66}}, {3,{16,31,65}}, {4,{23,35,41,64}} } },
   {3, {{4,{11,24,43,65}}, {3,{15,31,66}}, {4,{23,36,41,64}} } },
   {5, {{3,{11,24,43}}, {2,{12,36}}, {2,{14,22}}, {3,{16,23,41}}, {2,{26,32}} } },
   {5, {{3,{11,24,43}}, {2,{12,35}}, {2,{14,22}}, {3,{15,23,41}}, {2,{25,32}} } },
   {5, {{3,{11,24,43}}, {2,{12,25}}, {2,{14,22}}, {2,{15,31}}, {3,{23,35,41}} } },
   {5, {{3,{11,24,43}}, {2,{12,26}}, {2,{14,22}}, {2,{16,31}}, {3,{23,36,41}} } },
   {5, {{3,{11,24,43}}, {2,{14,31}}, {2,{22,34}}, {3,{23,35,41}}, {2,{25,32}} } },
   {5, {{3,{11,24,43}}, {2,{14,31}}, {2,{22,34}}, {3,{23,36,41}}, {2,{26,32}} } },
   {5, {{4,{11,24,43,65}}, {2,{15,21}}, {2,{23,41}}, {2,{25,44}}, {2,{45,61}} } },
   {5, {{4,{11,24,43,66}}, {2,{16,21}}, {2,{23,41}}, {2,{26,44}}, {2,{46,61}} } },
   {5, {{4,{11,24,43,55}}, {2,{15,21}}, {2,{23,41}}, {2,{25,44}}, {2,{45,51}} } },
   {5, {{4,{11,24,43,56}}, {2,{16,21}}, {2,{23,41}}, {2,{26,44}}, {2,{46,51}} } },
   {5, {{3,{11,24,42}}, {3,{14,23,41}}, {2,{22,53}}, {2,{43,55}}, {2,{45,52}} } },
   {5, {{3,{11,24,42}}, {3,{14,23,41}}, {2,{22,53}}, {2,{43,56}}, {2,{46,52}} } },
   {5, {{3,{11,24,42}}, {3,{14,23,41}}, {2,{22,63}}, {2,{43,65}}, {2,{45,62}} } },
   {5, {{3,{11,24,42}}, {3,{14,23,41}}, {2,{22,63}}, {2,{43,66}}, {2,{46,62}} } },
   {4, {{4,{11,24,53,66}}, {2,{16,31}}, {4,{23,36,41,54}}, {2,{46,61}} } },
   {4, {{4,{11,24,53,65}}, {2,{15,31}}, {4,{23,35,41,54}}, {2,{45,61}} } },
   {4, {{4,{11,24,63,56}}, {2,{16,31}}, {4,{23,36,41,64}}, {2,{46,51}} } },
   {4, {{4,{11,24,63,55}}, {2,{15,31}}, {4,{23,35,41,64}}, {2,{45,51}} } },
   {4, {{4,{11,24,45,63}}, {2,{15,31}}, {4,{23,35,41,54}}, {2,{53,64}} } },
   {4, {{4,{11,24,46,63}}, {2,{16,31}}, {4,{23,36,41,54}}, {2,{53,64}} } },
   {4, {{4,{11,24,45,53}}, {2,{15,31}}, {4,{23,35,41,64}}, {2,{63,54}} } },
   {4, {{4,{11,24,46,53}}, {2,{16,31}}, {4,{23,36,41,64}}, {2,{63,54}} } },
   {4, {{4,{11,24,43,66}}, {3,{16,31,65}}, {3,{23,35,41}}, {2,{25,64}} } },
   {4, {{4,{11,24,43,65}}, {3,{15,31,66}}, {3,{23,36,41}}, {2,{26,64}} } },
   {4, {{4,{11,24,43,56}}, {3,{16,31,55}}, {3,{23,35,41}}, {2,{25,54}} } },
   {4, {{4,{11,24,43,55}}, {3,{15,31,56}}, {3,{23,36,41}}, {2,{26,54}} } },
   {4, {{3,{11,24,43}}, {3,{14,46,63}}, {4,{15,23,41,64}}, {2,{16,45}} } },
   {4, {{3,{11,24,43}}, {3,{14,45,63}}, {4,{16,23,41,64}}, {2,{15,46}} } },
   {4, {{3,{11,24,43}}, {3,{14,46,53}}, {4,{15,23,41,54}}, {2,{16,45}} } },
   {4, {{3,{11,24,43}}, {3,{14,45,53}}, {4,{16,23,41,54}}, {2,{15,46}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,31,53,64}}, {4,{23,34,41,56}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,31,53,64}}, {4,{23,34,41,55}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,31,63,54}}, {4,{23,34,41,66}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,31,63,54}}, {4,{23,34,41,65}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,33,51,64}}, {4,{23,34,41,55}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,33,51,64}}, {4,{23,34,41,56}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,33,61,54}}, {4,{23,34,41,65}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,33,61,54}}, {4,{23,34,41,66}} } },
   {4, {{3,{11,24,43}}, {2,{14,45}}, {4,{15,23,41,66}}, {3,{26,44,63}} } },
   {4, {{3,{11,24,43}}, {2,{14,46}}, {4,{16,23,41,65}}, {3,{25,44,63}} } },
   {4, {{3,{11,24,43}}, {2,{14,45}}, {4,{15,23,41,56}}, {3,{26,44,53}} } },
   {4, {{3,{11,24,43}}, {2,{14,46}}, {4,{16,23,41,55}}, {3,{25,44,53}} } },
   {4, {{4,{11,24,45,63}}, {4,{15,23,41,54}}, {2,{43,64}}, {2,{44,53}} } },
   {4, {{4,{11,24,46,63}}, {4,{16,23,41,54}}, {2,{43,64}}, {2,{44,53}} } },
   {4, {{4,{11,24,45,53}}, {4,{15,23,41,64}}, {2,{43,54}}, {2,{44,63}} } },
   {4, {{4,{11,24,46,53}}, {4,{16,23,41,64}}, {2,{43,54}}, {2,{44,63}} } },
   {4, {{3,{11,24,43}}, {3,{14,46,53}}, {4,{16,23,41,64}}, {2,{54,63}} } },
   {4, {{3,{11,24,43}}, {3,{14,45,53}}, {4,{15,23,41,64}}, {2,{54,63}} } },
   {4, {{3,{11,24,43}}, {3,{14,46,63}}, {4,{16,23,41,54}}, {2,{64,53}} } },
   {4, {{3,{11,24,43}}, {3,{14,45,63}}, {4,{15,23,41,54}}, {2,{64,53}} } },
   {4, {{3,{11,24,43}}, {3,{14,45,53}}, {3,{15,21,54}}, {3,{23,41,55}} } },
   {4, {{3,{11,24,43}}, {3,{14,46,53}}, {3,{16,21,54}}, {3,{23,41,56}} } },
   {4, {{3,{11,24,43}}, {3,{14,45,63}}, {3,{15,21,64}}, {3,{23,41,65}} } },
   {4, {{3,{11,24,43}}, {3,{14,46,63}}, {3,{16,21,64}}, {3,{23,41,66}} } },
   {5, {{3,{11,24,43}}, {2,{14,65}}, {3,{15,23,41}}, {2,{25,66}}, {2,{26,64}} } },
   {5, {{3,{11,24,43}}, {2,{14,66}}, {3,{16,23,41}}, {2,{26,65}}, {2,{25,64}} } },
   {5, {{3,{11,24,43}}, {2,{14,55}}, {3,{15,23,41}}, {2,{25,56}}, {2,{26,54}} } },
   {5, {{3,{11,24,43}}, {2,{14,56}}, {3,{16,23,41}}, {2,{26,55}}, {2,{25,54}} } },
   {4, {{4,{11,24,43,55}}, {3,{15,31,44}}, {3,{23,34,41}}, {2,{45,51}} } },
   {4, {{4,{11,24,43,56}}, {3,{16,31,44}}, {3,{23,34,41}}, {2,{46,51}} } },
   {4, {{4,{11,24,43,65}}, {3,{15,31,44}}, {3,{23,34,41}}, {2,{45,61}} } },
   {4, {{4,{11,24,43,66}}, {3,{16,31,44}}, {3,{23,34,41}}, {2,{46,61}} } },
   {4, {{3,{11,24,63}}, {4,{14,23,41,66}}, {3,{26,44,51}}, {2,{56,61}} } },
   {4, {{3,{11,24,63}}, {4,{14,23,41,65}}, {3,{25,44,51}}, {2,{55,61}} } },
   {4, {{3,{11,24,53}}, {4,{14,23,41,56}}, {3,{26,44,61}}, {2,{66,51}} } },
   {4, {{3,{11,24,53}}, {4,{14,23,41,55}}, {3,{25,44,61}}, {2,{65,51}} } },
   {4, {{4,{11,24,46,62}}, {2,{14,31}}, {3,{22,43,66}}, {3,{23,34,41}} } },
   {4, {{4,{11,24,45,62}}, {2,{14,31}}, {3,{22,43,65}}, {3,{23,34,41}} } },
   {4, {{4,{11,24,46,52}}, {2,{14,31}}, {3,{22,43,56}}, {3,{23,34,41}} } },
   {4, {{4,{11,24,45,52}}, {2,{14,31}}, {3,{22,43,55}}, {3,{23,34,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,22,61}}, {3,{23,36,41}}, {3,{26,31,62}} } },
   {4, {{3,{11,24,43}}, {3,{14,22,61}}, {3,{23,35,41}}, {3,{25,31,62}} } },
   {4, {{3,{11,24,43}}, {3,{14,22,51}}, {3,{23,36,41}}, {3,{26,31,52}} } },
   {4, {{3,{11,24,43}}, {3,{14,22,51}}, {3,{23,35,41}}, {3,{25,31,52}} } },
   {4, {{3,{11,24,53}}, {3,{14,46,51}}, {4,{16,23,41,64}}, {2,{54,63}} } },
   {4, {{3,{11,24,53}}, {3,{14,45,51}}, {4,{15,23,41,64}}, {2,{54,63}} } },
   {4, {{3,{11,24,63}}, {3,{14,46,61}}, {4,{16,23,41,54}}, {2,{64,53}} } },
   {4, {{3,{11,24,63}}, {3,{14,45,61}}, {4,{15,23,41,54}}, {2,{64,53}} } },
   {5, {{2,{11,24}}, {2,{13,26}}, {3,{14,31,43}}, {2,{16,21}}, {3,{23,34,41}} } },
   {5, {{2,{11,24}}, {2,{13,25}}, {3,{14,31,43}}, {2,{15,21}}, {3,{23,34,41}} } },
   {4, {{4,{11,24,46,62}}, {4,{14,23,41,66}}, {2,{21,63}}, {2,{22,61}} } },
   {4, {{4,{11,24,45,62}}, {4,{14,23,41,65}}, {2,{21,63}}, {2,{22,61}} } },
   {4, {{4,{11,24,46,52}}, {4,{14,23,41,56}}, {2,{21,53}}, {2,{22,51}} } },
   {4, {{4,{11,24,45,52}}, {4,{14,23,41,55}}, {2,{21,53}}, {2,{22,51}} } },
   {4, {{4,{11,24,43,66}}, {4,{16,23,41,54}}, {2,{44,53}}, {2,{46,61}} } },
   {4, {{4,{11,24,43,65}}, {4,{15,23,41,54}}, {2,{44,53}}, {2,{45,61}} } },
   {4, {{4,{11,24,43,56}}, {4,{16,23,41,64}}, {2,{44,63}}, {2,{46,51}} } },
   {4, {{4,{11,24,43,55}}, {4,{15,23,41,64}}, {2,{44,63}}, {2,{45,51}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{25,43,56}}, {3,{26,45,51}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{26,43,55}}, {3,{25,46,51}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{25,43,66}}, {3,{26,45,61}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{26,43,65}}, {3,{25,46,61}} } },
   {4, {{3,{11,24,63}}, {3,{14,46,61}}, {3,{16,23,41}}, {3,{26,43,64}} } },
   {4, {{3,{11,24,63}}, {3,{14,45,61}}, {3,{15,23,41}}, {3,{25,43,64}} } },
   {4, {{3,{11,24,53}}, {3,{14,46,51}}, {3,{16,23,41}}, {3,{26,43,54}} } },
   {4, {{3,{11,24,53}}, {3,{14,45,51}}, {3,{15,23,41}}, {3,{25,43,54}} } },
   {3, {{4,{11,24,46,62}}, {4,{14,22,53,66}}, {4,{16,23,41,54}} } },
   {3, {{4,{11,24,45,62}}, {4,{14,22,53,65}}, {4,{15,23,41,54}} } },
   {3, {{4,{11,24,46,52}}, {4,{14,22,63,56}}, {4,{16,23,41,64}} } },
   {3, {{4,{11,24,45,52}}, {4,{14,22,63,55}}, {4,{15,23,41,64}} } },
   {4, {{2,{11,24}}, {2,{12,41}}, {2,{14,22}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {2,{13,41}}, {3,{14,32,43}}, {3,{21,34,42}} } },
   {5, {{2,{11,24}}, {2,{14,31}}, {2,{21,42}}, {2,{22,34}}, {2,{32,41}} } },
   {4, {{3,{11,24,56}}, {2,{12,41}}, {3,{16,22,54}}, {2,{21,42}} } },
   {4, {{3,{11,24,55}}, {2,{12,41}}, {3,{15,22,54}}, {2,{21,42}} } },
   {4, {{3,{11,24,66}}, {2,{12,41}}, {3,{16,22,64}}, {2,{21,42}} } },
   {4, {{3,{11,24,65}}, {2,{12,41}}, {3,{15,22,64}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {3,{12,44,51}}, {2,{14,22}}, {3,{21,42,54}} } },
   {4, {{2,{11,24}}, {3,{12,44,61}}, {2,{14,22}}, {3,{21,42,64}} } },
   {5, {{2,{11,24}}, {2,{13,41}}, {2,{14,33}}, {3,{21,34,42}}, {2,{32,43}} } },
   {4, {{3,{11,24,56}}, {4,{15,21,42,54}}, {2,{16,45}}, {2,{46,52}} } },
   {4, {{3,{11,24,55}}, {4,{16,21,42,54}}, {2,{15,46}}, {2,{45,52}} } },
   {4, {{3,{11,24,66}}, {4,{15,21,42,64}}, {2,{16,45}}, {2,{46,62}} } },
   {4, {{3,{11,24,65}}, {4,{16,21,42,64}}, {2,{15,46}}, {2,{45,62}} } },
   {4, {{2,{11,24}}, {3,{14,45,52}}, {2,{15,31}}, {4,{21,35,42,54}} } },
   {4, {{2,{11,24}}, {3,{14,46,52}}, {2,{16,31}}, {4,{21,36,42,54}} } },
   {4, {{2,{11,24}}, {3,{14,45,62}}, {2,{15,31}}, {4,{21,35,42,64}} } },
   {4, {{2,{11,24}}, {3,{14,46,62}}, {2,{16,31}}, {4,{21,36,42,64}} } },
   {4, {{3,{11,24,45}}, {2,{12,41}}, {4,{15,21,42,54}}, {2,{44,51}} } },
   {4, {{3,{11,24,46}}, {2,{12,41}}, {4,{16,21,42,54}}, {2,{44,51}} } },
   {4, {{3,{11,24,45}}, {2,{12,41}}, {4,{15,21,42,64}}, {2,{44,61}} } },
   {4, {{3,{11,24,46}}, {2,{12,41}}, {4,{16,21,42,64}}, {2,{44,61}} } },
   {5, {{2,{11,24}}, {2,{14,46}}, {3,{16,32,41}}, {2,{21,42}}, {3,{22,36,44}} } },
   {5, {{2,{11,24}}, {2,{14,45}}, {3,{15,32,41}}, {2,{21,42}}, {3,{22,35,44}} } },
   {4, {{2,{11,24}}, {4,{14,31,46,52}}, {4,{21,36,42,64}}, {2,{54,62}} } },
   {4, {{2,{11,24}}, {4,{14,31,45,52}}, {4,{21,35,42,64}}, {2,{54,62}} } },
   {4, {{2,{11,24}}, {4,{14,31,46,62}}, {4,{21,36,42,54}}, {2,{64,52}} } },
   {4, {{2,{11,24}}, {4,{14,31,45,62}}, {4,{21,35,42,54}}, {2,{64,52}} } },
   {4, {{3,{11,24,45}}, {4,{15,21,42,66}}, {3,{26,54,62}}, {2,{44,52}} } },
   {4, {{3,{11,24,46}}, {4,{16,21,42,65}}, {3,{25,54,62}}, {2,{44,52}} } },
   {4, {{3,{11,24,45}}, {4,{15,21,42,56}}, {3,{26,64,52}}, {2,{44,62}} } },
   {4, {{3,{11,24,46}}, {4,{16,21,42,55}}, {3,{25,64,52}}, {2,{44,62}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {3,{14,33,52}}, {2,{21,42}}, {3,{22,34,53}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {3,{14,33,62}}, {2,{21,42}}, {3,{22,34,63}} } },
   {5, {{3,{11,24,45}}, {2,{12,41}}, {3,{15,32,44}}, {2,{21,42}}, {2,{22,34}} } },
   {5, {{3,{11,24,46}}, {2,{12,41}}, {3,{16,32,44}}, {2,{21,42}}, {2,{22,34}} } },
   {6, {{2,{11,24}}, {2,{14,31}}, {2,{21,42}}, {2,{22,34}}, {2,{32,43}}, {2,{33,41}} } },
   {5, {{3,{11,24,66}}, {2,{12,41}}, {2,{16,32}}, {3,{21,34,42}}, {2,{36,64}} } },
   {5, {{3,{11,24,65}}, {2,{12,41}}, {2,{15,32}}, {3,{21,34,42}}, {2,{35,64}} } },
   {5, {{3,{11,24,56}}, {2,{12,41}}, {2,{16,32}}, {3,{21,34,42}}, {2,{36,54}} } },
   {5, {{3,{11,24,55}}, {2,{12,41}}, {2,{15,32}}, {3,{21,34,42}}, {2,{35,54}} } },
   {3, {{4,{11,24,55,62}}, {4,{15,31,46,52}}, {4,{21,36,42,64}} } },
   {3, {{4,{11,24,56,62}}, {4,{16,31,45,52}}, {4,{21,35,42,64}} } },
   {3, {{4,{11,24,65,52}}, {4,{15,31,46,62}}, {4,{21,36,42,54}} } },
   {3, {{4,{11,24,66,52}}, {4,{16,31,45,62}}, {4,{21,35,42,54}} } },
   {4, {{2,{11,24}}, {4,{14,31,45,52}}, {4,{21,36,42,54}}, {2,{35,46}} } },
   {4, {{2,{11,24}}, {4,{14,31,46,52}}, {4,{21,35,42,54}}, {2,{36,45}} } },
   {4, {{2,{11,24}}, {4,{14,31,45,62}}, {4,{21,36,42,64}}, {2,{35,46}} } },
   {4, {{2,{11,24}}, {4,{14,31,46,62}}, {4,{21,35,42,64}}, {2,{36,45}} } },
   {4, {{2,{11,24}}, {2,{13,51}}, {4,{14,32,45,53}}, {4,{21,34,42,55}} } },
   {4, {{2,{11,24}}, {2,{13,51}}, {4,{14,32,46,53}}, {4,{21,34,42,56}} } },
   {4, {{2,{11,24}}, {2,{13,61}}, {4,{14,32,45,63}}, {4,{21,34,42,65}} } },
   {4, {{2,{11,24}}, {2,{13,61}}, {4,{14,32,46,63}}, {4,{21,34,42,66}} } },
   {3, {{4,{11,24,45,62}}, {4,{15,32,51,64}}, {4,{21,34,42,55}} } },
   {3, {{4,{11,24,46,62}}, {4,{16,32,51,64}}, {4,{21,34,42,56}} } },
   {3, {{4,{11,24,45,52}}, {4,{15,32,61,54}}, {4,{21,34,42,65}} } },
   {3, {{4,{11,24,46,52}}, {4,{16,32,61,54}}, {4,{21,34,42,66}} } },
   {6, {{2,{11,24}}, {2,{13,41}}, {2,{14,33}}, {2,{21,42}}, {2,{22,34}}, {2,{32,43}} } },
   {6, {{2,{11,24}}, {2,{12,51}}, {2,{14,22}}, {2,{21,42}}, {2,{41,54}}, {2,{44,52}} } },
   {6, {{2,{11,24}}, {2,{12,61}}, {2,{14,22}}, {2,{21,42}}, {2,{41,64}}, {2,{44,62}} } },
   {5, {{2,{11,24}}, {3,{14,31,56}}, {2,{21,42}}, {3,{22,36,54}}, {2,{32,41}} } },
   {5, {{2,{11,24}}, {3,{14,31,55}}, {2,{21,42}}, {3,{22,35,54}}, {2,{32,41}} } },
   {5, {{2,{11,24}}, {3,{14,31,66}}, {2,{21,42}}, {3,{22,36,64}}, {2,{32,41}} } },
   {5, {{2,{11,24}}, {3,{14,31,65}}, {2,{21,42}}, {3,{22,35,64}}, {2,{32,41}} } },
   {6, {{2,{11,24}}, {2,{12,35}}, {2,{14,22}}, {2,{15,31}}, {2,{21,42}}, {2,{32,41}} } },
   {6, {{2,{11,24}}, {2,{12,36}}, {2,{14,22}}, {2,{16,31}}, {2,{21,42}}, {2,{32,41}} } },
   {4, {{3,{11,24,66}}, {4,{15,21,42,64}}, {3,{16,55,62}}, {2,{45,52}} } },
   {4, {{3,{11,24,65}}, {4,{16,21,42,64}}, {3,{15,56,62}}, {2,{46,52}} } },
   {4, {{3,{11,24,56}}, {4,{15,21,42,54}}, {3,{16,65,52}}, {2,{45,62}} } },
   {4, {{3,{11,24,55}}, {4,{16,21,42,54}}, {3,{15,66,52}}, {2,{46,62}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,65}}, {2,{15,22}}, {2,{21,42}}, {2,{25,64}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,66}}, {2,{16,22}}, {2,{21,42}}, {2,{26,64}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,55}}, {2,{15,22}}, {2,{21,42}}, {2,{25,54}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,56}}, {2,{16,22}}, {2,{21,42}}, {2,{26,54}} } },
   {5, {{3,{11,24,56}}, {2,{12,41}}, {3,{15,22,54}}, {2,{16,55}}, {2,{21,42}} } },
   {5, {{3,{11,24,55}}, {2,{12,41}}, {3,{16,22,54}}, {2,{15,56}}, {2,{21,42}} } },
   {5, {{3,{11,24,66}}, {2,{12,41}}, {3,{15,22,64}}, {2,{16,65}}, {2,{21,42}} } },
   {5, {{3,{11,24,65}}, {2,{12,41}}, {3,{16,22,64}}, {2,{15,66}}, {2,{21,42}} } },
   {5, {{2,{11,24}}, {2,{13,61}}, {4,{14,22,46,63}}, {2,{21,42}}, {2,{41,66}} } },
   {5, {{2,{11,24}}, {2,{13,61}}, {4,{14,22,45,63}}, {2,{21,42}}, {2,{41,65}} } },
   {5, {{2,{11,24}}, {2,{13,51}}, {4,{14,22,46,53}}, {2,{21,42}}, {2,{41,56}} } },
   {5, {{2,{11,24}}, {2,{13,51}}, {4,{14,22,45,53}}, {2,{21,42}}, {2,{41,55}} } },
   {4, {{2,{11,24}}, {3,{14,52,66}}, {4,{16,21,42,54}}, {3,{26,44,62}} } },
   {4, {{2,{11,24}}, {3,{14,52,65}}, {4,{15,21,42,54}}, {3,{25,44,62}} } },
   {4, {{2,{11,24}}, {3,{14,62,56}}, {4,{16,21,42,64}}, {3,{26,44,52}} } },
   {4, {{2,{11,24}}, {3,{14,62,55}}, {4,{15,21,42,64}}, {3,{25,44,52}} } },
   {4, {{3,{11,24,46}}, {3,{12,44,61}}, {3,{16,22,64}}, {3,{21,42,66}} } },
   {4, {{3,{11,24,45}}, {3,{12,44,61}}, {3,{15,22,64}}, {3,{21,42,65}} } },
   {4, {{3,{11,24,46}}, {3,{12,44,51}}, {3,{16,22,54}}, {3,{21,42,56}} } },
   {4, {{3,{11,24,45}}, {3,{12,44,51}}, {3,{15,22,54}}, {3,{21,42,55}} } },
   {4, {{2,{11,24}}, {3,{14,33,61}}, {3,{21,42,65}}, {4,{22,34,45,63}} } },
   {4, {{2,{11,24}}, {3,{14,33,61}}, {3,{21,42,66}}, {4,{22,34,46,63}} } },
   {4, {{2,{11,24}}, {3,{14,33,51}}, {3,{21,42,55}}, {4,{22,34,45,53}} } },
   {4, {{2,{11,24}}, {3,{14,33,51}}, {3,{21,42,56}}, {4,{22,34,46,53}} } },
   {4, {{4,{11,24,46,53}}, {4,{16,21,42,54}}, {2,{32,43}}, {2,{33,52}} } },
   {4, {{4,{11,24,45,53}}, {4,{15,21,42,54}}, {2,{32,43}}, {2,{33,52}} } },
   {4, {{4,{11,24,46,63}}, {4,{16,21,42,64}}, {2,{32,43}}, {2,{33,62}} } },
   {4, {{4,{11,24,45,63}}, {4,{15,21,42,64}}, {2,{32,43}}, {2,{33,62}} } },
   {4, {{4,{11,24,43,56}}, {2,{12,41}}, {4,{16,22,44,53}}, {2,{21,42}} } },
   {4, {{4,{11,24,43,55}}, {2,{12,41}}, {4,{15,22,44,53}}, {2,{21,42}} } },
   {4, {{4,{11,24,43,66}}, {2,{12,41}}, {4,{16,22,44,63}}, {2,{21,42}} } },
   {4, {{4,{11,24,43,65}}, {2,{12,41}}, {4,{15,22,44,63}}, {2,{21,42}} } },
   {5, {{3,{11,24,56}}, {2,{16,31}}, {2,{21,42}}, {3,{22,36,54}}, {2,{32,41}} } },
   {5, {{3,{11,24,55}}, {2,{15,31}}, {2,{21,42}}, {3,{22,35,54}}, {2,{32,41}} } },
   {5, {{3,{11,24,66}}, {2,{16,31}}, {2,{21,42}}, {3,{22,36,64}}, {2,{32,41}} } },
   {5, {{3,{11,24,65}}, {2,{15,31}}, {2,{21,42}}, {3,{22,35,64}}, {2,{32,41}} } },
   {5, {{2,{11,24}}, {3,{12,44,61}}, {2,{14,22}}, {3,{21,42,54}}, {2,{51,64}} } },
   {5, {{2,{11,24}}, {3,{12,44,51}}, {2,{14,22}}, {3,{21,42,64}}, {2,{61,54}} } },
   {5, {{3,{11,24,65}}, {3,{15,32,41}}, {2,{21,42}}, {2,{22,34}}, {2,{35,64}} } },
   {5, {{3,{11,24,66}}, {3,{16,32,41}}, {2,{21,42}}, {2,{22,34}}, {2,{36,64}} } },
   {5, {{3,{11,24,55}}, {3,{15,32,41}}, {2,{21,42}}, {2,{22,34}}, {2,{35,54}} } },
   {5, {{3,{11,24,56}}, {3,{16,32,41}}, {2,{21,42}}, {2,{22,34}}, {2,{36,54}} } },
   {4, {{3,{11,24,56}}, {3,{12,45,61}}, {3,{16,22,54}}, {3,{21,42,65}} } },
   {4, {{3,{11,24,55}}, {3,{12,46,61}}, {3,{15,22,54}}, {3,{21,42,66}} } },
   {4, {{3,{11,24,66}}, {3,{12,45,51}}, {3,{16,22,64}}, {3,{21,42,55}} } },
   {4, {{3,{11,24,65}}, {3,{12,46,51}}, {3,{15,22,64}}, {3,{21,42,56}} } },
   },
   {
   {4, {{2,{11,24}}, {2,{12,41}}, {3,{14,23,42}}, {2,{21,43}} } },
   {3, {{3,{11,24,43}}, {3,{14,32,41}}, {3,{23,34,42}} } },
   {3, {{4,{11,24,46,63}}, {2,{12,41}}, {4,{14,23,42,66}} } },
   {3, {{4,{11,24,45,63}}, {2,{12,41}}, {4,{14,23,42,65}} } },
   {3, {{4,{11,24,46,53}}, {2,{12,41}}, {4,{14,23,42,56}} } },
   {3, {{4,{11,24,45,53}}, {2,{12,41}}, {4,{14,23,42,55}} } },
   {3, {{3,{11,24,53}}, {3,{12,46,51}}, {4,{14,23,42,56}} } },
   {3, {{3,{11,24,53}}, {3,{12,45,51}}, {4,{14,23,42,55}} } },
   {3, {{3,{11,24,63}}, {3,{12,46,61}}, {4,{14,23,42,66}} } },
   {3, {{3,{11,24,63}}, {3,{12,45,61}}, {4,{14,23,42,65}} } },
   {4, {{3,{11,24,43}}, {2,{14,31}}, {3,{22,34,41}}, {2,{23,42}} } },
   {3, {{4,{11,24,53,66}}, {4,{16,22,44,61}}, {3,{23,42,54}} } },
   {3, {{4,{11,24,53,65}}, {4,{15,22,44,61}}, {3,{23,42,54}} } },
   {3, {{4,{11,24,63,56}}, {4,{16,22,44,51}}, {3,{23,42,64}} } },
   {3, {{4,{11,24,63,55}}, {4,{15,22,44,51}}, {3,{23,42,64}} } },
   {5, {{2,{11,24}}, {3,{14,32,41}}, {2,{21,43}}, {2,{22,34}}, {2,{23,42}} } },
   {4, {{3,{11,24,63}}, {3,{12,44,61}}, {2,{14,22}}, {3,{23,42,64}} } },
   {4, {{3,{11,24,53}}, {3,{12,44,51}}, {2,{14,22}}, {3,{23,42,54}} } },
   {4, {{3,{11,24,53}}, {2,{12,41}}, {4,{14,23,42,56}}, {2,{46,51}} } },
   {4, {{3,{11,24,53}}, {2,{12,41}}, {4,{14,23,42,55}}, {2,{45,51}} } },
   {4, {{3,{11,24,63}}, {2,{12,41}}, {4,{14,23,42,66}}, {2,{46,61}} } },
   {4, {{3,{11,24,63}}, {2,{12,41}}, {4,{14,23,42,65}}, {2,{45,61}} } },
   {4, {{3,{11,24,63}}, {3,{12,45,61}}, {2,{14,22}}, {3,{23,42,65}} } },
   {4, {{3,{11,24,63}}, {3,{12,46,61}}, {2,{14,22}}, {3,{23,42,66}} } },
   {4, {{3,{11,24,53}}, {3,{12,45,51}}, {2,{14,22}}, {3,{23,42,55}} } },
   {4, {{3,{11,24,53}}, {3,{12,46,51}}, {2,{14,22}}, {3,{23,42,56}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,22,41,54}}, {3,{23,42,56}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,22,41,54}}, {3,{23,42,55}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,22,41,64}}, {3,{23,42,66}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,22,41,64}}, {3,{23,42,65}} } },
   {5, {{3,{11,24,46}}, {2,{12,41}}, {3,{16,22,44}}, {2,{21,43}}, {2,{23,42}} } },
   {5, {{3,{11,24,45}}, {2,{12,41}}, {3,{15,22,44}}, {2,{21,43}}, {2,{23,42}} } },
   {4, {{4,{11,24,53,65}}, {3,{12,45,61}}, {2,{14,22}}, {3,{23,42,55}} } },
   {4, {{4,{11,24,53,66}}, {3,{12,46,61}}, {2,{14,22}}, {3,{23,42,56}} } },
   {4, {{4,{11,24,63,55}}, {3,{12,45,51}}, {2,{14,22}}, {3,{23,42,65}} } },
   {4, {{4,{11,24,63,56}}, {3,{12,46,51}}, {2,{14,22}}, {3,{23,42,66}} } },
   {3, {{4,{11,24,43,55}}, {4,{12,26,41,54}}, {4,{15,23,42,56}} } },
   {3, {{4,{11,24,43,56}}, {4,{12,25,41,54}}, {4,{16,23,42,55}} } },
   {3, {{4,{11,24,43,65}}, {4,{12,26,41,64}}, {4,{15,23,42,66}} } },
   {3, {{4,{11,24,43,66}}, {4,{12,25,41,64}}, {4,{16,23,42,65}} } },
   {4, {{4,{11,24,43,65}}, {2,{12,41}}, {4,{15,23,42,66}}, {2,{26,64}} } },
   {4, {{4,{11,24,43,66}}, {2,{12,41}}, {4,{16,23,42,65}}, {2,{25,64}} } },
   {4, {{4,{11,24,43,55}}, {2,{12,41}}, {4,{15,23,42,56}}, {2,{26,54}} } },
   {4, {{4,{11,24,43,56}}, {2,{12,41}}, {4,{16,23,42,55}}, {2,{25,54}} } },
   {3, {{4,{11,24,55,63}}, {4,{15,32,46,51}}, {4,{23,36,42,64}} } },
   {3, {{4,{11,24,56,63}}, {4,{16,32,45,51}}, {4,{23,35,42,64}} } },
   {3, {{4,{11,24,65,53}}, {4,{15,32,46,61}}, {4,{23,36,42,54}} } },
   {3, {{4,{11,24,66,53}}, {4,{16,32,45,61}}, {4,{23,35,42,54}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {2,{14,22}}, {3,{21,44,53}}, {3,{23,42,54}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {2,{14,22}}, {3,{21,44,63}}, {3,{23,42,64}} } },
   {5, {{2,{11,24}}, {3,{12,44,61}}, {2,{14,22}}, {3,{21,43,64}}, {2,{23,42}} } },
   {5, {{2,{11,24}}, {3,{12,44,51}}, {2,{14,22}}, {3,{21,43,54}}, {2,{23,42}} } },
   {5, {{3,{11,24,63}}, {2,{12,41}}, {2,{14,22}}, {3,{23,42,64}}, {2,{44,61}} } },
   {5, {{3,{11,24,53}}, {2,{12,41}}, {2,{14,22}}, {3,{23,42,54}}, {2,{44,51}} } },
   {4, {{4,{11,24,43,66}}, {2,{16,31}}, {4,{23,36,42,64}}, {2,{32,41}} } },
   {4, {{4,{11,24,43,65}}, {2,{15,31}}, {4,{23,35,42,64}}, {2,{32,41}} } },
   {4, {{4,{11,24,43,56}}, {2,{16,31}}, {4,{23,36,42,54}}, {2,{32,41}} } },
   {4, {{4,{11,24,43,55}}, {2,{15,31}}, {4,{23,35,42,54}}, {2,{32,41}} } },
   {5, {{3,{11,24,43}}, {2,{12,61}}, {3,{14,23,42}}, {2,{31,62}}, {2,{32,41}} } },
   {5, {{3,{11,24,43}}, {2,{12,51}}, {3,{14,23,42}}, {2,{31,52}}, {2,{32,41}} } },
   {4, {{3,{11,24,53}}, {3,{14,43,55}}, {4,{15,22,44,51}}, {2,{23,42}} } },
   {4, {{3,{11,24,53}}, {3,{14,43,56}}, {4,{16,22,44,51}}, {2,{23,42}} } },
   {4, {{3,{11,24,63}}, {3,{14,43,65}}, {4,{15,22,44,61}}, {2,{23,42}} } },
   {4, {{3,{11,24,63}}, {3,{14,43,66}}, {4,{16,22,44,61}}, {2,{23,42}} } },
   {4, {{4,{11,24,43,66}}, {2,{16,31}}, {4,{22,36,44,61}}, {2,{23,42}} } },
   {4, {{4,{11,24,43,65}}, {2,{15,31}}, {4,{22,35,44,61}}, {2,{23,42}} } },
   {4, {{4,{11,24,43,56}}, {2,{16,31}}, {4,{22,36,44,51}}, {2,{23,42}} } },
   {4, {{4,{11,24,43,55}}, {2,{15,31}}, {4,{22,35,44,51}}, {2,{23,42}} } },
   {4, {{4,{11,24,45,63}}, {3,{14,22,41}}, {3,{15,43,64}}, {2,{23,42}} } },
   {4, {{4,{11,24,46,63}}, {3,{14,22,41}}, {3,{16,43,64}}, {2,{23,42}} } },
   {4, {{4,{11,24,45,53}}, {3,{14,22,41}}, {3,{15,43,54}}, {2,{23,42}} } },
   {4, {{4,{11,24,46,53}}, {3,{14,22,41}}, {3,{16,43,54}}, {2,{23,42}} } },
   {4, {{4,{11,24,56,63}}, {2,{12,41}}, {4,{16,23,42,64}}, {2,{46,51}} } },
   {4, {{4,{11,24,55,63}}, {2,{12,41}}, {4,{15,23,42,64}}, {2,{45,51}} } },
   {4, {{4,{11,24,66,53}}, {2,{12,41}}, {4,{16,23,42,54}}, {2,{46,61}} } },
   {4, {{4,{11,24,65,53}}, {2,{12,41}}, {4,{15,23,42,54}}, {2,{45,61}} } },
   {4, {{3,{11,24,53}}, {3,{12,44,51}}, {2,{14,45}}, {4,{15,23,42,54}} } },
   {4, {{3,{11,24,53}}, {3,{12,44,51}}, {2,{14,46}}, {4,{16,23,42,54}} } },
   {4, {{3,{11,24,63}}, {3,{12,44,61}}, {2,{14,45}}, {4,{15,23,42,64}} } },
   {4, {{3,{11,24,63}}, {3,{12,44,61}}, {2,{14,46}}, {4,{16,23,42,64}} } },
   {4, {{4,{11,24,43,66}}, {3,{16,32,41}}, {3,{22,36,64}}, {2,{23,42}} } },
   {4, {{4,{11,24,43,65}}, {3,{15,32,41}}, {3,{22,35,64}}, {2,{23,42}} } },
   {4, {{4,{11,24,43,56}}, {3,{16,32,41}}, {3,{22,36,54}}, {2,{23,42}} } },
   {4, {{4,{11,24,43,55}}, {3,{15,32,41}}, {3,{22,35,54}}, {2,{23,42}} } },
   {4, {{3,{11,24,46}}, {3,{12,44,61}}, {4,{16,23,42,64}}, {2,{21,63}} } },
   {4, {{3,{11,24,45}}, {3,{12,44,61}}, {4,{15,23,42,64}}, {2,{21,63}} } },
   {4, {{3,{11,24,46}}, {3,{12,44,51}}, {4,{16,23,42,54}}, {2,{21,53}} } },
   {4, {{3,{11,24,45}}, {3,{12,44,51}}, {4,{15,23,42,54}}, {2,{21,53}} } },
   }
   };

int const cntpat2x2[ 4 ] = {134, 116, 250, 88};

UnavPat const unavpat2x3[ 4 ][ 744 ] =
   {
   {
   {2, {{4,{11,24,37,43}}, {4,{17,23,34,41}} } },
   {3, {{3,{11,24,43}}, {2,{14,27}}, {3,{17,23,41}} } },
   {2, {{4,{11,24,47,53}}, {4,{14,23,41,57}} } },
   {2, {{4,{11,24,47,63}}, {4,{14,23,41,67}} } },
   {3, {{3,{11,24,43}}, {3,{14,27,31}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,42}}, {2,{14,27}}, {3,{17,23,41}}, {2,{22,43}} } },
   {3, {{4,{11,24,45,53}}, {4,{14,23,41,57}}, {2,{47,55}} } },
   {3, {{4,{11,24,46,53}}, {4,{14,23,41,57}}, {2,{47,56}} } },
   {3, {{4,{11,24,45,63}}, {4,{14,23,41,67}}, {2,{47,65}} } },
   {3, {{4,{11,24,46,63}}, {4,{14,23,41,67}}, {2,{47,66}} } },
   {3, {{3,{11,24,43}}, {3,{14,27,59}}, {4,{19,23,41,57}} } },
   {3, {{3,{11,24,43}}, {3,{14,27,58}}, {4,{18,23,41,57}} } },
   {3, {{3,{11,24,43}}, {3,{14,27,69}}, {4,{19,23,41,67}} } },
   {3, {{3,{11,24,43}}, {3,{14,27,68}}, {4,{18,23,41,67}} } },
   {4, {{3,{11,24,43}}, {2,{14,27}}, {3,{16,23,41}}, {2,{17,26}} } },
   {4, {{3,{11,24,43}}, {2,{14,27}}, {3,{15,23,41}}, {2,{17,25}} } },
   {2, {{5,{11,24,46,57,63}}, {5,{14,23,41,56,67}} } },
   {2, {{5,{11,24,45,57,63}}, {5,{14,23,41,55,67}} } },
   {2, {{5,{11,24,46,67,53}}, {5,{14,23,41,66,57}} } },
   {2, {{5,{11,24,45,67,53}}, {5,{14,23,41,65,57}} } },
   {4, {{3,{11,24,43}}, {2,{14,37}}, {2,{17,31}}, {3,{23,34,41}} } },
   {3, {{4,{11,24,37,42}}, {4,{17,23,34,41}}, {2,{22,43}} } },
   {2, {{5,{11,24,38,43,57}}, {5,{17,23,34,41,58}} } },
   {2, {{5,{11,24,39,43,57}}, {5,{17,23,34,41,59}} } },
   {2, {{5,{11,24,38,43,67}}, {5,{17,23,34,41,68}} } },
   {2, {{5,{11,24,39,43,67}}, {5,{17,23,34,41,69}} } },
   {2, {{5,{11,24,43,57,65}}, {5,{15,23,41,54,67}} } },
   {2, {{5,{11,24,43,57,66}}, {5,{16,23,41,54,67}} } },
   {2, {{5,{11,24,43,67,55}}, {5,{15,23,41,64,57}} } },
   {2, {{5,{11,24,43,67,56}}, {5,{16,23,41,64,57}} } },
   {2, {{5,{11,24,47,53,66}}, {5,{16,23,41,54,67}} } },
   {2, {{5,{11,24,47,53,65}}, {5,{15,23,41,54,67}} } },
   {2, {{5,{11,24,47,63,56}}, {5,{16,23,41,64,57}} } },
   {2, {{5,{11,24,47,63,55}}, {5,{15,23,41,64,57}} } },
   {4, {{3,{11,24,43}}, {2,{14,37}}, {3,{17,23,41}}, {2,{27,34}} } },
   {4, {{3,{11,24,37}}, {3,{14,23,41}}, {3,{27,32,43}}, {2,{31,42}} } },
   {4, {{3,{11,24,43}}, {2,{14,27}}, {3,{17,26,31}}, {3,{23,36,41}} } },
   {4, {{3,{11,24,43}}, {2,{14,27}}, {3,{17,25,31}}, {3,{23,35,41}} } },
   {4, {{3,{11,24,37}}, {2,{14,29}}, {3,{19,23,41}}, {3,{27,31,43}} } },
   {4, {{3,{11,24,37}}, {2,{14,28}}, {3,{18,23,41}}, {3,{27,31,43}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,31}}, {2,{22,43}}, {3,{23,37,41}} } },
   {3, {{4,{11,24,37,43}}, {4,{17,23,41,69}}, {3,{29,34,67}} } },
   {3, {{4,{11,24,37,43}}, {4,{17,23,41,68}}, {3,{28,34,67}} } },
   {3, {{4,{11,24,37,43}}, {4,{17,23,41,59}}, {3,{29,34,57}} } },
   {3, {{4,{11,24,37,43}}, {4,{17,23,41,58}}, {3,{28,34,57}} } },
   {3, {{3,{11,24,37}}, {4,{14,23,41,69}}, {4,{27,31,49,63}} } },
   {3, {{3,{11,24,37}}, {4,{14,23,41,68}}, {4,{27,31,48,63}} } },
   {3, {{3,{11,24,37}}, {4,{14,23,41,59}}, {4,{27,31,49,53}} } },
   {3, {{3,{11,24,37}}, {4,{14,23,41,58}}, {4,{27,31,48,53}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,21,47,64}}, {3,{23,41,67}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,21,47,64}}, {3,{23,41,67}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,21,47,54}}, {3,{23,41,57}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,21,47,54}}, {3,{23,41,57}} } },
   {4, {{4,{11,24,47,62}}, {3,{14,23,41}}, {2,{22,63}}, {2,{43,67}} } },
   {4, {{4,{11,24,47,52}}, {3,{14,23,41}}, {2,{22,53}}, {2,{43,57}} } },
   {3, {{4,{11,24,49,63}}, {3,{14,27,31}}, {4,{23,37,41,69}} } },
   {3, {{4,{11,24,48,63}}, {3,{14,27,31}}, {4,{23,37,41,68}} } },
   {3, {{4,{11,24,49,53}}, {3,{14,27,31}}, {4,{23,37,41,59}} } },
   {3, {{4,{11,24,48,53}}, {3,{14,27,31}}, {4,{23,37,41,58}} } },
   {3, {{4,{11,24,49,63}}, {3,{14,27,69}}, {4,{19,23,41,67}} } },
   {3, {{4,{11,24,48,63}}, {3,{14,27,68}}, {4,{18,23,41,67}} } },
   {3, {{4,{11,24,49,53}}, {3,{14,27,59}}, {4,{19,23,41,57}} } },
   {3, {{4,{11,24,48,53}}, {3,{14,27,58}}, {4,{18,23,41,57}} } },
   {4, {{4,{11,24,37,42}}, {2,{14,29}}, {3,{19,23,41}}, {3,{27,32,43}} } },
   {4, {{4,{11,24,37,42}}, {2,{14,28}}, {3,{18,23,41}}, {3,{27,32,43}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{43,55,67}}, {3,{45,57,61}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{43,56,67}}, {3,{46,57,61}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{43,65,57}}, {3,{45,67,51}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{43,66,57}}, {3,{46,67,51}} } },
   {5, {{3,{11,24,63}}, {3,{14,23,41}}, {2,{43,65}}, {2,{45,67}}, {2,{47,61}} } },
   {5, {{3,{11,24,63}}, {3,{14,23,41}}, {2,{43,66}}, {2,{46,67}}, {2,{47,61}} } },
   {5, {{3,{11,24,53}}, {3,{14,23,41}}, {2,{43,55}}, {2,{45,57}}, {2,{47,51}} } },
   {5, {{3,{11,24,53}}, {3,{14,23,41}}, {2,{43,56}}, {2,{46,57}}, {2,{47,51}} } },
   {4, {{4,{11,24,46,53}}, {2,{14,28}}, {4,{18,23,41,57}}, {2,{47,56}} } },
   {4, {{4,{11,24,46,53}}, {2,{14,29}}, {4,{19,23,41,57}}, {2,{47,56}} } },
   {4, {{4,{11,24,45,53}}, {2,{14,28}}, {4,{18,23,41,57}}, {2,{47,55}} } },
   {4, {{4,{11,24,45,53}}, {2,{14,29}}, {4,{19,23,41,57}}, {2,{47,55}} } },
   {4, {{4,{11,24,46,63}}, {2,{14,28}}, {4,{18,23,41,67}}, {2,{47,66}} } },
   {4, {{4,{11,24,46,63}}, {2,{14,29}}, {4,{19,23,41,67}}, {2,{47,66}} } },
   {4, {{4,{11,24,45,63}}, {2,{14,28}}, {4,{18,23,41,67}}, {2,{47,65}} } },
   {4, {{4,{11,24,45,63}}, {2,{14,29}}, {4,{19,23,41,67}}, {2,{47,65}} } },
   {3, {{5,{11,24,39,47,63}}, {5,{19,23,34,41,65}}, {2,{45,67}} } },
   {3, {{5,{11,24,38,47,63}}, {5,{18,23,34,41,65}}, {2,{45,67}} } },
   {3, {{5,{11,24,39,47,63}}, {5,{19,23,34,41,66}}, {2,{46,67}} } },
   {3, {{5,{11,24,38,47,63}}, {5,{18,23,34,41,66}}, {2,{46,67}} } },
   {3, {{5,{11,24,39,47,53}}, {5,{19,23,34,41,55}}, {2,{45,57}} } },
   {3, {{5,{11,24,38,47,53}}, {5,{18,23,34,41,55}}, {2,{45,57}} } },
   {3, {{5,{11,24,39,47,53}}, {5,{19,23,34,41,56}}, {2,{46,57}} } },
   {3, {{5,{11,24,38,47,53}}, {5,{18,23,34,41,56}}, {2,{46,57}} } },
   {4, {{4,{11,24,38,43}}, {4,{15,23,37,41}}, {2,{18,35}}, {2,{27,34}} } },
   {4, {{4,{11,24,39,43}}, {4,{15,23,37,41}}, {2,{19,35}}, {2,{27,34}} } },
   {4, {{4,{11,24,38,43}}, {4,{16,23,37,41}}, {2,{18,36}}, {2,{27,34}} } },
   {4, {{4,{11,24,39,43}}, {4,{16,23,37,41}}, {2,{19,36}}, {2,{27,34}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,46}}, {3,{23,37,41}}, {3,{27,36,44}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,45}}, {3,{23,37,41}}, {3,{27,35,44}} } },
   {3, {{4,{11,24,47,63}}, {4,{14,31,46,67}}, {4,{23,36,41,64}} } },
   {3, {{4,{11,24,47,63}}, {4,{14,31,45,67}}, {4,{23,35,41,64}} } },
   {3, {{4,{11,24,47,53}}, {4,{14,31,46,57}}, {4,{23,36,41,54}} } },
   {3, {{4,{11,24,47,53}}, {4,{14,31,45,57}}, {4,{23,35,41,54}} } },
   {3, {{5,{11,24,39,53,67}}, {5,{19,23,34,41,57}}, {2,{47,61}} } },
   {3, {{5,{11,24,38,53,67}}, {5,{18,23,34,41,57}}, {2,{47,61}} } },
   {3, {{5,{11,24,39,63,57}}, {5,{19,23,34,41,67}}, {2,{47,51}} } },
   {3, {{5,{11,24,38,63,57}}, {5,{18,23,34,41,67}}, {2,{47,51}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,29,47,64}}, {4,{17,23,41,69}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,28,47,64}}, {4,{17,23,41,68}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,29,47,64}}, {4,{17,23,41,69}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,28,47,64}}, {4,{17,23,41,68}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,29,47,54}}, {4,{17,23,41,59}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,28,47,54}}, {4,{17,23,41,58}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,29,47,54}}, {4,{17,23,41,59}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,28,47,54}}, {4,{17,23,41,58}} } },
   {5, {{3,{11,24,37}}, {2,{13,27}}, {3,{14,23,41}}, {2,{17,33}}, {2,{31,43}} } },
   {4, {{4,{11,24,37,43}}, {4,{17,23,36,41}}, {2,{26,44}}, {2,{34,46}} } },
   {4, {{4,{11,24,37,43}}, {4,{17,23,35,41}}, {2,{25,44}}, {2,{34,45}} } },
   {3, {{4,{11,24,53,66}}, {5,{16,23,41,57,64}}, {3,{46,51,67}} } },
   {3, {{4,{11,24,53,65}}, {5,{15,23,41,57,64}}, {3,{45,51,67}} } },
   {3, {{4,{11,24,63,56}}, {5,{16,23,41,67,54}}, {3,{46,61,57}} } },
   {3, {{4,{11,24,63,55}}, {5,{15,23,41,67,54}}, {3,{45,61,57}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{43,54,67}}, {3,{47,51,64}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{43,64,57}}, {3,{47,61,54}} } },
   {4, {{4,{11,24,39,43}}, {2,{13,37}}, {4,{17,23,34,41}}, {2,{19,33}} } },
   {4, {{4,{11,24,38,43}}, {2,{13,37}}, {4,{17,23,34,41}}, {2,{18,33}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,27}}, {4,{17,23,41,66}}, {2,{46,67}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,27}}, {4,{17,23,41,65}}, {2,{45,67}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,27}}, {4,{17,23,41,56}}, {2,{46,57}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,27}}, {4,{17,23,41,55}}, {2,{45,57}} } },
   {3, {{5,{11,24,37,49,62}}, {5,{14,23,39,41,67}}, {2,{22,63}} } },
   {3, {{5,{11,24,37,48,62}}, {5,{14,23,38,41,67}}, {2,{22,63}} } },
   {3, {{5,{11,24,37,49,52}}, {5,{14,23,39,41,57}}, {2,{22,53}} } },
   {3, {{5,{11,24,37,48,52}}, {5,{14,23,38,41,57}}, {2,{22,53}} } },
   {4, {{4,{11,24,49,53}}, {2,{14,37}}, {4,{17,23,41,59}}, {2,{27,34}} } },
   {4, {{4,{11,24,48,53}}, {2,{14,37}}, {4,{17,23,41,58}}, {2,{27,34}} } },
   {4, {{4,{11,24,49,63}}, {2,{14,37}}, {4,{17,23,41,69}}, {2,{27,34}} } },
   {4, {{4,{11,24,48,63}}, {2,{14,37}}, {4,{17,23,41,68}}, {2,{27,34}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {2,{17,31}}, {3,{23,38,41}}, {2,{28,34}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {2,{17,31}}, {3,{23,39,41}}, {2,{29,34}} } },
   {5, {{2,{11,24}}, {3,{12,27,43}}, {2,{14,22}}, {3,{17,23,41}}, {2,{21,42}} } },
   {4, {{4,{11,24,43,65}}, {3,{15,37,64}}, {3,{17,23,41}}, {2,{27,34}} } },
   {4, {{4,{11,24,43,66}}, {3,{16,37,64}}, {3,{17,23,41}}, {2,{27,34}} } },
   {4, {{4,{11,24,43,55}}, {3,{15,37,54}}, {3,{17,23,41}}, {2,{27,34}} } },
   {4, {{4,{11,24,43,56}}, {3,{16,37,54}}, {3,{17,23,41}}, {2,{27,34}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{17,43,68}}, {3,{18,47,61}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{17,43,69}}, {3,{19,47,61}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{17,43,58}}, {3,{18,47,51}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{17,43,59}}, {3,{19,47,51}} } },
   {4, {{4,{11,24,49,53}}, {4,{14,23,41,57}}, {2,{17,59}}, {2,{19,47}} } },
   {4, {{4,{11,24,48,53}}, {4,{14,23,41,57}}, {2,{17,58}}, {2,{18,47}} } },
   {4, {{4,{11,24,49,63}}, {4,{14,23,41,67}}, {2,{17,69}}, {2,{19,47}} } },
   {4, {{4,{11,24,48,63}}, {4,{14,23,41,67}}, {2,{17,68}}, {2,{18,47}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {3,{17,23,41}}, {2,{22,34}}, {2,{27,32}} } },
   {5, {{3,{11,24,39}}, {2,{14,27}}, {3,{17,23,41}}, {2,{29,33}}, {2,{31,43}} } },
   {5, {{3,{11,24,38}}, {2,{14,27}}, {3,{17,23,41}}, {2,{28,33}}, {2,{31,43}} } },
   {4, {{4,{11,24,49,63}}, {2,{14,27}}, {4,{17,23,41,65}}, {2,{45,69}} } },
   {4, {{4,{11,24,48,63}}, {2,{14,27}}, {4,{17,23,41,65}}, {2,{45,68}} } },
   {4, {{4,{11,24,49,63}}, {2,{14,27}}, {4,{17,23,41,66}}, {2,{46,69}} } },
   {4, {{4,{11,24,48,63}}, {2,{14,27}}, {4,{17,23,41,66}}, {2,{46,68}} } },
   {4, {{4,{11,24,49,53}}, {2,{14,27}}, {4,{17,23,41,55}}, {2,{45,59}} } },
   {4, {{4,{11,24,48,53}}, {2,{14,27}}, {4,{17,23,41,55}}, {2,{45,58}} } },
   {4, {{4,{11,24,49,53}}, {2,{14,27}}, {4,{17,23,41,56}}, {2,{46,59}} } },
   {4, {{4,{11,24,48,53}}, {2,{14,27}}, {4,{17,23,41,56}}, {2,{46,58}} } },
   {5, {{3,{11,24,43}}, {2,{14,27}}, {2,{16,31}}, {2,{17,26}}, {3,{23,36,41}} } },
   {5, {{3,{11,24,43}}, {2,{14,27}}, {2,{15,31}}, {2,{17,25}}, {3,{23,35,41}} } },
   {5, {{3,{11,24,53}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,59}}, {2,{49,51}} } },
   {5, {{3,{11,24,53}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,58}}, {2,{48,51}} } },
   {5, {{3,{11,24,63}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,69}}, {2,{49,61}} } },
   {5, {{3,{11,24,63}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,68}}, {2,{48,61}} } },
   {4, {{3,{11,24,43}}, {2,{14,31}}, {4,{23,37,41,59}}, {3,{29,34,57}} } },
   {4, {{3,{11,24,43}}, {2,{14,31}}, {4,{23,37,41,58}}, {3,{28,34,57}} } },
   {4, {{3,{11,24,43}}, {2,{14,31}}, {4,{23,37,41,69}}, {3,{29,34,67}} } },
   {4, {{3,{11,24,43}}, {2,{14,31}}, {4,{23,37,41,68}}, {3,{28,34,67}} } },
   {5, {{3,{11,24,43}}, {2,{14,31}}, {3,{23,35,41}}, {2,{25,37}}, {2,{27,34}} } },
   {5, {{3,{11,24,43}}, {2,{14,31}}, {3,{23,36,41}}, {2,{26,37}}, {2,{27,34}} } },
   {4, {{4,{11,24,37,43}}, {4,{17,23,36,41}}, {2,{26,39}}, {2,{29,34}} } },
   {4, {{4,{11,24,37,43}}, {4,{17,23,36,41}}, {2,{26,38}}, {2,{28,34}} } },
   {4, {{4,{11,24,37,43}}, {4,{17,23,35,41}}, {2,{25,39}}, {2,{29,34}} } },
   {4, {{4,{11,24,37,43}}, {4,{17,23,35,41}}, {2,{25,38}}, {2,{28,34}} } },
   {4, {{4,{11,24,53,67}}, {3,{14,23,41}}, {3,{43,57,66}}, {2,{46,61}} } },
   {4, {{4,{11,24,53,67}}, {3,{14,23,41}}, {3,{43,57,65}}, {2,{45,61}} } },
   {4, {{4,{11,24,63,57}}, {3,{14,23,41}}, {3,{43,67,56}}, {2,{46,51}} } },
   {4, {{4,{11,24,63,57}}, {3,{14,23,41}}, {3,{43,67,55}}, {2,{45,51}} } },
   {4, {{3,{11,24,53}}, {4,{14,23,41,67}}, {2,{45,51}}, {3,{47,55,63}} } },
   {4, {{3,{11,24,53}}, {4,{14,23,41,67}}, {2,{46,51}}, {3,{47,56,63}} } },
   {4, {{3,{11,24,63}}, {4,{14,23,41,57}}, {2,{45,61}}, {3,{47,65,53}} } },
   {4, {{3,{11,24,63}}, {4,{14,23,41,57}}, {2,{46,61}}, {3,{47,66,53}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,49}}, {3,{19,23,41}}, {3,{29,34,47}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,48}}, {3,{18,23,41}}, {3,{28,34,47}} } },
   {4, {{4,{11,24,57,63}}, {4,{14,23,41,66}}, {2,{46,51}}, {2,{56,67}} } },
   {4, {{4,{11,24,57,63}}, {4,{14,23,41,65}}, {2,{45,51}}, {2,{55,67}} } },
   {4, {{4,{11,24,67,53}}, {4,{14,23,41,56}}, {2,{46,61}}, {2,{66,57}} } },
   {4, {{4,{11,24,67,53}}, {4,{14,23,41,55}}, {2,{45,61}}, {2,{65,57}} } },
   {4, {{4,{11,24,48,63}}, {4,{14,23,41,67}}, {2,{46,68}}, {2,{47,66}} } },
   {4, {{4,{11,24,49,63}}, {4,{14,23,41,67}}, {2,{46,69}}, {2,{47,66}} } },
   {4, {{4,{11,24,48,63}}, {4,{14,23,41,67}}, {2,{45,68}}, {2,{47,65}} } },
   {4, {{4,{11,24,49,63}}, {4,{14,23,41,67}}, {2,{45,69}}, {2,{47,65}} } },
   {4, {{4,{11,24,48,53}}, {4,{14,23,41,57}}, {2,{46,58}}, {2,{47,56}} } },
   {4, {{4,{11,24,49,53}}, {4,{14,23,41,57}}, {2,{46,59}}, {2,{47,56}} } },
   {4, {{4,{11,24,48,53}}, {4,{14,23,41,57}}, {2,{45,58}}, {2,{47,55}} } },
   {4, {{4,{11,24,49,53}}, {4,{14,23,41,57}}, {2,{45,59}}, {2,{47,55}} } },
   {3, {{5,{11,24,39,47,63}}, {5,{17,23,34,41,59}}, {2,{53,69}} } },
   {3, {{5,{11,24,38,47,63}}, {5,{17,23,34,41,58}}, {2,{53,68}} } },
   {3, {{5,{11,24,39,47,53}}, {5,{17,23,34,41,69}}, {2,{63,59}} } },
   {3, {{5,{11,24,38,47,53}}, {5,{17,23,34,41,68}}, {2,{63,58}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,27,44,61}}, {4,{17,23,41,66}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,27,44,61}}, {4,{17,23,41,65}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,27,44,51}}, {4,{17,23,41,56}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,27,44,51}}, {4,{17,23,41,55}} } },
   {5, {{3,{11,24,53}}, {3,{14,23,41}}, {2,{43,67}}, {2,{47,51}}, {2,{57,63}} } },
   {5, {{3,{11,24,63}}, {3,{14,23,41}}, {2,{43,57}}, {2,{47,61}}, {2,{67,53}} } },
   {2, {{6,{11,24,39,47,53,65}}, {6,{17,23,34,41,55,69}} } },
   {2, {{6,{11,24,38,47,53,65}}, {6,{17,23,34,41,55,68}} } },
   {2, {{6,{11,24,39,47,53,66}}, {6,{17,23,34,41,56,69}} } },
   {2, {{6,{11,24,38,47,53,66}}, {6,{17,23,34,41,56,68}} } },
   {2, {{6,{11,24,39,47,63,55}}, {6,{17,23,34,41,65,59}} } },
   {2, {{6,{11,24,38,47,63,55}}, {6,{17,23,34,41,65,58}} } },
   {2, {{6,{11,24,39,47,63,56}}, {6,{17,23,34,41,66,59}} } },
   {2, {{6,{11,24,38,47,63,56}}, {6,{17,23,34,41,66,58}} } },
   {5, {{3,{11,24,37}}, {3,{14,23,41}}, {2,{27,33}}, {2,{31,42}}, {2,{32,43}} } },
   {4, {{4,{11,24,53,67}}, {2,{14,29}}, {4,{19,23,41,57}}, {2,{47,61}} } },
   {4, {{4,{11,24,53,67}}, {2,{14,28}}, {4,{18,23,41,57}}, {2,{47,61}} } },
   {4, {{4,{11,24,63,57}}, {2,{14,29}}, {4,{19,23,41,67}}, {2,{47,51}} } },
   {4, {{4,{11,24,63,57}}, {2,{14,28}}, {4,{18,23,41,67}}, {2,{47,51}} } },
   {3, {{4,{11,24,45,63}}, {4,{14,27,53,65}}, {4,{17,23,41,55}} } },
   {3, {{4,{11,24,46,63}}, {4,{14,27,53,66}}, {4,{17,23,41,56}} } },
   {3, {{4,{11,24,45,53}}, {4,{14,27,63,55}}, {4,{17,23,41,65}} } },
   {3, {{4,{11,24,46,53}}, {4,{14,27,63,56}}, {4,{17,23,41,66}} } },
   {2, {{6,{11,24,39,47,53,66}}, {6,{19,23,34,41,56,67}} } },
   {2, {{6,{11,24,38,47,53,66}}, {6,{18,23,34,41,56,67}} } },
   {2, {{6,{11,24,39,47,53,65}}, {6,{19,23,34,41,55,67}} } },
   {2, {{6,{11,24,38,47,53,65}}, {6,{18,23,34,41,55,67}} } },
   {2, {{6,{11,24,39,47,63,56}}, {6,{19,23,34,41,66,57}} } },
   {2, {{6,{11,24,38,47,63,56}}, {6,{18,23,34,41,66,57}} } },
   {2, {{6,{11,24,39,47,63,55}}, {6,{19,23,34,41,65,57}} } },
   {2, {{6,{11,24,38,47,63,55}}, {6,{18,23,34,41,65,57}} } },
   {5, {{3,{11,24,43}}, {2,{14,28}}, {2,{16,27}}, {3,{17,23,41}}, {2,{18,26}} } },
   {5, {{3,{11,24,43}}, {2,{14,29}}, {2,{16,27}}, {3,{17,23,41}}, {2,{19,26}} } },
   {5, {{3,{11,24,43}}, {2,{14,28}}, {2,{15,27}}, {3,{17,23,41}}, {2,{18,25}} } },
   {5, {{3,{11,24,43}}, {2,{14,29}}, {2,{15,27}}, {3,{17,23,41}}, {2,{19,25}} } },
   {3, {{5,{11,24,37,43,66}}, {5,{17,23,35,41,64}}, {2,{36,65}} } },
   {3, {{5,{11,24,37,43,65}}, {5,{17,23,36,41,64}}, {2,{35,66}} } },
   {3, {{5,{11,24,37,43,56}}, {5,{17,23,35,41,54}}, {2,{36,55}} } },
   {3, {{5,{11,24,37,43,55}}, {5,{17,23,36,41,54}}, {2,{35,56}} } },
   {4, {{4,{11,24,39,43}}, {3,{12,27,34}}, {3,{17,23,41}}, {2,{19,32}} } },
   {4, {{4,{11,24,38,43}}, {3,{12,27,34}}, {3,{17,23,41}}, {2,{18,32}} } },
   {5, {{3,{11,24,37}}, {3,{14,23,41}}, {2,{22,43}}, {2,{27,32}}, {2,{31,42}} } },
   {4, {{4,{11,24,37,43}}, {2,{13,26}}, {3,{16,27,31}}, {3,{23,34,41}} } },
   {4, {{4,{11,24,37,43}}, {2,{13,25}}, {3,{15,27,31}}, {3,{23,34,41}} } },
   {3, {{4,{11,24,43,66}}, {4,{16,27,59,64}}, {4,{19,23,41,57}} } },
   {3, {{4,{11,24,43,66}}, {4,{16,27,58,64}}, {4,{18,23,41,57}} } },
   {3, {{4,{11,24,43,65}}, {4,{15,27,59,64}}, {4,{19,23,41,57}} } },
   {3, {{4,{11,24,43,65}}, {4,{15,27,58,64}}, {4,{18,23,41,57}} } },
   {3, {{4,{11,24,43,56}}, {4,{16,27,69,54}}, {4,{19,23,41,67}} } },
   {3, {{4,{11,24,43,56}}, {4,{16,27,68,54}}, {4,{18,23,41,67}} } },
   {3, {{4,{11,24,43,55}}, {4,{15,27,69,54}}, {4,{19,23,41,67}} } },
   {3, {{4,{11,24,43,55}}, {4,{15,27,68,54}}, {4,{18,23,41,67}} } },
   {2, {{6,{11,24,37,49,56,63}}, {6,{16,23,39,41,54,67}} } },
   {2, {{6,{11,24,37,48,56,63}}, {6,{16,23,38,41,54,67}} } },
   {2, {{6,{11,24,37,49,55,63}}, {6,{15,23,39,41,54,67}} } },
   {2, {{6,{11,24,37,48,55,63}}, {6,{15,23,38,41,54,67}} } },
   {2, {{6,{11,24,37,49,66,53}}, {6,{16,23,39,41,64,57}} } },
   {2, {{6,{11,24,37,48,66,53}}, {6,{16,23,38,41,64,57}} } },
   {2, {{6,{11,24,37,49,65,53}}, {6,{15,23,39,41,64,57}} } },
   {2, {{6,{11,24,37,48,65,53}}, {6,{15,23,38,41,64,57}} } },
   {3, {{5,{11,24,45,53,67}}, {2,{15,31}}, {5,{23,35,41,57,64}} } },
   {3, {{5,{11,24,46,53,67}}, {2,{16,31}}, {5,{23,36,41,57,64}} } },
   {3, {{5,{11,24,45,63,57}}, {2,{15,31}}, {5,{23,35,41,67,54}} } },
   {3, {{5,{11,24,46,63,57}}, {2,{16,31}}, {5,{23,36,41,67,54}} } },
   {4, {{4,{11,24,37,43}}, {2,{17,49}}, {3,{19,23,41}}, {3,{29,34,47}} } },
   {4, {{4,{11,24,37,43}}, {2,{17,48}}, {3,{18,23,41}}, {3,{28,34,47}} } },
   {3, {{5,{11,24,46,57,63}}, {3,{14,23,41}}, {4,{16,43,54,67}} } },
   {3, {{5,{11,24,45,57,63}}, {3,{14,23,41}}, {4,{15,43,54,67}} } },
   {3, {{5,{11,24,46,67,53}}, {3,{14,23,41}}, {4,{16,43,64,57}} } },
   {3, {{5,{11,24,45,67,53}}, {3,{14,23,41}}, {4,{15,43,64,57}} } },
   {2, {{6,{11,24,37,43,59,66}}, {6,{19,23,36,41,57,64}} } },
   {2, {{6,{11,24,37,43,58,66}}, {6,{18,23,36,41,57,64}} } },
   {2, {{6,{11,24,37,43,59,65}}, {6,{19,23,35,41,57,64}} } },
   {2, {{6,{11,24,37,43,58,65}}, {6,{18,23,35,41,57,64}} } },
   {2, {{6,{11,24,37,43,69,56}}, {6,{19,23,36,41,67,54}} } },
   {2, {{6,{11,24,37,43,68,56}}, {6,{18,23,36,41,67,54}} } },
   {2, {{6,{11,24,37,43,69,55}}, {6,{19,23,35,41,67,54}} } },
   {2, {{6,{11,24,37,43,68,55}}, {6,{18,23,35,41,67,54}} } },
   {3, {{4,{11,24,45,63}}, {3,{15,37,46}}, {5,{17,23,36,41,64}} } },
   {3, {{4,{11,24,46,63}}, {3,{16,37,45}}, {5,{17,23,35,41,64}} } },
   {3, {{4,{11,24,45,53}}, {3,{15,37,46}}, {5,{17,23,36,41,54}} } },
   {3, {{4,{11,24,46,53}}, {3,{16,37,45}}, {5,{17,23,35,41,54}} } },
   {3, {{5,{11,24,37,46,53}}, {4,{15,23,41,54}}, {3,{17,36,45}} } },
   {3, {{5,{11,24,37,45,53}}, {4,{16,23,41,54}}, {3,{17,35,46}} } },
   {3, {{5,{11,24,37,46,63}}, {4,{15,23,41,64}}, {3,{17,36,45}} } },
   {3, {{5,{11,24,37,45,63}}, {4,{16,23,41,64}}, {3,{17,35,46}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,59}}, {4,{19,23,41,57}}, {2,{22,43}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,58}}, {4,{18,23,41,57}}, {2,{22,43}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,69}}, {4,{19,23,41,67}}, {2,{22,43}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,68}}, {4,{18,23,41,67}}, {2,{22,43}} } },
   {3, {{3,{11,24,43}}, {4,{14,45,57,63}}, {5,{15,23,41,54,67}} } },
   {3, {{3,{11,24,43}}, {4,{14,46,57,63}}, {5,{16,23,41,54,67}} } },
   {3, {{3,{11,24,43}}, {4,{14,45,67,53}}, {5,{15,23,41,64,57}} } },
   {3, {{3,{11,24,43}}, {4,{14,46,67,53}}, {5,{16,23,41,64,57}} } },
   {5, {{3,{11,24,53}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,54}}, {2,{44,51}} } },
   {5, {{3,{11,24,63}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,64}}, {2,{44,61}} } },
   {3, {{4,{11,24,37,43}}, {5,{17,23,36,41,54}}, {3,{34,46,53}} } },
   {3, {{4,{11,24,37,43}}, {5,{17,23,35,41,54}}, {3,{34,45,53}} } },
   {3, {{4,{11,24,37,43}}, {5,{17,23,36,41,64}}, {3,{34,46,63}} } },
   {3, {{4,{11,24,37,43}}, {5,{17,23,35,41,64}}, {3,{34,45,63}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,49}}, {3,{15,23,41}}, {3,{19,25,47}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,48}}, {3,{15,23,41}}, {3,{18,25,47}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,49}}, {3,{16,23,41}}, {3,{19,26,47}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,48}}, {3,{16,23,41}}, {3,{18,26,47}} } },
   {4, {{4,{11,24,37,42}}, {3,{12,27,43}}, {3,{14,23,41}}, {2,{17,32}} } },
   {3, {{4,{11,24,49,63}}, {4,{14,27,46,69}}, {4,{17,23,41,66}} } },
   {3, {{4,{11,24,48,63}}, {4,{14,27,46,68}}, {4,{17,23,41,66}} } },
   {3, {{4,{11,24,49,63}}, {4,{14,27,45,69}}, {4,{17,23,41,65}} } },
   {3, {{4,{11,24,48,63}}, {4,{14,27,45,68}}, {4,{17,23,41,65}} } },
   {3, {{4,{11,24,49,53}}, {4,{14,27,46,59}}, {4,{17,23,41,56}} } },
   {3, {{4,{11,24,48,53}}, {4,{14,27,46,58}}, {4,{17,23,41,56}} } },
   {3, {{4,{11,24,49,53}}, {4,{14,27,45,59}}, {4,{17,23,41,55}} } },
   {3, {{4,{11,24,48,53}}, {4,{14,27,45,58}}, {4,{17,23,41,55}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {3,{15,23,41}}, {2,{17,25}}, {2,{27,34}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {3,{16,23,41}}, {2,{17,26}}, {2,{27,34}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {2,{17,21}}, {3,{23,34,41}}, {2,{27,31}} } },
   {4, {{3,{11,24,43}}, {2,{14,28}}, {3,{18,31,67}}, {4,{23,37,41,68}} } },
   {4, {{3,{11,24,43}}, {2,{14,29}}, {3,{19,31,67}}, {4,{23,37,41,69}} } },
   {4, {{3,{11,24,43}}, {2,{14,28}}, {3,{18,31,57}}, {4,{23,37,41,58}} } },
   {4, {{3,{11,24,43}}, {2,{14,29}}, {3,{19,31,57}}, {4,{23,37,41,59}} } },
   {5, {{3,{11,24,43}}, {2,{14,27}}, {2,{15,31}}, {2,{17,35}}, {3,{23,37,41}} } },
   {5, {{3,{11,24,43}}, {2,{14,27}}, {2,{16,31}}, {2,{17,36}}, {3,{23,37,41}} } },
   {3, {{5,{11,24,37,42,65}}, {4,{15,23,41,64}}, {3,{27,32,43}} } },
   {3, {{5,{11,24,37,42,66}}, {4,{16,23,41,64}}, {3,{27,32,43}} } },
   {3, {{5,{11,24,37,42,55}}, {4,{15,23,41,54}}, {3,{27,32,43}} } },
   {3, {{5,{11,24,37,42,56}}, {4,{16,23,41,54}}, {3,{27,32,43}} } },
   {4, {{4,{11,24,43,66}}, {2,{12,27}}, {3,{16,22,64}}, {3,{17,23,41}} } },
   {4, {{4,{11,24,43,65}}, {2,{12,27}}, {3,{15,22,64}}, {3,{17,23,41}} } },
   {4, {{4,{11,24,43,56}}, {2,{12,27}}, {3,{16,22,54}}, {3,{17,23,41}} } },
   {4, {{4,{11,24,43,55}}, {2,{12,27}}, {3,{15,22,54}}, {3,{17,23,41}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,27}}, {4,{17,23,41,64}}, {2,{44,67}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,27}}, {4,{17,23,41,54}}, {2,{44,57}} } },
   {2, {{6,{11,24,39,47,53,65}}, {6,{17,23,35,41,54,69}} } },
   {2, {{6,{11,24,38,47,53,65}}, {6,{17,23,35,41,54,68}} } },
   {2, {{6,{11,24,39,47,53,66}}, {6,{17,23,36,41,54,69}} } },
   {2, {{6,{11,24,38,47,53,66}}, {6,{17,23,36,41,54,68}} } },
   {2, {{6,{11,24,39,47,63,55}}, {6,{17,23,35,41,64,59}} } },
   {2, {{6,{11,24,38,47,63,55}}, {6,{17,23,35,41,64,58}} } },
   {2, {{6,{11,24,39,47,63,56}}, {6,{17,23,36,41,64,59}} } },
   {2, {{6,{11,24,38,47,63,56}}, {6,{17,23,36,41,64,58}} } },
   {2, {{6,{11,24,38,45,53,67}}, {6,{17,23,35,41,58,64}} } },
   {2, {{6,{11,24,39,45,53,67}}, {6,{17,23,35,41,59,64}} } },
   {2, {{6,{11,24,38,46,53,67}}, {6,{17,23,36,41,58,64}} } },
   {2, {{6,{11,24,39,46,53,67}}, {6,{17,23,36,41,59,64}} } },
   {2, {{6,{11,24,38,45,63,57}}, {6,{17,23,35,41,68,54}} } },
   {2, {{6,{11,24,39,45,63,57}}, {6,{17,23,35,41,69,54}} } },
   {2, {{6,{11,24,38,46,63,57}}, {6,{17,23,36,41,68,54}} } },
   {2, {{6,{11,24,39,46,63,57}}, {6,{17,23,36,41,69,54}} } },
   {3, {{5,{11,24,46,52,67}}, {4,{14,23,41,66}}, {3,{22,57,63}} } },
   {3, {{5,{11,24,45,52,67}}, {4,{14,23,41,65}}, {3,{22,57,63}} } },
   {3, {{5,{11,24,46,62,57}}, {4,{14,23,41,56}}, {3,{22,67,53}} } },
   {3, {{5,{11,24,45,62,57}}, {4,{14,23,41,55}}, {3,{22,67,53}} } },
   {3, {{5,{11,24,38,43,57}}, {4,{17,23,41,59}}, {3,{29,34,58}} } },
   {3, {{5,{11,24,39,43,57}}, {4,{17,23,41,58}}, {3,{28,34,59}} } },
   {3, {{5,{11,24,38,43,67}}, {4,{17,23,41,69}}, {3,{29,34,68}} } },
   {3, {{5,{11,24,39,43,67}}, {4,{17,23,41,68}}, {3,{28,34,69}} } },
   {2, {{6,{11,24,39,45,57,63}}, {6,{17,23,35,41,59,64}} } },
   {2, {{6,{11,24,38,45,57,63}}, {6,{17,23,35,41,58,64}} } },
   {2, {{6,{11,24,39,46,57,63}}, {6,{17,23,36,41,59,64}} } },
   {2, {{6,{11,24,38,46,57,63}}, {6,{17,23,36,41,58,64}} } },
   {2, {{6,{11,24,39,45,67,53}}, {6,{17,23,35,41,69,54}} } },
   {2, {{6,{11,24,38,45,67,53}}, {6,{17,23,35,41,68,54}} } },
   {2, {{6,{11,24,39,46,67,53}}, {6,{17,23,36,41,69,54}} } },
   {2, {{6,{11,24,38,46,67,53}}, {6,{17,23,36,41,68,54}} } },
   {2, {{6,{11,24,37,45,59,63}}, {6,{17,23,34,41,55,69}} } },
   {2, {{6,{11,24,37,45,58,63}}, {6,{17,23,34,41,55,68}} } },
   {2, {{6,{11,24,37,46,59,63}}, {6,{17,23,34,41,56,69}} } },
   {2, {{6,{11,24,37,46,58,63}}, {6,{17,23,34,41,56,68}} } },
   {2, {{6,{11,24,37,45,69,53}}, {6,{17,23,34,41,65,59}} } },
   {2, {{6,{11,24,37,45,68,53}}, {6,{17,23,34,41,65,58}} } },
   {2, {{6,{11,24,37,46,69,53}}, {6,{17,23,34,41,66,59}} } },
   {2, {{6,{11,24,37,46,68,53}}, {6,{17,23,34,41,66,58}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,56}}, {4,{17,23,36,41}}, {2,{26,54}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,55}}, {4,{17,23,35,41}}, {2,{25,54}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,66}}, {4,{17,23,36,41}}, {2,{26,64}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,65}}, {4,{17,23,35,41}}, {2,{25,64}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,69}}, {2,{19,31}}, {4,{23,39,41,67}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,68}}, {2,{18,31}}, {4,{23,38,41,67}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,59}}, {2,{19,31}}, {4,{23,39,41,57}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,58}}, {2,{18,31}}, {4,{23,38,41,57}} } },
   {4, {{4,{11,24,46,57}}, {4,{16,23,41,54}}, {2,{21,53}}, {2,{47,51}} } },
   {4, {{4,{11,24,45,57}}, {4,{15,23,41,54}}, {2,{21,53}}, {2,{47,51}} } },
   {4, {{4,{11,24,46,67}}, {4,{16,23,41,64}}, {2,{21,63}}, {2,{47,61}} } },
   {4, {{4,{11,24,45,67}}, {4,{15,23,41,64}}, {2,{21,63}}, {2,{47,61}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,39}}, {2,{19,31}}, {4,{23,34,41,67}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,38}}, {2,{18,31}}, {4,{23,34,41,67}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,39}}, {2,{19,31}}, {4,{23,34,41,57}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,38}}, {2,{18,31}}, {4,{23,34,41,57}} } },
   {4, {{4,{11,24,46,53}}, {3,{16,27,44}}, {3,{17,23,41}}, {2,{43,54}} } },
   {4, {{4,{11,24,45,53}}, {3,{15,27,44}}, {3,{17,23,41}}, {2,{43,54}} } },
   {4, {{4,{11,24,46,63}}, {3,{16,27,44}}, {3,{17,23,41}}, {2,{43,64}} } },
   {4, {{4,{11,24,45,63}}, {3,{15,27,44}}, {3,{17,23,41}}, {2,{43,64}} } },
   {3, {{4,{11,24,37,43}}, {4,{15,23,41,54}}, {4,{17,34,45,53}} } },
   {3, {{4,{11,24,37,43}}, {4,{16,23,41,54}}, {4,{17,34,46,53}} } },
   {3, {{4,{11,24,37,43}}, {4,{15,23,41,64}}, {4,{17,34,45,63}} } },
   {3, {{4,{11,24,37,43}}, {4,{16,23,41,64}}, {4,{17,34,46,63}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,48}}, {3,{18,31,47}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,49}}, {3,{19,31,47}}, {3,{23,37,41}} } },
   {5, {{3,{11,24,43}}, {2,{12,27}}, {2,{14,32}}, {3,{17,23,41}}, {2,{22,34}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,21,57,64}}, {4,{23,41,56,67}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,21,57,64}}, {4,{23,41,55,67}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,21,67,54}}, {4,{23,41,66,57}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,21,67,54}}, {4,{23,41,65,57}} } },
   {4, {{4,{11,24,46,63}}, {4,{16,23,41,54}}, {2,{53,67}}, {2,{57,64}} } },
   {4, {{4,{11,24,45,63}}, {4,{15,23,41,54}}, {2,{53,67}}, {2,{57,64}} } },
   {4, {{4,{11,24,46,53}}, {4,{16,23,41,64}}, {2,{63,57}}, {2,{67,54}} } },
   {4, {{4,{11,24,45,53}}, {4,{15,23,41,64}}, {2,{63,57}}, {2,{67,54}} } },
   {4, {{4,{11,24,47,53}}, {4,{14,23,41,56}}, {2,{26,54}}, {2,{44,57}} } },
   {4, {{4,{11,24,47,53}}, {4,{14,23,41,55}}, {2,{25,54}}, {2,{44,57}} } },
   {4, {{4,{11,24,47,63}}, {4,{14,23,41,66}}, {2,{26,64}}, {2,{44,67}} } },
   {4, {{4,{11,24,47,63}}, {4,{14,23,41,65}}, {2,{25,64}}, {2,{44,67}} } },
   {4, {{3,{11,24,63}}, {4,{14,23,41,57}}, {3,{47,54,61}}, {2,{53,64}} } },
   {4, {{3,{11,24,53}}, {4,{14,23,41,67}}, {3,{47,64,51}}, {2,{63,54}} } },
   {4, {{4,{11,24,37,55}}, {4,{15,23,41,54}}, {2,{27,33}}, {2,{31,43}} } },
   {4, {{4,{11,24,37,56}}, {4,{16,23,41,54}}, {2,{27,33}}, {2,{31,43}} } },
   {4, {{4,{11,24,37,65}}, {4,{15,23,41,64}}, {2,{27,33}}, {2,{31,43}} } },
   {4, {{4,{11,24,37,66}}, {4,{16,23,41,64}}, {2,{27,33}}, {2,{31,43}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,68}}, {3,{18,31,67}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,69}}, {3,{19,31,67}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,58}}, {3,{18,31,57}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,59}}, {3,{19,31,57}}, {3,{23,37,41}} } },
   {4, {{4,{11,24,45,53}}, {3,{14,23,41}}, {3,{15,47,54}}, {2,{43,57}} } },
   {4, {{4,{11,24,46,53}}, {3,{14,23,41}}, {3,{16,47,54}}, {2,{43,57}} } },
   {4, {{4,{11,24,45,63}}, {3,{14,23,41}}, {3,{15,47,64}}, {2,{43,67}} } },
   {4, {{4,{11,24,46,63}}, {3,{14,23,41}}, {3,{16,47,64}}, {2,{43,67}} } },
   {3, {{5,{11,24,37,48,63}}, {5,{14,23,39,41,67}}, {2,{38,49}} } },
   {3, {{5,{11,24,37,49,63}}, {5,{14,23,38,41,67}}, {2,{39,48}} } },
   {3, {{5,{11,24,37,48,53}}, {5,{14,23,39,41,57}}, {2,{38,49}} } },
   {3, {{5,{11,24,37,49,53}}, {5,{14,23,38,41,57}}, {2,{39,48}} } },
   {3, {{4,{11,24,47,53}}, {5,{14,23,37,41,59}}, {3,{29,34,57}} } },
   {3, {{4,{11,24,47,53}}, {5,{14,23,37,41,58}}, {3,{28,34,57}} } },
   {3, {{4,{11,24,47,63}}, {5,{14,23,37,41,69}}, {3,{29,34,67}} } },
   {3, {{4,{11,24,47,63}}, {5,{14,23,37,41,68}}, {3,{28,34,67}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {3,{17,23,41}}, {2,{21,34}}, {2,{27,31}} } },
   {3, {{4,{11,24,57,65}}, {5,{15,23,41,54,67}}, {3,{21,45,63}} } },
   {3, {{4,{11,24,57,66}}, {5,{16,23,41,54,67}}, {3,{21,46,63}} } },
   {3, {{4,{11,24,67,55}}, {5,{15,23,41,64,57}}, {3,{21,45,53}} } },
   {3, {{4,{11,24,67,56}}, {5,{16,23,41,64,57}}, {3,{21,46,53}} } },
   {3, {{5,{11,24,47,55,63}}, {3,{15,31,54}}, {4,{23,34,41,67}} } },
   {3, {{5,{11,24,47,56,63}}, {3,{16,31,54}}, {4,{23,34,41,67}} } },
   {3, {{5,{11,24,47,65,53}}, {3,{15,31,64}}, {4,{23,34,41,57}} } },
   {3, {{5,{11,24,47,66,53}}, {3,{16,31,64}}, {4,{23,34,41,57}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,69}}, {3,{19,31,67}}, {3,{23,34,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,68}}, {3,{18,31,67}}, {3,{23,34,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,59}}, {3,{19,31,57}}, {3,{23,34,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,58}}, {3,{18,31,57}}, {3,{23,34,41}} } },
   {3, {{4,{11,24,53,65}}, {4,{14,23,41,57}}, {4,{15,47,54,61}} } },
   {3, {{4,{11,24,53,66}}, {4,{14,23,41,57}}, {4,{16,47,54,61}} } },
   {3, {{4,{11,24,63,55}}, {4,{14,23,41,67}}, {4,{15,47,64,51}} } },
   {3, {{4,{11,24,63,56}}, {4,{14,23,41,67}}, {4,{16,47,64,51}} } },
   {4, {{3,{11,24,42}}, {4,{14,22,37,43}}, {2,{17,31}}, {3,{23,34,41}} } },
   {3, {{5,{11,24,37,49,63}}, {3,{19,31,67}}, {4,{23,34,41,69}} } },
   {3, {{5,{11,24,37,48,63}}, {3,{18,31,67}}, {4,{23,34,41,68}} } },
   {3, {{5,{11,24,37,49,53}}, {3,{19,31,57}}, {4,{23,34,41,59}} } },
   {3, {{5,{11,24,37,48,53}}, {3,{18,31,57}}, {4,{23,34,41,58}} } },
   {4, {{3,{11,24,43}}, {3,{14,33,61}}, {4,{23,34,41,67}}, {2,{47,63}} } },
   {4, {{3,{11,24,43}}, {3,{14,33,51}}, {4,{23,34,41,57}}, {2,{47,53}} } },
   {3, {{5,{11,24,37,46,62}}, {3,{14,23,41}}, {4,{27,32,43,66}} } },
   {3, {{5,{11,24,37,45,62}}, {3,{14,23,41}}, {4,{27,32,43,65}} } },
   {3, {{5,{11,24,37,46,52}}, {3,{14,23,41}}, {4,{27,32,43,56}} } },
   {3, {{5,{11,24,37,45,52}}, {3,{14,23,41}}, {4,{27,32,43,55}} } },
   {3, {{3,{11,24,63}}, {5,{14,23,37,41,69}}, {4,{29,34,47,61}} } },
   {3, {{3,{11,24,63}}, {5,{14,23,37,41,68}}, {4,{28,34,47,61}} } },
   {3, {{3,{11,24,53}}, {5,{14,23,37,41,59}}, {4,{29,34,47,51}} } },
   {3, {{3,{11,24,53}}, {5,{14,23,37,41,58}}, {4,{28,34,47,51}} } },
   {4, {{4,{11,24,47,62}}, {4,{14,23,41,57}}, {2,{22,63}}, {2,{53,67}} } },
   {4, {{4,{11,24,47,52}}, {4,{14,23,41,67}}, {2,{22,53}}, {2,{63,57}} } },
   {4, {{3,{11,24,43}}, {3,{14,29,67}}, {3,{15,23,41}}, {3,{17,25,69}} } },
   {4, {{3,{11,24,43}}, {3,{14,28,67}}, {3,{15,23,41}}, {3,{17,25,68}} } },
   {4, {{3,{11,24,43}}, {3,{14,29,67}}, {3,{16,23,41}}, {3,{17,26,69}} } },
   {4, {{3,{11,24,43}}, {3,{14,28,67}}, {3,{16,23,41}}, {3,{17,26,68}} } },
   {4, {{3,{11,24,43}}, {3,{14,29,57}}, {3,{15,23,41}}, {3,{17,25,59}} } },
   {4, {{3,{11,24,43}}, {3,{14,28,57}}, {3,{15,23,41}}, {3,{17,25,58}} } },
   {4, {{3,{11,24,43}}, {3,{14,29,57}}, {3,{16,23,41}}, {3,{17,26,59}} } },
   {4, {{3,{11,24,43}}, {3,{14,28,57}}, {3,{16,23,41}}, {3,{17,26,58}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,65}}, {3,{23,37,41}}, {3,{27,35,64}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,66}}, {3,{23,37,41}}, {3,{27,36,64}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,55}}, {3,{23,37,41}}, {3,{27,35,54}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,56}}, {3,{23,37,41}}, {3,{27,36,54}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,27}}, {4,{15,23,41,67}}, {2,{17,25}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,27}}, {4,{16,23,41,67}}, {2,{17,26}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,27}}, {4,{15,23,41,57}}, {2,{17,25}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,27}}, {4,{16,23,41,57}}, {2,{17,26}} } },
   {3, {{4,{11,24,37,49}}, {4,{14,23,41,59}}, {4,{29,31,47,53}} } },
   {3, {{4,{11,24,37,48}}, {4,{14,23,41,58}}, {4,{28,31,47,53}} } },
   {3, {{4,{11,24,37,49}}, {4,{14,23,41,69}}, {4,{29,31,47,63}} } },
   {3, {{4,{11,24,37,48}}, {4,{14,23,41,68}}, {4,{28,31,47,63}} } },
   },
   {
   {5, {{2,{11,24}}, {2,{12,41}}, {2,{14,27}}, {2,{17,22}}, {2,{21,42}} } },
   {4, {{3,{11,24,37}}, {2,{12,41}}, {3,{17,22,34}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {3,{12,47,61}}, {2,{14,22}}, {3,{21,42,67}} } },
   {4, {{2,{11,24}}, {3,{12,47,51}}, {2,{14,22}}, {3,{21,42,57}} } },
   {4, {{2,{11,24}}, {2,{12,41}}, {3,{14,27,32}}, {3,{21,37,42}} } },
   {3, {{2,{11,24}}, {4,{14,32,47,61}}, {4,{21,34,42,67}} } },
   {3, {{2,{11,24}}, {4,{14,32,47,51}}, {4,{21,34,42,57}} } },
   {4, {{2,{11,24}}, {3,{14,22,37}}, {3,{17,32,41}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {3,{14,32,41}}, {3,{21,37,42}}, {2,{27,34}} } },
   {4, {{2,{11,24}}, {2,{14,37}}, {3,{17,32,41}}, {3,{21,34,42}} } },
   {5, {{2,{11,24}}, {2,{13,41}}, {2,{14,27}}, {3,{17,22,43}}, {2,{21,42}} } },
   {5, {{2,{11,24}}, {3,{14,22,37}}, {2,{17,31}}, {2,{21,42}}, {2,{32,41}} } },
   {5, {{2,{11,24}}, {3,{14,32,41}}, {2,{21,42}}, {2,{22,37}}, {2,{27,34}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {2,{14,32}}, {3,{21,37,42}}, {2,{27,34}} } },
   {4, {{3,{11,24,37}}, {2,{13,41}}, {4,{17,22,34,43}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {3,{12,47,61}}, {2,{14,32}}, {4,{21,34,42,67}} } },
   {4, {{2,{11,24}}, {3,{12,47,51}}, {2,{14,32}}, {4,{21,34,42,57}} } },
   {5, {{2,{11,24}}, {2,{14,27}}, {3,{17,32,41}}, {2,{21,42}}, {2,{22,37}} } },
   {3, {{4,{11,24,37,59}}, {4,{19,32,41,57}}, {3,{21,34,42}} } },
   {3, {{4,{11,24,37,58}}, {4,{18,32,41,57}}, {3,{21,34,42}} } },
   {3, {{4,{11,24,37,69}}, {4,{19,32,41,67}}, {3,{21,34,42}} } },
   {3, {{4,{11,24,37,68}}, {4,{18,32,41,67}}, {3,{21,34,42}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {2,{14,27}}, {3,{17,26,32}}, {3,{21,36,42}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {2,{14,27}}, {3,{17,25,32}}, {3,{21,35,42}} } },
   {6, {{2,{11,24}}, {2,{12,43}}, {2,{13,41}}, {2,{14,27}}, {2,{17,22}}, {2,{21,42}} } },
   {4, {{3,{11,24,37}}, {4,{12,28,34,41}}, {2,{17,32}}, {3,{21,38,42}} } },
   {4, {{3,{11,24,37}}, {4,{12,29,34,41}}, {2,{17,32}}, {3,{21,39,42}} } },
   {4, {{2,{11,24}}, {4,{14,31,45,57}}, {4,{21,35,42,54}}, {2,{47,52}} } },
   {4, {{2,{11,24}}, {4,{14,31,46,57}}, {4,{21,36,42,54}}, {2,{47,52}} } },
   {4, {{2,{11,24}}, {4,{14,31,45,67}}, {4,{21,35,42,64}}, {2,{47,62}} } },
   {4, {{2,{11,24}}, {4,{14,31,46,67}}, {4,{21,36,42,64}}, {2,{47,62}} } },
   {2, {{6,{11,24,37,46,59,62}}, {6,{19,21,36,42,54,67}} } },
   {2, {{6,{11,24,37,46,58,62}}, {6,{18,21,36,42,54,67}} } },
   {2, {{6,{11,24,37,45,59,62}}, {6,{19,21,35,42,54,67}} } },
   {2, {{6,{11,24,37,45,58,62}}, {6,{18,21,35,42,54,67}} } },
   {2, {{6,{11,24,37,46,69,52}}, {6,{19,21,36,42,64,57}} } },
   {2, {{6,{11,24,37,46,68,52}}, {6,{18,21,36,42,64,57}} } },
   {2, {{6,{11,24,37,45,69,52}}, {6,{19,21,35,42,64,57}} } },
   {2, {{6,{11,24,37,45,68,52}}, {6,{18,21,35,42,64,57}} } },
   {3, {{4,{11,24,46,57}}, {4,{16,32,47,51}}, {4,{21,36,42,54}} } },
   {3, {{4,{11,24,45,57}}, {4,{15,32,47,51}}, {4,{21,35,42,54}} } },
   {3, {{4,{11,24,46,67}}, {4,{16,32,47,61}}, {4,{21,36,42,64}} } },
   {3, {{4,{11,24,45,67}}, {4,{15,32,47,61}}, {4,{21,35,42,64}} } },
   {3, {{2,{11,24}}, {5,{14,29,47,52,66}}, {5,{17,21,42,56,69}} } },
   {3, {{2,{11,24}}, {5,{14,28,47,52,66}}, {5,{17,21,42,56,68}} } },
   {3, {{2,{11,24}}, {5,{14,29,47,52,65}}, {5,{17,21,42,55,69}} } },
   {3, {{2,{11,24}}, {5,{14,28,47,52,65}}, {5,{17,21,42,55,68}} } },
   {3, {{2,{11,24}}, {5,{14,29,47,62,56}}, {5,{17,21,42,66,59}} } },
   {3, {{2,{11,24}}, {5,{14,28,47,62,56}}, {5,{17,21,42,66,58}} } },
   {3, {{2,{11,24}}, {5,{14,29,47,62,55}}, {5,{17,21,42,65,59}} } },
   {3, {{2,{11,24}}, {5,{14,28,47,62,55}}, {5,{17,21,42,65,58}} } },
   {5, {{3,{11,24,38}}, {2,{12,41}}, {2,{18,32}}, {3,{21,37,42}}, {2,{27,34}} } },
   {5, {{3,{11,24,39}}, {2,{12,41}}, {2,{19,32}}, {3,{21,37,42}}, {2,{27,34}} } },
   {5, {{3,{11,24,37}}, {2,{12,43}}, {2,{13,41}}, {3,{17,22,34}}, {2,{21,42}} } },
   {3, {{4,{11,24,46,52}}, {4,{16,27,49,54}}, {4,{19,21,42,57}} } },
   {3, {{4,{11,24,46,52}}, {4,{16,27,48,54}}, {4,{18,21,42,57}} } },
   {3, {{4,{11,24,45,52}}, {4,{15,27,49,54}}, {4,{19,21,42,57}} } },
   {3, {{4,{11,24,45,52}}, {4,{15,27,48,54}}, {4,{18,21,42,57}} } },
   {3, {{4,{11,24,46,62}}, {4,{16,27,49,64}}, {4,{19,21,42,67}} } },
   {3, {{4,{11,24,46,62}}, {4,{16,27,48,64}}, {4,{18,21,42,67}} } },
   {3, {{4,{11,24,45,62}}, {4,{15,27,49,64}}, {4,{19,21,42,67}} } },
   {3, {{4,{11,24,45,62}}, {4,{15,27,48,64}}, {4,{18,21,42,67}} } },
   {5, {{2,{11,24}}, {2,{13,61}}, {4,{14,22,47,63}}, {2,{21,42}}, {2,{41,67}} } },
   {5, {{2,{11,24}}, {2,{13,51}}, {4,{14,22,47,53}}, {2,{21,42}}, {2,{41,57}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,27}}, {2,{16,22}}, {2,{17,26}}, {2,{21,42}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,27}}, {2,{15,22}}, {2,{17,25}}, {2,{21,42}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,27}}, {2,{17,32}}, {2,{21,42}}, {2,{22,37}} } },
   {5, {{2,{11,24}}, {3,{14,22,37}}, {3,{17,33,41}}, {2,{21,42}}, {2,{32,43}} } },
   {5, {{2,{11,24}}, {3,{12,44,51}}, {2,{14,27}}, {2,{17,22}}, {3,{21,42,54}} } },
   {5, {{2,{11,24}}, {3,{12,44,61}}, {2,{14,27}}, {2,{17,22}}, {3,{21,42,64}} } },
   {5, {{2,{11,24}}, {2,{13,41}}, {2,{14,27}}, {3,{17,32,43}}, {3,{21,37,42}} } },
   {5, {{3,{11,24,46}}, {2,{12,41}}, {3,{16,27,44}}, {2,{17,22}}, {2,{21,42}} } },
   {5, {{3,{11,24,45}}, {2,{12,41}}, {3,{15,27,44}}, {2,{17,22}}, {2,{21,42}} } },
   {3, {{4,{11,24,47,55}}, {4,{14,32,45,51}}, {4,{21,34,42,57}} } },
   {3, {{4,{11,24,47,56}}, {4,{14,32,46,51}}, {4,{21,34,42,57}} } },
   {3, {{4,{11,24,47,65}}, {4,{14,32,45,61}}, {4,{21,34,42,67}} } },
   {3, {{4,{11,24,47,66}}, {4,{14,32,46,61}}, {4,{21,34,42,67}} } },
   {3, {{5,{11,24,47,52,65}}, {2,{15,66}}, {5,{16,21,42,54,67}} } },
   {3, {{5,{11,24,47,52,66}}, {2,{16,65}}, {5,{15,21,42,54,67}} } },
   {3, {{5,{11,24,47,62,55}}, {2,{15,56}}, {5,{16,21,42,64,57}} } },
   {3, {{5,{11,24,47,62,56}}, {2,{16,55}}, {5,{15,21,42,64,57}} } },
   {5, {{2,{11,24}}, {2,{14,27}}, {3,{16,32,41}}, {2,{17,26}}, {3,{21,36,42}} } },
   {5, {{2,{11,24}}, {2,{14,27}}, {3,{15,32,41}}, {2,{17,25}}, {3,{21,35,42}} } },
   {3, {{4,{11,24,45,52}}, {3,{15,47,66}}, {5,{16,21,42,54,67}} } },
   {3, {{4,{11,24,46,52}}, {3,{16,47,65}}, {5,{15,21,42,54,67}} } },
   {3, {{4,{11,24,45,62}}, {3,{15,47,56}}, {5,{16,21,42,64,57}} } },
   {3, {{4,{11,24,46,62}}, {3,{16,47,55}}, {5,{15,21,42,64,57}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,32}}, {2,{21,42}}, {2,{22,37}}, {2,{27,34}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {3,{14,27,58}}, {3,{18,22,57}}, {2,{21,42}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {3,{14,27,59}}, {3,{19,22,57}}, {2,{21,42}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {3,{14,27,68}}, {3,{18,22,67}}, {2,{21,42}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {3,{14,27,69}}, {3,{19,22,67}}, {2,{21,42}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,37}}, {2,{17,32}}, {2,{21,42}}, {2,{22,34}} } },
   {5, {{3,{11,24,37}}, {2,{12,41}}, {2,{15,32}}, {2,{17,35}}, {3,{21,34,42}} } },
   {5, {{3,{11,24,37}}, {2,{12,41}}, {2,{16,32}}, {2,{17,36}}, {3,{21,34,42}} } },
   {4, {{2,{11,24}}, {4,{14,27,48,62}}, {4,{18,21,42,69}}, {2,{29,67}} } },
   {4, {{2,{11,24}}, {4,{14,27,49,62}}, {4,{19,21,42,68}}, {2,{28,67}} } },
   {4, {{2,{11,24}}, {4,{14,27,48,52}}, {4,{18,21,42,59}}, {2,{29,57}} } },
   {4, {{2,{11,24}}, {4,{14,27,49,52}}, {4,{19,21,42,58}}, {2,{28,57}} } },
   {5, {{2,{11,24}}, {3,{12,37,41}}, {2,{14,31}}, {3,{17,22,34}}, {2,{21,42}} } },
   {5, {{2,{11,24}}, {2,{13,61}}, {3,{14,22,43}}, {3,{21,42,67}}, {2,{47,63}} } },
   {5, {{2,{11,24}}, {2,{13,51}}, {3,{14,22,43}}, {3,{21,42,57}}, {2,{47,53}} } },
   {5, {{3,{11,24,37}}, {2,{12,43}}, {2,{13,41}}, {2,{17,32}}, {3,{21,34,42}} } },
   {4, {{2,{11,24}}, {3,{14,27,68}}, {3,{18,32,41}}, {4,{21,38,42,67}} } },
   {4, {{2,{11,24}}, {3,{14,27,69}}, {3,{19,32,41}}, {4,{21,39,42,67}} } },
   {4, {{2,{11,24}}, {3,{14,27,58}}, {3,{18,32,41}}, {4,{21,38,42,57}} } },
   {4, {{2,{11,24}}, {3,{14,27,59}}, {3,{19,32,41}}, {4,{21,39,42,57}} } },
   {4, {{3,{11,24,65}}, {3,{15,46,67}}, {4,{16,21,42,64}}, {2,{47,62}} } },
   {4, {{3,{11,24,66}}, {3,{16,45,67}}, {4,{15,21,42,64}}, {2,{47,62}} } },
   {4, {{3,{11,24,55}}, {3,{15,46,57}}, {4,{16,21,42,54}}, {2,{47,52}} } },
   {4, {{3,{11,24,56}}, {3,{16,45,57}}, {4,{15,21,42,54}}, {2,{47,52}} } },
   {4, {{2,{11,24}}, {4,{14,32,51,67}}, {4,{21,34,42,57}}, {2,{47,62}} } },
   {4, {{2,{11,24}}, {4,{14,32,61,57}}, {4,{21,34,42,67}}, {2,{47,52}} } },
   {3, {{4,{11,24,45,52}}, {4,{15,27,49,54}}, {4,{17,21,42,59}} } },
   {3, {{4,{11,24,45,52}}, {4,{15,27,48,54}}, {4,{17,21,42,58}} } },
   {3, {{4,{11,24,46,52}}, {4,{16,27,49,54}}, {4,{17,21,42,59}} } },
   {3, {{4,{11,24,46,52}}, {4,{16,27,48,54}}, {4,{17,21,42,58}} } },
   {3, {{4,{11,24,45,62}}, {4,{15,27,49,64}}, {4,{17,21,42,69}} } },
   {3, {{4,{11,24,45,62}}, {4,{15,27,48,64}}, {4,{17,21,42,68}} } },
   {3, {{4,{11,24,46,62}}, {4,{16,27,49,64}}, {4,{17,21,42,69}} } },
   {3, {{4,{11,24,46,62}}, {4,{16,27,48,64}}, {4,{17,21,42,68}} } },
   {3, {{2,{11,24}}, {5,{14,32,45,51,67}}, {5,{21,34,42,57,65}} } },
   {3, {{2,{11,24}}, {5,{14,32,46,51,67}}, {5,{21,34,42,57,66}} } },
   {3, {{2,{11,24}}, {5,{14,32,45,61,57}}, {5,{21,34,42,67,55}} } },
   {3, {{2,{11,24}}, {5,{14,32,46,61,57}}, {5,{21,34,42,67,56}} } },
   {4, {{3,{11,24,46}}, {4,{16,21,42,65}}, {3,{25,44,67}}, {2,{47,62}} } },
   {4, {{3,{11,24,45}}, {4,{15,21,42,66}}, {3,{26,44,67}}, {2,{47,62}} } },
   {4, {{3,{11,24,46}}, {4,{16,21,42,55}}, {3,{25,44,57}}, {2,{47,52}} } },
   {4, {{3,{11,24,45}}, {4,{15,21,42,56}}, {3,{26,44,57}}, {2,{47,52}} } },
   {4, {{4,{11,24,45,57}}, {4,{15,21,42,56}}, {2,{26,54}}, {2,{47,52}} } },
   {4, {{4,{11,24,46,57}}, {4,{16,21,42,55}}, {2,{25,54}}, {2,{47,52}} } },
   {4, {{4,{11,24,45,67}}, {4,{15,21,42,66}}, {2,{26,64}}, {2,{47,62}} } },
   {4, {{4,{11,24,46,67}}, {4,{16,21,42,65}}, {2,{25,64}}, {2,{47,62}} } },
   {3, {{3,{11,24,56}}, {5,{16,21,42,57,65}}, {4,{25,47,54,62}} } },
   {3, {{3,{11,24,55}}, {5,{15,21,42,57,66}}, {4,{26,47,54,62}} } },
   {3, {{3,{11,24,66}}, {5,{16,21,42,67,55}}, {4,{25,47,64,52}} } },
   {3, {{3,{11,24,65}}, {5,{15,21,42,67,56}}, {4,{26,47,64,52}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,37}}, {2,{17,22}}, {2,{21,42}}, {2,{27,34}} } },
   {6, {{2,{11,24}}, {2,{12,37}}, {2,{14,22}}, {2,{17,31}}, {2,{21,42}}, {2,{32,41}} } },
   {4, {{2,{11,24}}, {4,{14,32,47,51}}, {4,{21,34,42,67}}, {2,{57,61}} } },
   {4, {{2,{11,24}}, {4,{14,32,47,61}}, {4,{21,34,42,57}}, {2,{67,51}} } },
   {5, {{2,{11,24}}, {2,{14,39}}, {3,{19,32,41}}, {3,{21,37,42}}, {2,{27,34}} } },
   {5, {{2,{11,24}}, {2,{14,38}}, {3,{18,32,41}}, {3,{21,37,42}}, {2,{27,34}} } },
   {3, {{4,{11,24,37,45}}, {3,{12,44,61}}, {5,{17,21,35,42,64}} } },
   {3, {{4,{11,24,37,46}}, {3,{12,44,61}}, {5,{17,21,36,42,64}} } },
   {3, {{4,{11,24,37,45}}, {3,{12,44,51}}, {5,{17,21,35,42,54}} } },
   {3, {{4,{11,24,37,46}}, {3,{12,44,51}}, {5,{17,21,36,42,54}} } },
   {5, {{2,{11,24}}, {3,{14,33,41}}, {2,{21,42}}, {3,{22,37,43}}, {2,{27,34}} } },
   {4, {{4,{11,24,37,65}}, {2,{12,41}}, {4,{17,22,35,64}}, {2,{21,42}} } },
   {4, {{4,{11,24,37,66}}, {2,{12,41}}, {4,{17,22,36,64}}, {2,{21,42}} } },
   {4, {{4,{11,24,37,55}}, {2,{12,41}}, {4,{17,22,35,54}}, {2,{21,42}} } },
   {4, {{4,{11,24,37,56}}, {2,{12,41}}, {4,{17,22,36,54}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {3,{13,41,67}}, {4,{14,32,47,63}}, {3,{21,34,42}} } },
   {4, {{2,{11,24}}, {3,{13,41,57}}, {4,{14,32,47,53}}, {3,{21,34,42}} } },
   {5, {{2,{11,24}}, {3,{12,47,51}}, {2,{14,22}}, {3,{21,42,67}}, {2,{57,61}} } },
   {5, {{2,{11,24}}, {3,{12,47,61}}, {2,{14,22}}, {3,{21,42,57}}, {2,{67,51}} } },
   {2, {{6,{11,24,37,49,52,65}}, {6,{19,21,35,42,57,64}} } },
   {2, {{6,{11,24,37,48,52,65}}, {6,{18,21,35,42,57,64}} } },
   {2, {{6,{11,24,37,49,52,66}}, {6,{19,21,36,42,57,64}} } },
   {2, {{6,{11,24,37,48,52,66}}, {6,{18,21,36,42,57,64}} } },
   {2, {{6,{11,24,37,49,62,55}}, {6,{19,21,35,42,67,54}} } },
   {2, {{6,{11,24,37,48,62,55}}, {6,{18,21,35,42,67,54}} } },
   {2, {{6,{11,24,37,49,62,56}}, {6,{19,21,36,42,67,54}} } },
   {2, {{6,{11,24,37,48,62,56}}, {6,{18,21,36,42,67,54}} } },
   {4, {{3,{11,24,46}}, {4,{16,22,37,44}}, {3,{17,32,41}}, {2,{21,42}} } },
   {4, {{3,{11,24,45}}, {4,{15,22,37,44}}, {3,{17,32,41}}, {2,{21,42}} } },
   {4, {{3,{11,24,37}}, {4,{17,32,44,51}}, {3,{21,42,54}}, {2,{22,34}} } },
   {4, {{3,{11,24,37}}, {4,{17,32,44,61}}, {3,{21,42,64}}, {2,{22,34}} } },
   {4, {{2,{11,24}}, {3,{12,49,51}}, {3,{14,27,32}}, {4,{21,37,42,59}} } },
   {4, {{2,{11,24}}, {3,{12,48,51}}, {3,{14,27,32}}, {4,{21,37,42,58}} } },
   {4, {{2,{11,24}}, {3,{12,49,61}}, {3,{14,27,32}}, {4,{21,37,42,69}} } },
   {4, {{2,{11,24}}, {3,{12,48,61}}, {3,{14,27,32}}, {4,{21,37,42,68}} } },
   {4, {{2,{11,24}}, {4,{14,32,45,61}}, {4,{21,37,42,65}}, {2,{27,34}} } },
   {4, {{2,{11,24}}, {4,{14,32,46,61}}, {4,{21,37,42,66}}, {2,{27,34}} } },
   {4, {{2,{11,24}}, {4,{14,32,45,51}}, {4,{21,37,42,55}}, {2,{27,34}} } },
   {4, {{2,{11,24}}, {4,{14,32,46,51}}, {4,{21,37,42,56}}, {2,{27,34}} } },
   {4, {{4,{11,24,39,57}}, {2,{12,41}}, {3,{17,32,59}}, {3,{21,34,42}} } },
   {4, {{4,{11,24,38,57}}, {2,{12,41}}, {3,{17,32,58}}, {3,{21,34,42}} } },
   {4, {{4,{11,24,39,67}}, {2,{12,41}}, {3,{17,32,69}}, {3,{21,34,42}} } },
   {4, {{4,{11,24,38,67}}, {2,{12,41}}, {3,{17,32,68}}, {3,{21,34,42}} } },
   {4, {{2,{11,24}}, {4,{14,22,37,59}}, {4,{19,32,41,57}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {4,{14,22,37,58}}, {4,{18,32,41,57}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {4,{14,22,37,69}}, {4,{19,32,41,67}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {4,{14,22,37,68}}, {4,{18,32,41,67}}, {2,{21,42}} } },
   {6, {{2,{11,24}}, {2,{12,61}}, {2,{14,22}}, {2,{21,42}}, {2,{41,67}}, {2,{47,62}} } },
   {6, {{2,{11,24}}, {2,{12,51}}, {2,{14,22}}, {2,{21,42}}, {2,{41,57}}, {2,{47,52}} } },
   {4, {{3,{11,24,65}}, {4,{15,21,42,54}}, {2,{45,67}}, {3,{47,52,64}} } },
   {4, {{3,{11,24,66}}, {4,{16,21,42,54}}, {2,{46,67}}, {3,{47,52,64}} } },
   {4, {{3,{11,24,55}}, {4,{15,21,42,64}}, {2,{45,57}}, {3,{47,62,54}} } },
   {4, {{3,{11,24,56}}, {4,{16,21,42,64}}, {2,{46,57}}, {3,{47,62,54}} } },
   {4, {{4,{11,24,46,67}}, {4,{16,21,42,54}}, {2,{47,52}}, {2,{57,64}} } },
   {4, {{4,{11,24,45,67}}, {4,{15,21,42,54}}, {2,{47,52}}, {2,{57,64}} } },
   {4, {{4,{11,24,46,57}}, {4,{16,21,42,64}}, {2,{47,62}}, {2,{67,54}} } },
   {4, {{4,{11,24,45,57}}, {4,{15,21,42,64}}, {2,{47,62}}, {2,{67,54}} } },
   {4, {{4,{11,24,46,67}}, {4,{16,21,42,64}}, {2,{45,62}}, {2,{47,65}} } },
   {4, {{4,{11,24,45,67}}, {4,{15,21,42,64}}, {2,{46,62}}, {2,{47,66}} } },
   {4, {{4,{11,24,46,57}}, {4,{16,21,42,54}}, {2,{45,52}}, {2,{47,55}} } },
   {4, {{4,{11,24,45,57}}, {4,{15,21,42,54}}, {2,{46,52}}, {2,{47,56}} } },
   {4, {{2,{11,24}}, {4,{12,47,51,64}}, {2,{14,22}}, {4,{21,42,54,67}} } },
   {4, {{2,{11,24}}, {4,{12,47,61,54}}, {2,{14,22}}, {4,{21,42,64,57}} } },
   {4, {{2,{11,24}}, {2,{14,37}}, {4,{17,32,48,61}}, {4,{21,34,42,68}} } },
   {4, {{2,{11,24}}, {2,{14,37}}, {4,{17,32,49,61}}, {4,{21,34,42,69}} } },
   {4, {{2,{11,24}}, {2,{14,37}}, {4,{17,32,48,51}}, {4,{21,34,42,58}} } },
   {4, {{2,{11,24}}, {2,{14,37}}, {4,{17,32,49,51}}, {4,{21,34,42,59}} } },
   {4, {{4,{11,24,45,53}}, {4,{15,21,42,54}}, {2,{43,57}}, {2,{47,52}} } },
   {4, {{4,{11,24,46,53}}, {4,{16,21,42,54}}, {2,{43,57}}, {2,{47,52}} } },
   {4, {{4,{11,24,45,63}}, {4,{15,21,42,64}}, {2,{43,67}}, {2,{47,62}} } },
   {4, {{4,{11,24,46,63}}, {4,{16,21,42,64}}, {2,{43,67}}, {2,{47,62}} } },
   {4, {{2,{11,24}}, {2,{14,29}}, {4,{19,32,41,67}}, {4,{21,37,42,69}} } },
   {4, {{2,{11,24}}, {2,{14,28}}, {4,{18,32,41,67}}, {4,{21,37,42,68}} } },
   {4, {{2,{11,24}}, {2,{14,29}}, {4,{19,32,41,57}}, {4,{21,37,42,59}} } },
   {4, {{2,{11,24}}, {2,{14,28}}, {4,{18,32,41,57}}, {4,{21,37,42,58}} } },
   {3, {{2,{11,24}}, {5,{14,32,47,51,66}}, {5,{21,36,42,57,64}} } },
   {3, {{2,{11,24}}, {5,{14,32,47,51,65}}, {5,{21,35,42,57,64}} } },
   {3, {{2,{11,24}}, {5,{14,32,47,61,56}}, {5,{21,36,42,67,54}} } },
   {3, {{2,{11,24}}, {5,{14,32,47,61,55}}, {5,{21,35,42,67,54}} } },
   {4, {{2,{11,24}}, {4,{12,47,51,65}}, {2,{14,22}}, {4,{21,42,55,67}} } },
   {4, {{2,{11,24}}, {4,{12,47,51,66}}, {2,{14,22}}, {4,{21,42,56,67}} } },
   {4, {{2,{11,24}}, {4,{12,47,61,55}}, {2,{14,22}}, {4,{21,42,65,57}} } },
   {4, {{2,{11,24}}, {4,{12,47,61,56}}, {2,{14,22}}, {4,{21,42,66,57}} } },
   {5, {{2,{11,24}}, {2,{12,27}}, {2,{14,22}}, {3,{17,32,41}}, {3,{21,37,42}} } },
   {4, {{3,{11,24,46}}, {4,{16,22,44,51}}, {3,{21,42,57}}, {2,{47,56}} } },
   {4, {{3,{11,24,45}}, {4,{15,22,44,51}}, {3,{21,42,57}}, {2,{47,55}} } },
   {4, {{3,{11,24,46}}, {4,{16,22,44,61}}, {3,{21,42,67}}, {2,{47,66}} } },
   {4, {{3,{11,24,45}}, {4,{15,22,44,61}}, {3,{21,42,67}}, {2,{47,65}} } },
   {3, {{5,{11,24,37,45,59}}, {4,{15,21,42,54}}, {3,{39,47,52}} } },
   {3, {{5,{11,24,37,45,58}}, {4,{15,21,42,54}}, {3,{38,47,52}} } },
   {3, {{5,{11,24,37,46,59}}, {4,{16,21,42,54}}, {3,{39,47,52}} } },
   {3, {{5,{11,24,37,46,58}}, {4,{16,21,42,54}}, {3,{38,47,52}} } },
   {3, {{5,{11,24,37,45,69}}, {4,{15,21,42,64}}, {3,{39,47,62}} } },
   {3, {{5,{11,24,37,45,68}}, {4,{15,21,42,64}}, {3,{38,47,62}} } },
   {3, {{5,{11,24,37,46,69}}, {4,{16,21,42,64}}, {3,{39,47,62}} } },
   {3, {{5,{11,24,37,46,68}}, {4,{16,21,42,64}}, {3,{38,47,62}} } },
   {5, {{2,{11,24}}, {3,{12,37,43}}, {2,{14,22}}, {3,{17,33,41}}, {2,{21,42}} } },
   {4, {{3,{11,24,46}}, {3,{16,27,54}}, {4,{17,21,42,56}}, {2,{44,52}} } },
   {4, {{3,{11,24,45}}, {3,{15,27,54}}, {4,{17,21,42,55}}, {2,{44,52}} } },
   {4, {{3,{11,24,46}}, {3,{16,27,64}}, {4,{17,21,42,66}}, {2,{44,62}} } },
   {4, {{3,{11,24,45}}, {3,{15,27,64}}, {4,{17,21,42,65}}, {2,{44,62}} } },
   {4, {{4,{11,24,47,65}}, {3,{12,45,61}}, {2,{14,22}}, {3,{21,42,67}} } },
   {4, {{4,{11,24,47,66}}, {3,{12,46,61}}, {2,{14,22}}, {3,{21,42,67}} } },
   {4, {{4,{11,24,47,55}}, {3,{12,45,51}}, {2,{14,22}}, {3,{21,42,57}} } },
   {4, {{4,{11,24,47,56}}, {3,{12,46,51}}, {2,{14,22}}, {3,{21,42,57}} } },
   {3, {{4,{11,24,47,62}}, {4,{14,32,51,67}}, {4,{21,34,42,57}} } },
   {3, {{4,{11,24,47,52}}, {4,{14,32,61,57}}, {4,{21,34,42,67}} } },
   {5, {{2,{11,24}}, {3,{14,33,41}}, {3,{21,37,42}}, {2,{27,34}}, {2,{32,43}} } },
   {4, {{2,{11,24}}, {2,{14,27}}, {4,{17,32,44,51}}, {4,{21,37,42,54}} } },
   {4, {{2,{11,24}}, {2,{14,27}}, {4,{17,32,44,61}}, {4,{21,37,42,64}} } },
   {4, {{2,{11,24}}, {2,{14,27}}, {4,{17,31,48,52}}, {4,{21,38,42,57}} } },
   {4, {{2,{11,24}}, {2,{14,27}}, {4,{17,31,49,52}}, {4,{21,39,42,57}} } },
   {4, {{2,{11,24}}, {2,{14,27}}, {4,{17,31,48,62}}, {4,{21,38,42,67}} } },
   {4, {{2,{11,24}}, {2,{14,27}}, {4,{17,31,49,62}}, {4,{21,39,42,67}} } },
   {3, {{3,{11,24,56}}, {5,{16,32,47,54,61}}, {4,{21,34,42,67}} } },
   {3, {{3,{11,24,55}}, {5,{15,32,47,54,61}}, {4,{21,34,42,67}} } },
   {3, {{3,{11,24,66}}, {5,{16,32,47,64,51}}, {4,{21,34,42,57}} } },
   {3, {{3,{11,24,65}}, {5,{15,32,47,64,51}}, {4,{21,34,42,57}} } },
   },
   {
   {5, {{2,{11,24}}, {2,{12,41}}, {2,{14,27}}, {2,{17,22}}, {2,{21,42}} } },
   {4, {{3,{11,24,37}}, {2,{12,41}}, {3,{17,22,34}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {3,{12,47,61}}, {2,{14,22}}, {3,{21,42,67}} } },
   {4, {{2,{11,24}}, {3,{12,47,51}}, {2,{14,22}}, {3,{21,42,57}} } },
   {4, {{2,{11,24}}, {2,{12,41}}, {3,{14,27,32}}, {3,{21,37,42}} } },
   {3, {{2,{11,24}}, {4,{14,32,47,61}}, {4,{21,34,42,67}} } },
   {3, {{2,{11,24}}, {4,{14,32,47,51}}, {4,{21,34,42,57}} } },
   {4, {{2,{11,24}}, {3,{14,22,37}}, {3,{17,32,41}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {3,{14,32,41}}, {3,{21,37,42}}, {2,{27,34}} } },
   {4, {{2,{11,24}}, {2,{14,37}}, {3,{17,32,41}}, {3,{21,34,42}} } },
   {5, {{2,{11,24}}, {2,{13,41}}, {2,{14,27}}, {3,{17,22,43}}, {2,{21,42}} } },
   {5, {{2,{11,24}}, {3,{14,22,37}}, {2,{17,31}}, {2,{21,42}}, {2,{32,41}} } },
   {5, {{2,{11,24}}, {3,{14,32,41}}, {2,{21,42}}, {2,{22,37}}, {2,{27,34}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {2,{14,32}}, {3,{21,37,42}}, {2,{27,34}} } },
   {4, {{3,{11,24,37}}, {2,{13,41}}, {4,{17,22,34,43}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {3,{12,47,61}}, {2,{14,32}}, {4,{21,34,42,67}} } },
   {4, {{2,{11,24}}, {3,{12,47,51}}, {2,{14,32}}, {4,{21,34,42,57}} } },
   {5, {{2,{11,24}}, {2,{14,27}}, {3,{17,32,41}}, {2,{21,42}}, {2,{22,37}} } },
   {3, {{4,{11,24,37,59}}, {4,{19,32,41,57}}, {3,{21,34,42}} } },
   {3, {{4,{11,24,37,58}}, {4,{18,32,41,57}}, {3,{21,34,42}} } },
   {3, {{4,{11,24,37,69}}, {4,{19,32,41,67}}, {3,{21,34,42}} } },
   {3, {{4,{11,24,37,68}}, {4,{18,32,41,67}}, {3,{21,34,42}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {2,{14,27}}, {3,{17,26,32}}, {3,{21,36,42}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {2,{14,27}}, {3,{17,25,32}}, {3,{21,35,42}} } },
   {6, {{2,{11,24}}, {2,{12,43}}, {2,{13,41}}, {2,{14,27}}, {2,{17,22}}, {2,{21,42}} } },
   {4, {{3,{11,24,37}}, {4,{12,28,34,41}}, {2,{17,32}}, {3,{21,38,42}} } },
   {4, {{3,{11,24,37}}, {4,{12,29,34,41}}, {2,{17,32}}, {3,{21,39,42}} } },
   {4, {{2,{11,24}}, {4,{14,31,45,57}}, {4,{21,35,42,54}}, {2,{47,52}} } },
   {4, {{2,{11,24}}, {4,{14,31,46,57}}, {4,{21,36,42,54}}, {2,{47,52}} } },
   {4, {{2,{11,24}}, {4,{14,31,45,67}}, {4,{21,35,42,64}}, {2,{47,62}} } },
   {4, {{2,{11,24}}, {4,{14,31,46,67}}, {4,{21,36,42,64}}, {2,{47,62}} } },
   {2, {{6,{11,24,37,46,59,62}}, {6,{19,21,36,42,54,67}} } },
   {2, {{6,{11,24,37,46,58,62}}, {6,{18,21,36,42,54,67}} } },
   {2, {{6,{11,24,37,45,59,62}}, {6,{19,21,35,42,54,67}} } },
   {2, {{6,{11,24,37,45,58,62}}, {6,{18,21,35,42,54,67}} } },
   {2, {{6,{11,24,37,46,69,52}}, {6,{19,21,36,42,64,57}} } },
   {2, {{6,{11,24,37,46,68,52}}, {6,{18,21,36,42,64,57}} } },
   {2, {{6,{11,24,37,45,69,52}}, {6,{19,21,35,42,64,57}} } },
   {2, {{6,{11,24,37,45,68,52}}, {6,{18,21,35,42,64,57}} } },
   {3, {{4,{11,24,46,57}}, {4,{16,32,47,51}}, {4,{21,36,42,54}} } },
   {3, {{4,{11,24,45,57}}, {4,{15,32,47,51}}, {4,{21,35,42,54}} } },
   {3, {{4,{11,24,46,67}}, {4,{16,32,47,61}}, {4,{21,36,42,64}} } },
   {3, {{4,{11,24,45,67}}, {4,{15,32,47,61}}, {4,{21,35,42,64}} } },
   {3, {{2,{11,24}}, {5,{14,29,47,52,66}}, {5,{17,21,42,56,69}} } },
   {3, {{2,{11,24}}, {5,{14,28,47,52,66}}, {5,{17,21,42,56,68}} } },
   {3, {{2,{11,24}}, {5,{14,29,47,52,65}}, {5,{17,21,42,55,69}} } },
   {3, {{2,{11,24}}, {5,{14,28,47,52,65}}, {5,{17,21,42,55,68}} } },
   {3, {{2,{11,24}}, {5,{14,29,47,62,56}}, {5,{17,21,42,66,59}} } },
   {3, {{2,{11,24}}, {5,{14,28,47,62,56}}, {5,{17,21,42,66,58}} } },
   {3, {{2,{11,24}}, {5,{14,29,47,62,55}}, {5,{17,21,42,65,59}} } },
   {3, {{2,{11,24}}, {5,{14,28,47,62,55}}, {5,{17,21,42,65,58}} } },
   {5, {{3,{11,24,38}}, {2,{12,41}}, {2,{18,32}}, {3,{21,37,42}}, {2,{27,34}} } },
   {5, {{3,{11,24,39}}, {2,{12,41}}, {2,{19,32}}, {3,{21,37,42}}, {2,{27,34}} } },
   {5, {{3,{11,24,37}}, {2,{12,43}}, {2,{13,41}}, {3,{17,22,34}}, {2,{21,42}} } },
   {3, {{4,{11,24,46,52}}, {4,{16,27,49,54}}, {4,{19,21,42,57}} } },
   {3, {{4,{11,24,46,52}}, {4,{16,27,48,54}}, {4,{18,21,42,57}} } },
   {3, {{4,{11,24,45,52}}, {4,{15,27,49,54}}, {4,{19,21,42,57}} } },
   {3, {{4,{11,24,45,52}}, {4,{15,27,48,54}}, {4,{18,21,42,57}} } },
   {3, {{4,{11,24,46,62}}, {4,{16,27,49,64}}, {4,{19,21,42,67}} } },
   {3, {{4,{11,24,46,62}}, {4,{16,27,48,64}}, {4,{18,21,42,67}} } },
   {3, {{4,{11,24,45,62}}, {4,{15,27,49,64}}, {4,{19,21,42,67}} } },
   {3, {{4,{11,24,45,62}}, {4,{15,27,48,64}}, {4,{18,21,42,67}} } },
   {5, {{2,{11,24}}, {2,{13,61}}, {4,{14,22,47,63}}, {2,{21,42}}, {2,{41,67}} } },
   {5, {{2,{11,24}}, {2,{13,51}}, {4,{14,22,47,53}}, {2,{21,42}}, {2,{41,57}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,27}}, {2,{16,22}}, {2,{17,26}}, {2,{21,42}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,27}}, {2,{15,22}}, {2,{17,25}}, {2,{21,42}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,27}}, {2,{17,32}}, {2,{21,42}}, {2,{22,37}} } },
   {5, {{2,{11,24}}, {3,{14,22,37}}, {3,{17,33,41}}, {2,{21,42}}, {2,{32,43}} } },
   {5, {{2,{11,24}}, {3,{12,44,51}}, {2,{14,27}}, {2,{17,22}}, {3,{21,42,54}} } },
   {5, {{2,{11,24}}, {3,{12,44,61}}, {2,{14,27}}, {2,{17,22}}, {3,{21,42,64}} } },
   {5, {{2,{11,24}}, {2,{13,41}}, {2,{14,27}}, {3,{17,32,43}}, {3,{21,37,42}} } },
   {5, {{3,{11,24,46}}, {2,{12,41}}, {3,{16,27,44}}, {2,{17,22}}, {2,{21,42}} } },
   {5, {{3,{11,24,45}}, {2,{12,41}}, {3,{15,27,44}}, {2,{17,22}}, {2,{21,42}} } },
   {3, {{4,{11,24,47,55}}, {4,{14,32,45,51}}, {4,{21,34,42,57}} } },
   {3, {{4,{11,24,47,56}}, {4,{14,32,46,51}}, {4,{21,34,42,57}} } },
   {3, {{4,{11,24,47,65}}, {4,{14,32,45,61}}, {4,{21,34,42,67}} } },
   {3, {{4,{11,24,47,66}}, {4,{14,32,46,61}}, {4,{21,34,42,67}} } },
   {3, {{5,{11,24,47,52,65}}, {2,{15,66}}, {5,{16,21,42,54,67}} } },
   {3, {{5,{11,24,47,52,66}}, {2,{16,65}}, {5,{15,21,42,54,67}} } },
   {3, {{5,{11,24,47,62,55}}, {2,{15,56}}, {5,{16,21,42,64,57}} } },
   {3, {{5,{11,24,47,62,56}}, {2,{16,55}}, {5,{15,21,42,64,57}} } },
   {5, {{2,{11,24}}, {2,{14,27}}, {3,{16,32,41}}, {2,{17,26}}, {3,{21,36,42}} } },
   {5, {{2,{11,24}}, {2,{14,27}}, {3,{15,32,41}}, {2,{17,25}}, {3,{21,35,42}} } },
   {3, {{4,{11,24,45,52}}, {3,{15,47,66}}, {5,{16,21,42,54,67}} } },
   {3, {{4,{11,24,46,52}}, {3,{16,47,65}}, {5,{15,21,42,54,67}} } },
   {3, {{4,{11,24,45,62}}, {3,{15,47,56}}, {5,{16,21,42,64,57}} } },
   {3, {{4,{11,24,46,62}}, {3,{16,47,55}}, {5,{15,21,42,64,57}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,32}}, {2,{21,42}}, {2,{22,37}}, {2,{27,34}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {3,{14,27,58}}, {3,{18,22,57}}, {2,{21,42}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {3,{14,27,59}}, {3,{19,22,57}}, {2,{21,42}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {3,{14,27,68}}, {3,{18,22,67}}, {2,{21,42}} } },
   {5, {{2,{11,24}}, {2,{12,41}}, {3,{14,27,69}}, {3,{19,22,67}}, {2,{21,42}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,37}}, {2,{17,32}}, {2,{21,42}}, {2,{22,34}} } },
   {5, {{3,{11,24,37}}, {2,{12,41}}, {2,{15,32}}, {2,{17,35}}, {3,{21,34,42}} } },
   {5, {{3,{11,24,37}}, {2,{12,41}}, {2,{16,32}}, {2,{17,36}}, {3,{21,34,42}} } },
   {4, {{2,{11,24}}, {4,{14,27,48,62}}, {4,{18,21,42,69}}, {2,{29,67}} } },
   {4, {{2,{11,24}}, {4,{14,27,49,62}}, {4,{19,21,42,68}}, {2,{28,67}} } },
   {4, {{2,{11,24}}, {4,{14,27,48,52}}, {4,{18,21,42,59}}, {2,{29,57}} } },
   {4, {{2,{11,24}}, {4,{14,27,49,52}}, {4,{19,21,42,58}}, {2,{28,57}} } },
   {5, {{2,{11,24}}, {3,{12,37,41}}, {2,{14,31}}, {3,{17,22,34}}, {2,{21,42}} } },
   {5, {{2,{11,24}}, {2,{13,61}}, {3,{14,22,43}}, {3,{21,42,67}}, {2,{47,63}} } },
   {5, {{2,{11,24}}, {2,{13,51}}, {3,{14,22,43}}, {3,{21,42,57}}, {2,{47,53}} } },
   {5, {{3,{11,24,37}}, {2,{12,43}}, {2,{13,41}}, {2,{17,32}}, {3,{21,34,42}} } },
   {4, {{2,{11,24}}, {3,{14,27,68}}, {3,{18,32,41}}, {4,{21,38,42,67}} } },
   {4, {{2,{11,24}}, {3,{14,27,69}}, {3,{19,32,41}}, {4,{21,39,42,67}} } },
   {4, {{2,{11,24}}, {3,{14,27,58}}, {3,{18,32,41}}, {4,{21,38,42,57}} } },
   {4, {{2,{11,24}}, {3,{14,27,59}}, {3,{19,32,41}}, {4,{21,39,42,57}} } },
   {4, {{3,{11,24,65}}, {3,{15,46,67}}, {4,{16,21,42,64}}, {2,{47,62}} } },
   {4, {{3,{11,24,66}}, {3,{16,45,67}}, {4,{15,21,42,64}}, {2,{47,62}} } },
   {4, {{3,{11,24,55}}, {3,{15,46,57}}, {4,{16,21,42,54}}, {2,{47,52}} } },
   {4, {{3,{11,24,56}}, {3,{16,45,57}}, {4,{15,21,42,54}}, {2,{47,52}} } },
   {4, {{2,{11,24}}, {4,{14,32,51,67}}, {4,{21,34,42,57}}, {2,{47,62}} } },
   {4, {{2,{11,24}}, {4,{14,32,61,57}}, {4,{21,34,42,67}}, {2,{47,52}} } },
   {3, {{4,{11,24,45,52}}, {4,{15,27,49,54}}, {4,{17,21,42,59}} } },
   {3, {{4,{11,24,45,52}}, {4,{15,27,48,54}}, {4,{17,21,42,58}} } },
   {3, {{4,{11,24,46,52}}, {4,{16,27,49,54}}, {4,{17,21,42,59}} } },
   {3, {{4,{11,24,46,52}}, {4,{16,27,48,54}}, {4,{17,21,42,58}} } },
   {3, {{4,{11,24,45,62}}, {4,{15,27,49,64}}, {4,{17,21,42,69}} } },
   {3, {{4,{11,24,45,62}}, {4,{15,27,48,64}}, {4,{17,21,42,68}} } },
   {3, {{4,{11,24,46,62}}, {4,{16,27,49,64}}, {4,{17,21,42,69}} } },
   {3, {{4,{11,24,46,62}}, {4,{16,27,48,64}}, {4,{17,21,42,68}} } },
   {3, {{2,{11,24}}, {5,{14,32,45,51,67}}, {5,{21,34,42,57,65}} } },
   {3, {{2,{11,24}}, {5,{14,32,46,51,67}}, {5,{21,34,42,57,66}} } },
   {3, {{2,{11,24}}, {5,{14,32,45,61,57}}, {5,{21,34,42,67,55}} } },
   {3, {{2,{11,24}}, {5,{14,32,46,61,57}}, {5,{21,34,42,67,56}} } },
   {4, {{3,{11,24,46}}, {4,{16,21,42,65}}, {3,{25,44,67}}, {2,{47,62}} } },
   {4, {{3,{11,24,45}}, {4,{15,21,42,66}}, {3,{26,44,67}}, {2,{47,62}} } },
   {4, {{3,{11,24,46}}, {4,{16,21,42,55}}, {3,{25,44,57}}, {2,{47,52}} } },
   {4, {{3,{11,24,45}}, {4,{15,21,42,56}}, {3,{26,44,57}}, {2,{47,52}} } },
   {4, {{4,{11,24,45,57}}, {4,{15,21,42,56}}, {2,{26,54}}, {2,{47,52}} } },
   {4, {{4,{11,24,46,57}}, {4,{16,21,42,55}}, {2,{25,54}}, {2,{47,52}} } },
   {4, {{4,{11,24,45,67}}, {4,{15,21,42,66}}, {2,{26,64}}, {2,{47,62}} } },
   {4, {{4,{11,24,46,67}}, {4,{16,21,42,65}}, {2,{25,64}}, {2,{47,62}} } },
   {3, {{3,{11,24,56}}, {5,{16,21,42,57,65}}, {4,{25,47,54,62}} } },
   {3, {{3,{11,24,55}}, {5,{15,21,42,57,66}}, {4,{26,47,54,62}} } },
   {3, {{3,{11,24,66}}, {5,{16,21,42,67,55}}, {4,{25,47,64,52}} } },
   {3, {{3,{11,24,65}}, {5,{15,21,42,67,56}}, {4,{26,47,64,52}} } },
   {6, {{2,{11,24}}, {2,{12,41}}, {2,{14,37}}, {2,{17,22}}, {2,{21,42}}, {2,{27,34}} } },
   {6, {{2,{11,24}}, {2,{12,37}}, {2,{14,22}}, {2,{17,31}}, {2,{21,42}}, {2,{32,41}} } },
   {4, {{2,{11,24}}, {4,{14,32,47,51}}, {4,{21,34,42,67}}, {2,{57,61}} } },
   {4, {{2,{11,24}}, {4,{14,32,47,61}}, {4,{21,34,42,57}}, {2,{67,51}} } },
   {5, {{2,{11,24}}, {2,{14,39}}, {3,{19,32,41}}, {3,{21,37,42}}, {2,{27,34}} } },
   {5, {{2,{11,24}}, {2,{14,38}}, {3,{18,32,41}}, {3,{21,37,42}}, {2,{27,34}} } },
   {3, {{4,{11,24,37,45}}, {3,{12,44,61}}, {5,{17,21,35,42,64}} } },
   {3, {{4,{11,24,37,46}}, {3,{12,44,61}}, {5,{17,21,36,42,64}} } },
   {3, {{4,{11,24,37,45}}, {3,{12,44,51}}, {5,{17,21,35,42,54}} } },
   {3, {{4,{11,24,37,46}}, {3,{12,44,51}}, {5,{17,21,36,42,54}} } },
   {5, {{2,{11,24}}, {3,{14,33,41}}, {2,{21,42}}, {3,{22,37,43}}, {2,{27,34}} } },
   {4, {{4,{11,24,37,65}}, {2,{12,41}}, {4,{17,22,35,64}}, {2,{21,42}} } },
   {4, {{4,{11,24,37,66}}, {2,{12,41}}, {4,{17,22,36,64}}, {2,{21,42}} } },
   {4, {{4,{11,24,37,55}}, {2,{12,41}}, {4,{17,22,35,54}}, {2,{21,42}} } },
   {4, {{4,{11,24,37,56}}, {2,{12,41}}, {4,{17,22,36,54}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {3,{13,41,67}}, {4,{14,32,47,63}}, {3,{21,34,42}} } },
   {4, {{2,{11,24}}, {3,{13,41,57}}, {4,{14,32,47,53}}, {3,{21,34,42}} } },
   {5, {{2,{11,24}}, {3,{12,47,51}}, {2,{14,22}}, {3,{21,42,67}}, {2,{57,61}} } },
   {5, {{2,{11,24}}, {3,{12,47,61}}, {2,{14,22}}, {3,{21,42,57}}, {2,{67,51}} } },
   {2, {{6,{11,24,37,49,52,65}}, {6,{19,21,35,42,57,64}} } },
   {2, {{6,{11,24,37,48,52,65}}, {6,{18,21,35,42,57,64}} } },
   {2, {{6,{11,24,37,49,52,66}}, {6,{19,21,36,42,57,64}} } },
   {2, {{6,{11,24,37,48,52,66}}, {6,{18,21,36,42,57,64}} } },
   {2, {{6,{11,24,37,49,62,55}}, {6,{19,21,35,42,67,54}} } },
   {2, {{6,{11,24,37,48,62,55}}, {6,{18,21,35,42,67,54}} } },
   {2, {{6,{11,24,37,49,62,56}}, {6,{19,21,36,42,67,54}} } },
   {2, {{6,{11,24,37,48,62,56}}, {6,{18,21,36,42,67,54}} } },
   {4, {{3,{11,24,46}}, {4,{16,22,37,44}}, {3,{17,32,41}}, {2,{21,42}} } },
   {4, {{3,{11,24,45}}, {4,{15,22,37,44}}, {3,{17,32,41}}, {2,{21,42}} } },
   {4, {{3,{11,24,37}}, {4,{17,32,44,51}}, {3,{21,42,54}}, {2,{22,34}} } },
   {4, {{3,{11,24,37}}, {4,{17,32,44,61}}, {3,{21,42,64}}, {2,{22,34}} } },
   {4, {{2,{11,24}}, {3,{12,49,51}}, {3,{14,27,32}}, {4,{21,37,42,59}} } },
   {4, {{2,{11,24}}, {3,{12,48,51}}, {3,{14,27,32}}, {4,{21,37,42,58}} } },
   {4, {{2,{11,24}}, {3,{12,49,61}}, {3,{14,27,32}}, {4,{21,37,42,69}} } },
   {4, {{2,{11,24}}, {3,{12,48,61}}, {3,{14,27,32}}, {4,{21,37,42,68}} } },
   {4, {{2,{11,24}}, {4,{14,32,45,61}}, {4,{21,37,42,65}}, {2,{27,34}} } },
   {4, {{2,{11,24}}, {4,{14,32,46,61}}, {4,{21,37,42,66}}, {2,{27,34}} } },
   {4, {{2,{11,24}}, {4,{14,32,45,51}}, {4,{21,37,42,55}}, {2,{27,34}} } },
   {4, {{2,{11,24}}, {4,{14,32,46,51}}, {4,{21,37,42,56}}, {2,{27,34}} } },
   {4, {{4,{11,24,39,57}}, {2,{12,41}}, {3,{17,32,59}}, {3,{21,34,42}} } },
   {4, {{4,{11,24,38,57}}, {2,{12,41}}, {3,{17,32,58}}, {3,{21,34,42}} } },
   {4, {{4,{11,24,39,67}}, {2,{12,41}}, {3,{17,32,69}}, {3,{21,34,42}} } },
   {4, {{4,{11,24,38,67}}, {2,{12,41}}, {3,{17,32,68}}, {3,{21,34,42}} } },
   {4, {{2,{11,24}}, {4,{14,22,37,59}}, {4,{19,32,41,57}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {4,{14,22,37,58}}, {4,{18,32,41,57}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {4,{14,22,37,69}}, {4,{19,32,41,67}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {4,{14,22,37,68}}, {4,{18,32,41,67}}, {2,{21,42}} } },
   {6, {{2,{11,24}}, {2,{12,61}}, {2,{14,22}}, {2,{21,42}}, {2,{41,67}}, {2,{47,62}} } },
   {6, {{2,{11,24}}, {2,{12,51}}, {2,{14,22}}, {2,{21,42}}, {2,{41,57}}, {2,{47,52}} } },
   {4, {{3,{11,24,65}}, {4,{15,21,42,54}}, {2,{45,67}}, {3,{47,52,64}} } },
   {4, {{3,{11,24,66}}, {4,{16,21,42,54}}, {2,{46,67}}, {3,{47,52,64}} } },
   {4, {{3,{11,24,55}}, {4,{15,21,42,64}}, {2,{45,57}}, {3,{47,62,54}} } },
   {4, {{3,{11,24,56}}, {4,{16,21,42,64}}, {2,{46,57}}, {3,{47,62,54}} } },
   {4, {{4,{11,24,46,67}}, {4,{16,21,42,54}}, {2,{47,52}}, {2,{57,64}} } },
   {4, {{4,{11,24,45,67}}, {4,{15,21,42,54}}, {2,{47,52}}, {2,{57,64}} } },
   {4, {{4,{11,24,46,57}}, {4,{16,21,42,64}}, {2,{47,62}}, {2,{67,54}} } },
   {4, {{4,{11,24,45,57}}, {4,{15,21,42,64}}, {2,{47,62}}, {2,{67,54}} } },
   {4, {{4,{11,24,46,67}}, {4,{16,21,42,64}}, {2,{45,62}}, {2,{47,65}} } },
   {4, {{4,{11,24,45,67}}, {4,{15,21,42,64}}, {2,{46,62}}, {2,{47,66}} } },
   {4, {{4,{11,24,46,57}}, {4,{16,21,42,54}}, {2,{45,52}}, {2,{47,55}} } },
   {4, {{4,{11,24,45,57}}, {4,{15,21,42,54}}, {2,{46,52}}, {2,{47,56}} } },
   {4, {{2,{11,24}}, {4,{12,47,51,64}}, {2,{14,22}}, {4,{21,42,54,67}} } },
   {4, {{2,{11,24}}, {4,{12,47,61,54}}, {2,{14,22}}, {4,{21,42,64,57}} } },
   {4, {{2,{11,24}}, {2,{14,37}}, {4,{17,32,48,61}}, {4,{21,34,42,68}} } },
   {4, {{2,{11,24}}, {2,{14,37}}, {4,{17,32,49,61}}, {4,{21,34,42,69}} } },
   {4, {{2,{11,24}}, {2,{14,37}}, {4,{17,32,48,51}}, {4,{21,34,42,58}} } },
   {4, {{2,{11,24}}, {2,{14,37}}, {4,{17,32,49,51}}, {4,{21,34,42,59}} } },
   {4, {{4,{11,24,45,53}}, {4,{15,21,42,54}}, {2,{43,57}}, {2,{47,52}} } },
   {4, {{4,{11,24,46,53}}, {4,{16,21,42,54}}, {2,{43,57}}, {2,{47,52}} } },
   {4, {{4,{11,24,45,63}}, {4,{15,21,42,64}}, {2,{43,67}}, {2,{47,62}} } },
   {4, {{4,{11,24,46,63}}, {4,{16,21,42,64}}, {2,{43,67}}, {2,{47,62}} } },
   {4, {{2,{11,24}}, {2,{14,29}}, {4,{19,32,41,67}}, {4,{21,37,42,69}} } },
   {4, {{2,{11,24}}, {2,{14,28}}, {4,{18,32,41,67}}, {4,{21,37,42,68}} } },
   {4, {{2,{11,24}}, {2,{14,29}}, {4,{19,32,41,57}}, {4,{21,37,42,59}} } },
   {4, {{2,{11,24}}, {2,{14,28}}, {4,{18,32,41,57}}, {4,{21,37,42,58}} } },
   {3, {{2,{11,24}}, {5,{14,32,47,51,66}}, {5,{21,36,42,57,64}} } },
   {3, {{2,{11,24}}, {5,{14,32,47,51,65}}, {5,{21,35,42,57,64}} } },
   {3, {{2,{11,24}}, {5,{14,32,47,61,56}}, {5,{21,36,42,67,54}} } },
   {3, {{2,{11,24}}, {5,{14,32,47,61,55}}, {5,{21,35,42,67,54}} } },
   {4, {{2,{11,24}}, {4,{12,47,51,65}}, {2,{14,22}}, {4,{21,42,55,67}} } },
   {4, {{2,{11,24}}, {4,{12,47,51,66}}, {2,{14,22}}, {4,{21,42,56,67}} } },
   {4, {{2,{11,24}}, {4,{12,47,61,55}}, {2,{14,22}}, {4,{21,42,65,57}} } },
   {4, {{2,{11,24}}, {4,{12,47,61,56}}, {2,{14,22}}, {4,{21,42,66,57}} } },
   {5, {{2,{11,24}}, {2,{12,27}}, {2,{14,22}}, {3,{17,32,41}}, {3,{21,37,42}} } },
   {4, {{3,{11,24,46}}, {4,{16,22,44,51}}, {3,{21,42,57}}, {2,{47,56}} } },
   {4, {{3,{11,24,45}}, {4,{15,22,44,51}}, {3,{21,42,57}}, {2,{47,55}} } },
   {4, {{3,{11,24,46}}, {4,{16,22,44,61}}, {3,{21,42,67}}, {2,{47,66}} } },
   {4, {{3,{11,24,45}}, {4,{15,22,44,61}}, {3,{21,42,67}}, {2,{47,65}} } },
   {3, {{5,{11,24,37,45,59}}, {4,{15,21,42,54}}, {3,{39,47,52}} } },
   {3, {{5,{11,24,37,45,58}}, {4,{15,21,42,54}}, {3,{38,47,52}} } },
   {3, {{5,{11,24,37,46,59}}, {4,{16,21,42,54}}, {3,{39,47,52}} } },
   {3, {{5,{11,24,37,46,58}}, {4,{16,21,42,54}}, {3,{38,47,52}} } },
   {3, {{5,{11,24,37,45,69}}, {4,{15,21,42,64}}, {3,{39,47,62}} } },
   {3, {{5,{11,24,37,45,68}}, {4,{15,21,42,64}}, {3,{38,47,62}} } },
   {3, {{5,{11,24,37,46,69}}, {4,{16,21,42,64}}, {3,{39,47,62}} } },
   {3, {{5,{11,24,37,46,68}}, {4,{16,21,42,64}}, {3,{38,47,62}} } },
   {5, {{2,{11,24}}, {3,{12,37,43}}, {2,{14,22}}, {3,{17,33,41}}, {2,{21,42}} } },
   {4, {{3,{11,24,46}}, {3,{16,27,54}}, {4,{17,21,42,56}}, {2,{44,52}} } },
   {4, {{3,{11,24,45}}, {3,{15,27,54}}, {4,{17,21,42,55}}, {2,{44,52}} } },
   {4, {{3,{11,24,46}}, {3,{16,27,64}}, {4,{17,21,42,66}}, {2,{44,62}} } },
   {4, {{3,{11,24,45}}, {3,{15,27,64}}, {4,{17,21,42,65}}, {2,{44,62}} } },
   {4, {{4,{11,24,47,65}}, {3,{12,45,61}}, {2,{14,22}}, {3,{21,42,67}} } },
   {4, {{4,{11,24,47,66}}, {3,{12,46,61}}, {2,{14,22}}, {3,{21,42,67}} } },
   {4, {{4,{11,24,47,55}}, {3,{12,45,51}}, {2,{14,22}}, {3,{21,42,57}} } },
   {4, {{4,{11,24,47,56}}, {3,{12,46,51}}, {2,{14,22}}, {3,{21,42,57}} } },
   {3, {{4,{11,24,47,62}}, {4,{14,32,51,67}}, {4,{21,34,42,57}} } },
   {3, {{4,{11,24,47,52}}, {4,{14,32,61,57}}, {4,{21,34,42,67}} } },
   {5, {{2,{11,24}}, {3,{14,33,41}}, {3,{21,37,42}}, {2,{27,34}}, {2,{32,43}} } },
   {4, {{2,{11,24}}, {2,{14,27}}, {4,{17,32,44,51}}, {4,{21,37,42,54}} } },
   {4, {{2,{11,24}}, {2,{14,27}}, {4,{17,32,44,61}}, {4,{21,37,42,64}} } },
   {4, {{2,{11,24}}, {2,{14,27}}, {4,{17,31,48,52}}, {4,{21,38,42,57}} } },
   {4, {{2,{11,24}}, {2,{14,27}}, {4,{17,31,49,52}}, {4,{21,39,42,57}} } },
   {4, {{2,{11,24}}, {2,{14,27}}, {4,{17,31,48,62}}, {4,{21,38,42,67}} } },
   {4, {{2,{11,24}}, {2,{14,27}}, {4,{17,31,49,62}}, {4,{21,39,42,67}} } },
   {3, {{3,{11,24,56}}, {5,{16,32,47,54,61}}, {4,{21,34,42,67}} } },
   {3, {{3,{11,24,55}}, {5,{15,32,47,54,61}}, {4,{21,34,42,67}} } },
   {3, {{3,{11,24,66}}, {5,{16,32,47,64,51}}, {4,{21,34,42,57}} } },
   {3, {{3,{11,24,65}}, {5,{15,32,47,64,51}}, {4,{21,34,42,57}} } },
   {2, {{4,{11,24,37,43}}, {4,{17,23,34,41}} } },
   {3, {{3,{11,24,43}}, {2,{14,27}}, {3,{17,23,41}} } },
   {2, {{4,{11,24,47,53}}, {4,{14,23,41,57}} } },
   {2, {{4,{11,24,47,63}}, {4,{14,23,41,67}} } },
   {3, {{3,{11,24,43}}, {3,{14,27,31}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,42}}, {2,{14,27}}, {3,{17,23,41}}, {2,{22,43}} } },
   {3, {{4,{11,24,45,53}}, {4,{14,23,41,57}}, {2,{47,55}} } },
   {3, {{4,{11,24,46,53}}, {4,{14,23,41,57}}, {2,{47,56}} } },
   {3, {{4,{11,24,45,63}}, {4,{14,23,41,67}}, {2,{47,65}} } },
   {3, {{4,{11,24,46,63}}, {4,{14,23,41,67}}, {2,{47,66}} } },
   {3, {{3,{11,24,43}}, {3,{14,27,59}}, {4,{19,23,41,57}} } },
   {3, {{3,{11,24,43}}, {3,{14,27,58}}, {4,{18,23,41,57}} } },
   {3, {{3,{11,24,43}}, {3,{14,27,69}}, {4,{19,23,41,67}} } },
   {3, {{3,{11,24,43}}, {3,{14,27,68}}, {4,{18,23,41,67}} } },
   {4, {{3,{11,24,43}}, {2,{14,27}}, {3,{16,23,41}}, {2,{17,26}} } },
   {4, {{3,{11,24,43}}, {2,{14,27}}, {3,{15,23,41}}, {2,{17,25}} } },
   {2, {{5,{11,24,46,57,63}}, {5,{14,23,41,56,67}} } },
   {2, {{5,{11,24,45,57,63}}, {5,{14,23,41,55,67}} } },
   {2, {{5,{11,24,46,67,53}}, {5,{14,23,41,66,57}} } },
   {2, {{5,{11,24,45,67,53}}, {5,{14,23,41,65,57}} } },
   {4, {{3,{11,24,43}}, {2,{14,37}}, {2,{17,31}}, {3,{23,34,41}} } },
   {3, {{4,{11,24,37,42}}, {4,{17,23,34,41}}, {2,{22,43}} } },
   {2, {{5,{11,24,38,43,57}}, {5,{17,23,34,41,58}} } },
   {2, {{5,{11,24,39,43,57}}, {5,{17,23,34,41,59}} } },
   {2, {{5,{11,24,38,43,67}}, {5,{17,23,34,41,68}} } },
   {2, {{5,{11,24,39,43,67}}, {5,{17,23,34,41,69}} } },
   {2, {{5,{11,24,43,57,65}}, {5,{15,23,41,54,67}} } },
   {2, {{5,{11,24,43,57,66}}, {5,{16,23,41,54,67}} } },
   {2, {{5,{11,24,43,67,55}}, {5,{15,23,41,64,57}} } },
   {2, {{5,{11,24,43,67,56}}, {5,{16,23,41,64,57}} } },
   {2, {{5,{11,24,47,53,66}}, {5,{16,23,41,54,67}} } },
   {2, {{5,{11,24,47,53,65}}, {5,{15,23,41,54,67}} } },
   {2, {{5,{11,24,47,63,56}}, {5,{16,23,41,64,57}} } },
   {2, {{5,{11,24,47,63,55}}, {5,{15,23,41,64,57}} } },
   {4, {{3,{11,24,43}}, {2,{14,37}}, {3,{17,23,41}}, {2,{27,34}} } },
   {4, {{3,{11,24,37}}, {3,{14,23,41}}, {3,{27,32,43}}, {2,{31,42}} } },
   {4, {{3,{11,24,43}}, {2,{14,27}}, {3,{17,26,31}}, {3,{23,36,41}} } },
   {4, {{3,{11,24,43}}, {2,{14,27}}, {3,{17,25,31}}, {3,{23,35,41}} } },
   {4, {{3,{11,24,37}}, {2,{14,29}}, {3,{19,23,41}}, {3,{27,31,43}} } },
   {4, {{3,{11,24,37}}, {2,{14,28}}, {3,{18,23,41}}, {3,{27,31,43}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,31}}, {2,{22,43}}, {3,{23,37,41}} } },
   {3, {{4,{11,24,37,43}}, {4,{17,23,41,69}}, {3,{29,34,67}} } },
   {3, {{4,{11,24,37,43}}, {4,{17,23,41,68}}, {3,{28,34,67}} } },
   {3, {{4,{11,24,37,43}}, {4,{17,23,41,59}}, {3,{29,34,57}} } },
   {3, {{4,{11,24,37,43}}, {4,{17,23,41,58}}, {3,{28,34,57}} } },
   {3, {{3,{11,24,37}}, {4,{14,23,41,69}}, {4,{27,31,49,63}} } },
   {3, {{3,{11,24,37}}, {4,{14,23,41,68}}, {4,{27,31,48,63}} } },
   {3, {{3,{11,24,37}}, {4,{14,23,41,59}}, {4,{27,31,49,53}} } },
   {3, {{3,{11,24,37}}, {4,{14,23,41,58}}, {4,{27,31,48,53}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,21,47,64}}, {3,{23,41,67}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,21,47,64}}, {3,{23,41,67}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,21,47,54}}, {3,{23,41,57}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,21,47,54}}, {3,{23,41,57}} } },
   {4, {{4,{11,24,47,62}}, {3,{14,23,41}}, {2,{22,63}}, {2,{43,67}} } },
   {4, {{4,{11,24,47,52}}, {3,{14,23,41}}, {2,{22,53}}, {2,{43,57}} } },
   {3, {{4,{11,24,49,63}}, {3,{14,27,31}}, {4,{23,37,41,69}} } },
   {3, {{4,{11,24,48,63}}, {3,{14,27,31}}, {4,{23,37,41,68}} } },
   {3, {{4,{11,24,49,53}}, {3,{14,27,31}}, {4,{23,37,41,59}} } },
   {3, {{4,{11,24,48,53}}, {3,{14,27,31}}, {4,{23,37,41,58}} } },
   {3, {{4,{11,24,49,63}}, {3,{14,27,69}}, {4,{19,23,41,67}} } },
   {3, {{4,{11,24,48,63}}, {3,{14,27,68}}, {4,{18,23,41,67}} } },
   {3, {{4,{11,24,49,53}}, {3,{14,27,59}}, {4,{19,23,41,57}} } },
   {3, {{4,{11,24,48,53}}, {3,{14,27,58}}, {4,{18,23,41,57}} } },
   {4, {{4,{11,24,37,42}}, {2,{14,29}}, {3,{19,23,41}}, {3,{27,32,43}} } },
   {4, {{4,{11,24,37,42}}, {2,{14,28}}, {3,{18,23,41}}, {3,{27,32,43}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{43,55,67}}, {3,{45,57,61}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{43,56,67}}, {3,{46,57,61}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{43,65,57}}, {3,{45,67,51}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{43,66,57}}, {3,{46,67,51}} } },
   {5, {{3,{11,24,63}}, {3,{14,23,41}}, {2,{43,65}}, {2,{45,67}}, {2,{47,61}} } },
   {5, {{3,{11,24,63}}, {3,{14,23,41}}, {2,{43,66}}, {2,{46,67}}, {2,{47,61}} } },
   {5, {{3,{11,24,53}}, {3,{14,23,41}}, {2,{43,55}}, {2,{45,57}}, {2,{47,51}} } },
   {5, {{3,{11,24,53}}, {3,{14,23,41}}, {2,{43,56}}, {2,{46,57}}, {2,{47,51}} } },
   {4, {{4,{11,24,46,53}}, {2,{14,28}}, {4,{18,23,41,57}}, {2,{47,56}} } },
   {4, {{4,{11,24,46,53}}, {2,{14,29}}, {4,{19,23,41,57}}, {2,{47,56}} } },
   {4, {{4,{11,24,45,53}}, {2,{14,28}}, {4,{18,23,41,57}}, {2,{47,55}} } },
   {4, {{4,{11,24,45,53}}, {2,{14,29}}, {4,{19,23,41,57}}, {2,{47,55}} } },
   {4, {{4,{11,24,46,63}}, {2,{14,28}}, {4,{18,23,41,67}}, {2,{47,66}} } },
   {4, {{4,{11,24,46,63}}, {2,{14,29}}, {4,{19,23,41,67}}, {2,{47,66}} } },
   {4, {{4,{11,24,45,63}}, {2,{14,28}}, {4,{18,23,41,67}}, {2,{47,65}} } },
   {4, {{4,{11,24,45,63}}, {2,{14,29}}, {4,{19,23,41,67}}, {2,{47,65}} } },
   {3, {{5,{11,24,39,47,63}}, {5,{19,23,34,41,65}}, {2,{45,67}} } },
   {3, {{5,{11,24,38,47,63}}, {5,{18,23,34,41,65}}, {2,{45,67}} } },
   {3, {{5,{11,24,39,47,63}}, {5,{19,23,34,41,66}}, {2,{46,67}} } },
   {3, {{5,{11,24,38,47,63}}, {5,{18,23,34,41,66}}, {2,{46,67}} } },
   {3, {{5,{11,24,39,47,53}}, {5,{19,23,34,41,55}}, {2,{45,57}} } },
   {3, {{5,{11,24,38,47,53}}, {5,{18,23,34,41,55}}, {2,{45,57}} } },
   {3, {{5,{11,24,39,47,53}}, {5,{19,23,34,41,56}}, {2,{46,57}} } },
   {3, {{5,{11,24,38,47,53}}, {5,{18,23,34,41,56}}, {2,{46,57}} } },
   {4, {{4,{11,24,38,43}}, {4,{15,23,37,41}}, {2,{18,35}}, {2,{27,34}} } },
   {4, {{4,{11,24,39,43}}, {4,{15,23,37,41}}, {2,{19,35}}, {2,{27,34}} } },
   {4, {{4,{11,24,38,43}}, {4,{16,23,37,41}}, {2,{18,36}}, {2,{27,34}} } },
   {4, {{4,{11,24,39,43}}, {4,{16,23,37,41}}, {2,{19,36}}, {2,{27,34}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,46}}, {3,{23,37,41}}, {3,{27,36,44}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,45}}, {3,{23,37,41}}, {3,{27,35,44}} } },
   {3, {{4,{11,24,47,63}}, {4,{14,31,46,67}}, {4,{23,36,41,64}} } },
   {3, {{4,{11,24,47,63}}, {4,{14,31,45,67}}, {4,{23,35,41,64}} } },
   {3, {{4,{11,24,47,53}}, {4,{14,31,46,57}}, {4,{23,36,41,54}} } },
   {3, {{4,{11,24,47,53}}, {4,{14,31,45,57}}, {4,{23,35,41,54}} } },
   {3, {{5,{11,24,39,53,67}}, {5,{19,23,34,41,57}}, {2,{47,61}} } },
   {3, {{5,{11,24,38,53,67}}, {5,{18,23,34,41,57}}, {2,{47,61}} } },
   {3, {{5,{11,24,39,63,57}}, {5,{19,23,34,41,67}}, {2,{47,51}} } },
   {3, {{5,{11,24,38,63,57}}, {5,{18,23,34,41,67}}, {2,{47,51}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,29,47,64}}, {4,{17,23,41,69}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,28,47,64}}, {4,{17,23,41,68}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,29,47,64}}, {4,{17,23,41,69}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,28,47,64}}, {4,{17,23,41,68}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,29,47,54}}, {4,{17,23,41,59}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,28,47,54}}, {4,{17,23,41,58}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,29,47,54}}, {4,{17,23,41,59}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,28,47,54}}, {4,{17,23,41,58}} } },
   {5, {{3,{11,24,37}}, {2,{13,27}}, {3,{14,23,41}}, {2,{17,33}}, {2,{31,43}} } },
   {4, {{4,{11,24,37,43}}, {4,{17,23,36,41}}, {2,{26,44}}, {2,{34,46}} } },
   {4, {{4,{11,24,37,43}}, {4,{17,23,35,41}}, {2,{25,44}}, {2,{34,45}} } },
   {3, {{4,{11,24,53,66}}, {5,{16,23,41,57,64}}, {3,{46,51,67}} } },
   {3, {{4,{11,24,53,65}}, {5,{15,23,41,57,64}}, {3,{45,51,67}} } },
   {3, {{4,{11,24,63,56}}, {5,{16,23,41,67,54}}, {3,{46,61,57}} } },
   {3, {{4,{11,24,63,55}}, {5,{15,23,41,67,54}}, {3,{45,61,57}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{43,54,67}}, {3,{47,51,64}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{43,64,57}}, {3,{47,61,54}} } },
   {4, {{4,{11,24,39,43}}, {2,{13,37}}, {4,{17,23,34,41}}, {2,{19,33}} } },
   {4, {{4,{11,24,38,43}}, {2,{13,37}}, {4,{17,23,34,41}}, {2,{18,33}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,27}}, {4,{17,23,41,66}}, {2,{46,67}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,27}}, {4,{17,23,41,65}}, {2,{45,67}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,27}}, {4,{17,23,41,56}}, {2,{46,57}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,27}}, {4,{17,23,41,55}}, {2,{45,57}} } },
   {3, {{5,{11,24,37,49,62}}, {5,{14,23,39,41,67}}, {2,{22,63}} } },
   {3, {{5,{11,24,37,48,62}}, {5,{14,23,38,41,67}}, {2,{22,63}} } },
   {3, {{5,{11,24,37,49,52}}, {5,{14,23,39,41,57}}, {2,{22,53}} } },
   {3, {{5,{11,24,37,48,52}}, {5,{14,23,38,41,57}}, {2,{22,53}} } },
   {4, {{4,{11,24,49,53}}, {2,{14,37}}, {4,{17,23,41,59}}, {2,{27,34}} } },
   {4, {{4,{11,24,48,53}}, {2,{14,37}}, {4,{17,23,41,58}}, {2,{27,34}} } },
   {4, {{4,{11,24,49,63}}, {2,{14,37}}, {4,{17,23,41,69}}, {2,{27,34}} } },
   {4, {{4,{11,24,48,63}}, {2,{14,37}}, {4,{17,23,41,68}}, {2,{27,34}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {2,{17,31}}, {3,{23,38,41}}, {2,{28,34}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {2,{17,31}}, {3,{23,39,41}}, {2,{29,34}} } },
   {5, {{2,{11,24}}, {3,{12,27,43}}, {2,{14,22}}, {3,{17,23,41}}, {2,{21,42}} } },
   {4, {{4,{11,24,43,65}}, {3,{15,37,64}}, {3,{17,23,41}}, {2,{27,34}} } },
   {4, {{4,{11,24,43,66}}, {3,{16,37,64}}, {3,{17,23,41}}, {2,{27,34}} } },
   {4, {{4,{11,24,43,55}}, {3,{15,37,54}}, {3,{17,23,41}}, {2,{27,34}} } },
   {4, {{4,{11,24,43,56}}, {3,{16,37,54}}, {3,{17,23,41}}, {2,{27,34}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{17,43,68}}, {3,{18,47,61}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{17,43,69}}, {3,{19,47,61}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{17,43,58}}, {3,{18,47,51}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{17,43,59}}, {3,{19,47,51}} } },
   {4, {{4,{11,24,49,53}}, {4,{14,23,41,57}}, {2,{17,59}}, {2,{19,47}} } },
   {4, {{4,{11,24,48,53}}, {4,{14,23,41,57}}, {2,{17,58}}, {2,{18,47}} } },
   {4, {{4,{11,24,49,63}}, {4,{14,23,41,67}}, {2,{17,69}}, {2,{19,47}} } },
   {4, {{4,{11,24,48,63}}, {4,{14,23,41,67}}, {2,{17,68}}, {2,{18,47}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {3,{17,23,41}}, {2,{22,34}}, {2,{27,32}} } },
   {5, {{3,{11,24,39}}, {2,{14,27}}, {3,{17,23,41}}, {2,{29,33}}, {2,{31,43}} } },
   {5, {{3,{11,24,38}}, {2,{14,27}}, {3,{17,23,41}}, {2,{28,33}}, {2,{31,43}} } },
   {4, {{4,{11,24,49,63}}, {2,{14,27}}, {4,{17,23,41,65}}, {2,{45,69}} } },
   {4, {{4,{11,24,48,63}}, {2,{14,27}}, {4,{17,23,41,65}}, {2,{45,68}} } },
   {4, {{4,{11,24,49,63}}, {2,{14,27}}, {4,{17,23,41,66}}, {2,{46,69}} } },
   {4, {{4,{11,24,48,63}}, {2,{14,27}}, {4,{17,23,41,66}}, {2,{46,68}} } },
   {4, {{4,{11,24,49,53}}, {2,{14,27}}, {4,{17,23,41,55}}, {2,{45,59}} } },
   {4, {{4,{11,24,48,53}}, {2,{14,27}}, {4,{17,23,41,55}}, {2,{45,58}} } },
   {4, {{4,{11,24,49,53}}, {2,{14,27}}, {4,{17,23,41,56}}, {2,{46,59}} } },
   {4, {{4,{11,24,48,53}}, {2,{14,27}}, {4,{17,23,41,56}}, {2,{46,58}} } },
   {5, {{3,{11,24,43}}, {2,{14,27}}, {2,{16,31}}, {2,{17,26}}, {3,{23,36,41}} } },
   {5, {{3,{11,24,43}}, {2,{14,27}}, {2,{15,31}}, {2,{17,25}}, {3,{23,35,41}} } },
   {5, {{3,{11,24,53}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,59}}, {2,{49,51}} } },
   {5, {{3,{11,24,53}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,58}}, {2,{48,51}} } },
   {5, {{3,{11,24,63}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,69}}, {2,{49,61}} } },
   {5, {{3,{11,24,63}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,68}}, {2,{48,61}} } },
   {4, {{3,{11,24,43}}, {2,{14,31}}, {4,{23,37,41,59}}, {3,{29,34,57}} } },
   {4, {{3,{11,24,43}}, {2,{14,31}}, {4,{23,37,41,58}}, {3,{28,34,57}} } },
   {4, {{3,{11,24,43}}, {2,{14,31}}, {4,{23,37,41,69}}, {3,{29,34,67}} } },
   {4, {{3,{11,24,43}}, {2,{14,31}}, {4,{23,37,41,68}}, {3,{28,34,67}} } },
   {5, {{3,{11,24,43}}, {2,{14,31}}, {3,{23,35,41}}, {2,{25,37}}, {2,{27,34}} } },
   {5, {{3,{11,24,43}}, {2,{14,31}}, {3,{23,36,41}}, {2,{26,37}}, {2,{27,34}} } },
   {4, {{4,{11,24,37,43}}, {4,{17,23,36,41}}, {2,{26,39}}, {2,{29,34}} } },
   {4, {{4,{11,24,37,43}}, {4,{17,23,36,41}}, {2,{26,38}}, {2,{28,34}} } },
   {4, {{4,{11,24,37,43}}, {4,{17,23,35,41}}, {2,{25,39}}, {2,{29,34}} } },
   {4, {{4,{11,24,37,43}}, {4,{17,23,35,41}}, {2,{25,38}}, {2,{28,34}} } },
   {4, {{4,{11,24,53,67}}, {3,{14,23,41}}, {3,{43,57,66}}, {2,{46,61}} } },
   {4, {{4,{11,24,53,67}}, {3,{14,23,41}}, {3,{43,57,65}}, {2,{45,61}} } },
   {4, {{4,{11,24,63,57}}, {3,{14,23,41}}, {3,{43,67,56}}, {2,{46,51}} } },
   {4, {{4,{11,24,63,57}}, {3,{14,23,41}}, {3,{43,67,55}}, {2,{45,51}} } },
   {4, {{3,{11,24,53}}, {4,{14,23,41,67}}, {2,{45,51}}, {3,{47,55,63}} } },
   {4, {{3,{11,24,53}}, {4,{14,23,41,67}}, {2,{46,51}}, {3,{47,56,63}} } },
   {4, {{3,{11,24,63}}, {4,{14,23,41,57}}, {2,{45,61}}, {3,{47,65,53}} } },
   {4, {{3,{11,24,63}}, {4,{14,23,41,57}}, {2,{46,61}}, {3,{47,66,53}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,49}}, {3,{19,23,41}}, {3,{29,34,47}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,48}}, {3,{18,23,41}}, {3,{28,34,47}} } },
   {4, {{4,{11,24,57,63}}, {4,{14,23,41,66}}, {2,{46,51}}, {2,{56,67}} } },
   {4, {{4,{11,24,57,63}}, {4,{14,23,41,65}}, {2,{45,51}}, {2,{55,67}} } },
   {4, {{4,{11,24,67,53}}, {4,{14,23,41,56}}, {2,{46,61}}, {2,{66,57}} } },
   {4, {{4,{11,24,67,53}}, {4,{14,23,41,55}}, {2,{45,61}}, {2,{65,57}} } },
   {4, {{4,{11,24,48,63}}, {4,{14,23,41,67}}, {2,{46,68}}, {2,{47,66}} } },
   {4, {{4,{11,24,49,63}}, {4,{14,23,41,67}}, {2,{46,69}}, {2,{47,66}} } },
   {4, {{4,{11,24,48,63}}, {4,{14,23,41,67}}, {2,{45,68}}, {2,{47,65}} } },
   {4, {{4,{11,24,49,63}}, {4,{14,23,41,67}}, {2,{45,69}}, {2,{47,65}} } },
   {4, {{4,{11,24,48,53}}, {4,{14,23,41,57}}, {2,{46,58}}, {2,{47,56}} } },
   {4, {{4,{11,24,49,53}}, {4,{14,23,41,57}}, {2,{46,59}}, {2,{47,56}} } },
   {4, {{4,{11,24,48,53}}, {4,{14,23,41,57}}, {2,{45,58}}, {2,{47,55}} } },
   {4, {{4,{11,24,49,53}}, {4,{14,23,41,57}}, {2,{45,59}}, {2,{47,55}} } },
   {3, {{5,{11,24,39,47,63}}, {5,{17,23,34,41,59}}, {2,{53,69}} } },
   {3, {{5,{11,24,38,47,63}}, {5,{17,23,34,41,58}}, {2,{53,68}} } },
   {3, {{5,{11,24,39,47,53}}, {5,{17,23,34,41,69}}, {2,{63,59}} } },
   {3, {{5,{11,24,38,47,53}}, {5,{17,23,34,41,68}}, {2,{63,58}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,27,44,61}}, {4,{17,23,41,66}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,27,44,61}}, {4,{17,23,41,65}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,27,44,51}}, {4,{17,23,41,56}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,27,44,51}}, {4,{17,23,41,55}} } },
   {5, {{3,{11,24,53}}, {3,{14,23,41}}, {2,{43,67}}, {2,{47,51}}, {2,{57,63}} } },
   {5, {{3,{11,24,63}}, {3,{14,23,41}}, {2,{43,57}}, {2,{47,61}}, {2,{67,53}} } },
   {2, {{6,{11,24,39,47,53,65}}, {6,{17,23,34,41,55,69}} } },
   {2, {{6,{11,24,38,47,53,65}}, {6,{17,23,34,41,55,68}} } },
   {2, {{6,{11,24,39,47,53,66}}, {6,{17,23,34,41,56,69}} } },
   {2, {{6,{11,24,38,47,53,66}}, {6,{17,23,34,41,56,68}} } },
   {2, {{6,{11,24,39,47,63,55}}, {6,{17,23,34,41,65,59}} } },
   {2, {{6,{11,24,38,47,63,55}}, {6,{17,23,34,41,65,58}} } },
   {2, {{6,{11,24,39,47,63,56}}, {6,{17,23,34,41,66,59}} } },
   {2, {{6,{11,24,38,47,63,56}}, {6,{17,23,34,41,66,58}} } },
   {5, {{3,{11,24,37}}, {3,{14,23,41}}, {2,{27,33}}, {2,{31,42}}, {2,{32,43}} } },
   {4, {{4,{11,24,53,67}}, {2,{14,29}}, {4,{19,23,41,57}}, {2,{47,61}} } },
   {4, {{4,{11,24,53,67}}, {2,{14,28}}, {4,{18,23,41,57}}, {2,{47,61}} } },
   {4, {{4,{11,24,63,57}}, {2,{14,29}}, {4,{19,23,41,67}}, {2,{47,51}} } },
   {4, {{4,{11,24,63,57}}, {2,{14,28}}, {4,{18,23,41,67}}, {2,{47,51}} } },
   {3, {{4,{11,24,45,63}}, {4,{14,27,53,65}}, {4,{17,23,41,55}} } },
   {3, {{4,{11,24,46,63}}, {4,{14,27,53,66}}, {4,{17,23,41,56}} } },
   {3, {{4,{11,24,45,53}}, {4,{14,27,63,55}}, {4,{17,23,41,65}} } },
   {3, {{4,{11,24,46,53}}, {4,{14,27,63,56}}, {4,{17,23,41,66}} } },
   {2, {{6,{11,24,39,47,53,66}}, {6,{19,23,34,41,56,67}} } },
   {2, {{6,{11,24,38,47,53,66}}, {6,{18,23,34,41,56,67}} } },
   {2, {{6,{11,24,39,47,53,65}}, {6,{19,23,34,41,55,67}} } },
   {2, {{6,{11,24,38,47,53,65}}, {6,{18,23,34,41,55,67}} } },
   {2, {{6,{11,24,39,47,63,56}}, {6,{19,23,34,41,66,57}} } },
   {2, {{6,{11,24,38,47,63,56}}, {6,{18,23,34,41,66,57}} } },
   {2, {{6,{11,24,39,47,63,55}}, {6,{19,23,34,41,65,57}} } },
   {2, {{6,{11,24,38,47,63,55}}, {6,{18,23,34,41,65,57}} } },
   {5, {{3,{11,24,43}}, {2,{14,28}}, {2,{16,27}}, {3,{17,23,41}}, {2,{18,26}} } },
   {5, {{3,{11,24,43}}, {2,{14,29}}, {2,{16,27}}, {3,{17,23,41}}, {2,{19,26}} } },
   {5, {{3,{11,24,43}}, {2,{14,28}}, {2,{15,27}}, {3,{17,23,41}}, {2,{18,25}} } },
   {5, {{3,{11,24,43}}, {2,{14,29}}, {2,{15,27}}, {3,{17,23,41}}, {2,{19,25}} } },
   {3, {{5,{11,24,37,43,66}}, {5,{17,23,35,41,64}}, {2,{36,65}} } },
   {3, {{5,{11,24,37,43,65}}, {5,{17,23,36,41,64}}, {2,{35,66}} } },
   {3, {{5,{11,24,37,43,56}}, {5,{17,23,35,41,54}}, {2,{36,55}} } },
   {3, {{5,{11,24,37,43,55}}, {5,{17,23,36,41,54}}, {2,{35,56}} } },
   {4, {{4,{11,24,39,43}}, {3,{12,27,34}}, {3,{17,23,41}}, {2,{19,32}} } },
   {4, {{4,{11,24,38,43}}, {3,{12,27,34}}, {3,{17,23,41}}, {2,{18,32}} } },
   {5, {{3,{11,24,37}}, {3,{14,23,41}}, {2,{22,43}}, {2,{27,32}}, {2,{31,42}} } },
   {4, {{4,{11,24,37,43}}, {2,{13,26}}, {3,{16,27,31}}, {3,{23,34,41}} } },
   {4, {{4,{11,24,37,43}}, {2,{13,25}}, {3,{15,27,31}}, {3,{23,34,41}} } },
   {3, {{4,{11,24,43,66}}, {4,{16,27,59,64}}, {4,{19,23,41,57}} } },
   {3, {{4,{11,24,43,66}}, {4,{16,27,58,64}}, {4,{18,23,41,57}} } },
   {3, {{4,{11,24,43,65}}, {4,{15,27,59,64}}, {4,{19,23,41,57}} } },
   {3, {{4,{11,24,43,65}}, {4,{15,27,58,64}}, {4,{18,23,41,57}} } },
   {3, {{4,{11,24,43,56}}, {4,{16,27,69,54}}, {4,{19,23,41,67}} } },
   {3, {{4,{11,24,43,56}}, {4,{16,27,68,54}}, {4,{18,23,41,67}} } },
   {3, {{4,{11,24,43,55}}, {4,{15,27,69,54}}, {4,{19,23,41,67}} } },
   {3, {{4,{11,24,43,55}}, {4,{15,27,68,54}}, {4,{18,23,41,67}} } },
   {2, {{6,{11,24,37,49,56,63}}, {6,{16,23,39,41,54,67}} } },
   {2, {{6,{11,24,37,48,56,63}}, {6,{16,23,38,41,54,67}} } },
   {2, {{6,{11,24,37,49,55,63}}, {6,{15,23,39,41,54,67}} } },
   {2, {{6,{11,24,37,48,55,63}}, {6,{15,23,38,41,54,67}} } },
   {2, {{6,{11,24,37,49,66,53}}, {6,{16,23,39,41,64,57}} } },
   {2, {{6,{11,24,37,48,66,53}}, {6,{16,23,38,41,64,57}} } },
   {2, {{6,{11,24,37,49,65,53}}, {6,{15,23,39,41,64,57}} } },
   {2, {{6,{11,24,37,48,65,53}}, {6,{15,23,38,41,64,57}} } },
   {3, {{5,{11,24,45,53,67}}, {2,{15,31}}, {5,{23,35,41,57,64}} } },
   {3, {{5,{11,24,46,53,67}}, {2,{16,31}}, {5,{23,36,41,57,64}} } },
   {3, {{5,{11,24,45,63,57}}, {2,{15,31}}, {5,{23,35,41,67,54}} } },
   {3, {{5,{11,24,46,63,57}}, {2,{16,31}}, {5,{23,36,41,67,54}} } },
   {4, {{4,{11,24,37,43}}, {2,{17,49}}, {3,{19,23,41}}, {3,{29,34,47}} } },
   {4, {{4,{11,24,37,43}}, {2,{17,48}}, {3,{18,23,41}}, {3,{28,34,47}} } },
   {3, {{5,{11,24,46,57,63}}, {3,{14,23,41}}, {4,{16,43,54,67}} } },
   {3, {{5,{11,24,45,57,63}}, {3,{14,23,41}}, {4,{15,43,54,67}} } },
   {3, {{5,{11,24,46,67,53}}, {3,{14,23,41}}, {4,{16,43,64,57}} } },
   {3, {{5,{11,24,45,67,53}}, {3,{14,23,41}}, {4,{15,43,64,57}} } },
   {2, {{6,{11,24,37,43,59,66}}, {6,{19,23,36,41,57,64}} } },
   {2, {{6,{11,24,37,43,58,66}}, {6,{18,23,36,41,57,64}} } },
   {2, {{6,{11,24,37,43,59,65}}, {6,{19,23,35,41,57,64}} } },
   {2, {{6,{11,24,37,43,58,65}}, {6,{18,23,35,41,57,64}} } },
   {2, {{6,{11,24,37,43,69,56}}, {6,{19,23,36,41,67,54}} } },
   {2, {{6,{11,24,37,43,68,56}}, {6,{18,23,36,41,67,54}} } },
   {2, {{6,{11,24,37,43,69,55}}, {6,{19,23,35,41,67,54}} } },
   {2, {{6,{11,24,37,43,68,55}}, {6,{18,23,35,41,67,54}} } },
   {3, {{4,{11,24,45,63}}, {3,{15,37,46}}, {5,{17,23,36,41,64}} } },
   {3, {{4,{11,24,46,63}}, {3,{16,37,45}}, {5,{17,23,35,41,64}} } },
   {3, {{4,{11,24,45,53}}, {3,{15,37,46}}, {5,{17,23,36,41,54}} } },
   {3, {{4,{11,24,46,53}}, {3,{16,37,45}}, {5,{17,23,35,41,54}} } },
   {3, {{5,{11,24,37,46,53}}, {4,{15,23,41,54}}, {3,{17,36,45}} } },
   {3, {{5,{11,24,37,45,53}}, {4,{16,23,41,54}}, {3,{17,35,46}} } },
   {3, {{5,{11,24,37,46,63}}, {4,{15,23,41,64}}, {3,{17,36,45}} } },
   {3, {{5,{11,24,37,45,63}}, {4,{16,23,41,64}}, {3,{17,35,46}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,59}}, {4,{19,23,41,57}}, {2,{22,43}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,58}}, {4,{18,23,41,57}}, {2,{22,43}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,69}}, {4,{19,23,41,67}}, {2,{22,43}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,68}}, {4,{18,23,41,67}}, {2,{22,43}} } },
   {3, {{3,{11,24,43}}, {4,{14,45,57,63}}, {5,{15,23,41,54,67}} } },
   {3, {{3,{11,24,43}}, {4,{14,46,57,63}}, {5,{16,23,41,54,67}} } },
   {3, {{3,{11,24,43}}, {4,{14,45,67,53}}, {5,{15,23,41,64,57}} } },
   {3, {{3,{11,24,43}}, {4,{14,46,67,53}}, {5,{16,23,41,64,57}} } },
   {5, {{3,{11,24,53}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,54}}, {2,{44,51}} } },
   {5, {{3,{11,24,63}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,64}}, {2,{44,61}} } },
   {3, {{4,{11,24,37,43}}, {5,{17,23,36,41,54}}, {3,{34,46,53}} } },
   {3, {{4,{11,24,37,43}}, {5,{17,23,35,41,54}}, {3,{34,45,53}} } },
   {3, {{4,{11,24,37,43}}, {5,{17,23,36,41,64}}, {3,{34,46,63}} } },
   {3, {{4,{11,24,37,43}}, {5,{17,23,35,41,64}}, {3,{34,45,63}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,49}}, {3,{15,23,41}}, {3,{19,25,47}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,48}}, {3,{15,23,41}}, {3,{18,25,47}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,49}}, {3,{16,23,41}}, {3,{19,26,47}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,48}}, {3,{16,23,41}}, {3,{18,26,47}} } },
   {4, {{4,{11,24,37,42}}, {3,{12,27,43}}, {3,{14,23,41}}, {2,{17,32}} } },
   {3, {{4,{11,24,49,63}}, {4,{14,27,46,69}}, {4,{17,23,41,66}} } },
   {3, {{4,{11,24,48,63}}, {4,{14,27,46,68}}, {4,{17,23,41,66}} } },
   {3, {{4,{11,24,49,63}}, {4,{14,27,45,69}}, {4,{17,23,41,65}} } },
   {3, {{4,{11,24,48,63}}, {4,{14,27,45,68}}, {4,{17,23,41,65}} } },
   {3, {{4,{11,24,49,53}}, {4,{14,27,46,59}}, {4,{17,23,41,56}} } },
   {3, {{4,{11,24,48,53}}, {4,{14,27,46,58}}, {4,{17,23,41,56}} } },
   {3, {{4,{11,24,49,53}}, {4,{14,27,45,59}}, {4,{17,23,41,55}} } },
   {3, {{4,{11,24,48,53}}, {4,{14,27,45,58}}, {4,{17,23,41,55}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {3,{15,23,41}}, {2,{17,25}}, {2,{27,34}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {3,{16,23,41}}, {2,{17,26}}, {2,{27,34}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {2,{17,21}}, {3,{23,34,41}}, {2,{27,31}} } },
   {4, {{3,{11,24,43}}, {2,{14,28}}, {3,{18,31,67}}, {4,{23,37,41,68}} } },
   {4, {{3,{11,24,43}}, {2,{14,29}}, {3,{19,31,67}}, {4,{23,37,41,69}} } },
   {4, {{3,{11,24,43}}, {2,{14,28}}, {3,{18,31,57}}, {4,{23,37,41,58}} } },
   {4, {{3,{11,24,43}}, {2,{14,29}}, {3,{19,31,57}}, {4,{23,37,41,59}} } },
   {5, {{3,{11,24,43}}, {2,{14,27}}, {2,{15,31}}, {2,{17,35}}, {3,{23,37,41}} } },
   {5, {{3,{11,24,43}}, {2,{14,27}}, {2,{16,31}}, {2,{17,36}}, {3,{23,37,41}} } },
   {3, {{5,{11,24,37,42,65}}, {4,{15,23,41,64}}, {3,{27,32,43}} } },
   {3, {{5,{11,24,37,42,66}}, {4,{16,23,41,64}}, {3,{27,32,43}} } },
   {3, {{5,{11,24,37,42,55}}, {4,{15,23,41,54}}, {3,{27,32,43}} } },
   {3, {{5,{11,24,37,42,56}}, {4,{16,23,41,54}}, {3,{27,32,43}} } },
   {4, {{4,{11,24,43,66}}, {2,{12,27}}, {3,{16,22,64}}, {3,{17,23,41}} } },
   {4, {{4,{11,24,43,65}}, {2,{12,27}}, {3,{15,22,64}}, {3,{17,23,41}} } },
   {4, {{4,{11,24,43,56}}, {2,{12,27}}, {3,{16,22,54}}, {3,{17,23,41}} } },
   {4, {{4,{11,24,43,55}}, {2,{12,27}}, {3,{15,22,54}}, {3,{17,23,41}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,27}}, {4,{17,23,41,64}}, {2,{44,67}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,27}}, {4,{17,23,41,54}}, {2,{44,57}} } },
   {2, {{6,{11,24,39,47,53,65}}, {6,{17,23,35,41,54,69}} } },
   {2, {{6,{11,24,38,47,53,65}}, {6,{17,23,35,41,54,68}} } },
   {2, {{6,{11,24,39,47,53,66}}, {6,{17,23,36,41,54,69}} } },
   {2, {{6,{11,24,38,47,53,66}}, {6,{17,23,36,41,54,68}} } },
   {2, {{6,{11,24,39,47,63,55}}, {6,{17,23,35,41,64,59}} } },
   {2, {{6,{11,24,38,47,63,55}}, {6,{17,23,35,41,64,58}} } },
   {2, {{6,{11,24,39,47,63,56}}, {6,{17,23,36,41,64,59}} } },
   {2, {{6,{11,24,38,47,63,56}}, {6,{17,23,36,41,64,58}} } },
   {2, {{6,{11,24,38,45,53,67}}, {6,{17,23,35,41,58,64}} } },
   {2, {{6,{11,24,39,45,53,67}}, {6,{17,23,35,41,59,64}} } },
   {2, {{6,{11,24,38,46,53,67}}, {6,{17,23,36,41,58,64}} } },
   {2, {{6,{11,24,39,46,53,67}}, {6,{17,23,36,41,59,64}} } },
   {2, {{6,{11,24,38,45,63,57}}, {6,{17,23,35,41,68,54}} } },
   {2, {{6,{11,24,39,45,63,57}}, {6,{17,23,35,41,69,54}} } },
   {2, {{6,{11,24,38,46,63,57}}, {6,{17,23,36,41,68,54}} } },
   {2, {{6,{11,24,39,46,63,57}}, {6,{17,23,36,41,69,54}} } },
   {3, {{5,{11,24,46,52,67}}, {4,{14,23,41,66}}, {3,{22,57,63}} } },
   {3, {{5,{11,24,45,52,67}}, {4,{14,23,41,65}}, {3,{22,57,63}} } },
   {3, {{5,{11,24,46,62,57}}, {4,{14,23,41,56}}, {3,{22,67,53}} } },
   {3, {{5,{11,24,45,62,57}}, {4,{14,23,41,55}}, {3,{22,67,53}} } },
   {3, {{5,{11,24,38,43,57}}, {4,{17,23,41,59}}, {3,{29,34,58}} } },
   {3, {{5,{11,24,39,43,57}}, {4,{17,23,41,58}}, {3,{28,34,59}} } },
   {3, {{5,{11,24,38,43,67}}, {4,{17,23,41,69}}, {3,{29,34,68}} } },
   {3, {{5,{11,24,39,43,67}}, {4,{17,23,41,68}}, {3,{28,34,69}} } },
   {2, {{6,{11,24,39,45,57,63}}, {6,{17,23,35,41,59,64}} } },
   {2, {{6,{11,24,38,45,57,63}}, {6,{17,23,35,41,58,64}} } },
   {2, {{6,{11,24,39,46,57,63}}, {6,{17,23,36,41,59,64}} } },
   {2, {{6,{11,24,38,46,57,63}}, {6,{17,23,36,41,58,64}} } },
   {2, {{6,{11,24,39,45,67,53}}, {6,{17,23,35,41,69,54}} } },
   {2, {{6,{11,24,38,45,67,53}}, {6,{17,23,35,41,68,54}} } },
   {2, {{6,{11,24,39,46,67,53}}, {6,{17,23,36,41,69,54}} } },
   {2, {{6,{11,24,38,46,67,53}}, {6,{17,23,36,41,68,54}} } },
   {2, {{6,{11,24,37,45,59,63}}, {6,{17,23,34,41,55,69}} } },
   {2, {{6,{11,24,37,45,58,63}}, {6,{17,23,34,41,55,68}} } },
   {2, {{6,{11,24,37,46,59,63}}, {6,{17,23,34,41,56,69}} } },
   {2, {{6,{11,24,37,46,58,63}}, {6,{17,23,34,41,56,68}} } },
   {2, {{6,{11,24,37,45,69,53}}, {6,{17,23,34,41,65,59}} } },
   {2, {{6,{11,24,37,45,68,53}}, {6,{17,23,34,41,65,58}} } },
   {2, {{6,{11,24,37,46,69,53}}, {6,{17,23,34,41,66,59}} } },
   {2, {{6,{11,24,37,46,68,53}}, {6,{17,23,34,41,66,58}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,56}}, {4,{17,23,36,41}}, {2,{26,54}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,55}}, {4,{17,23,35,41}}, {2,{25,54}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,66}}, {4,{17,23,36,41}}, {2,{26,64}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,65}}, {4,{17,23,35,41}}, {2,{25,64}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,69}}, {2,{19,31}}, {4,{23,39,41,67}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,68}}, {2,{18,31}}, {4,{23,38,41,67}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,59}}, {2,{19,31}}, {4,{23,39,41,57}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,58}}, {2,{18,31}}, {4,{23,38,41,57}} } },
   {4, {{4,{11,24,46,57}}, {4,{16,23,41,54}}, {2,{21,53}}, {2,{47,51}} } },
   {4, {{4,{11,24,45,57}}, {4,{15,23,41,54}}, {2,{21,53}}, {2,{47,51}} } },
   {4, {{4,{11,24,46,67}}, {4,{16,23,41,64}}, {2,{21,63}}, {2,{47,61}} } },
   {4, {{4,{11,24,45,67}}, {4,{15,23,41,64}}, {2,{21,63}}, {2,{47,61}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,39}}, {2,{19,31}}, {4,{23,34,41,67}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,38}}, {2,{18,31}}, {4,{23,34,41,67}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,39}}, {2,{19,31}}, {4,{23,34,41,57}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,38}}, {2,{18,31}}, {4,{23,34,41,57}} } },
   {4, {{4,{11,24,46,53}}, {3,{16,27,44}}, {3,{17,23,41}}, {2,{43,54}} } },
   {4, {{4,{11,24,45,53}}, {3,{15,27,44}}, {3,{17,23,41}}, {2,{43,54}} } },
   {4, {{4,{11,24,46,63}}, {3,{16,27,44}}, {3,{17,23,41}}, {2,{43,64}} } },
   {4, {{4,{11,24,45,63}}, {3,{15,27,44}}, {3,{17,23,41}}, {2,{43,64}} } },
   {3, {{4,{11,24,37,43}}, {4,{15,23,41,54}}, {4,{17,34,45,53}} } },
   {3, {{4,{11,24,37,43}}, {4,{16,23,41,54}}, {4,{17,34,46,53}} } },
   {3, {{4,{11,24,37,43}}, {4,{15,23,41,64}}, {4,{17,34,45,63}} } },
   {3, {{4,{11,24,37,43}}, {4,{16,23,41,64}}, {4,{17,34,46,63}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,48}}, {3,{18,31,47}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,49}}, {3,{19,31,47}}, {3,{23,37,41}} } },
   {5, {{3,{11,24,43}}, {2,{12,27}}, {2,{14,32}}, {3,{17,23,41}}, {2,{22,34}} } },
   {3, {{4,{11,24,46,63}}, {4,{16,21,57,64}}, {4,{23,41,56,67}} } },
   {3, {{4,{11,24,45,63}}, {4,{15,21,57,64}}, {4,{23,41,55,67}} } },
   {3, {{4,{11,24,46,53}}, {4,{16,21,67,54}}, {4,{23,41,66,57}} } },
   {3, {{4,{11,24,45,53}}, {4,{15,21,67,54}}, {4,{23,41,65,57}} } },
   {4, {{4,{11,24,46,63}}, {4,{16,23,41,54}}, {2,{53,67}}, {2,{57,64}} } },
   {4, {{4,{11,24,45,63}}, {4,{15,23,41,54}}, {2,{53,67}}, {2,{57,64}} } },
   {4, {{4,{11,24,46,53}}, {4,{16,23,41,64}}, {2,{63,57}}, {2,{67,54}} } },
   {4, {{4,{11,24,45,53}}, {4,{15,23,41,64}}, {2,{63,57}}, {2,{67,54}} } },
   {4, {{4,{11,24,47,53}}, {4,{14,23,41,56}}, {2,{26,54}}, {2,{44,57}} } },
   {4, {{4,{11,24,47,53}}, {4,{14,23,41,55}}, {2,{25,54}}, {2,{44,57}} } },
   {4, {{4,{11,24,47,63}}, {4,{14,23,41,66}}, {2,{26,64}}, {2,{44,67}} } },
   {4, {{4,{11,24,47,63}}, {4,{14,23,41,65}}, {2,{25,64}}, {2,{44,67}} } },
   {4, {{3,{11,24,63}}, {4,{14,23,41,57}}, {3,{47,54,61}}, {2,{53,64}} } },
   {4, {{3,{11,24,53}}, {4,{14,23,41,67}}, {3,{47,64,51}}, {2,{63,54}} } },
   {4, {{4,{11,24,37,55}}, {4,{15,23,41,54}}, {2,{27,33}}, {2,{31,43}} } },
   {4, {{4,{11,24,37,56}}, {4,{16,23,41,54}}, {2,{27,33}}, {2,{31,43}} } },
   {4, {{4,{11,24,37,65}}, {4,{15,23,41,64}}, {2,{27,33}}, {2,{31,43}} } },
   {4, {{4,{11,24,37,66}}, {4,{16,23,41,64}}, {2,{27,33}}, {2,{31,43}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,68}}, {3,{18,31,67}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,69}}, {3,{19,31,67}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,58}}, {3,{18,31,57}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,59}}, {3,{19,31,57}}, {3,{23,37,41}} } },
   {4, {{4,{11,24,45,53}}, {3,{14,23,41}}, {3,{15,47,54}}, {2,{43,57}} } },
   {4, {{4,{11,24,46,53}}, {3,{14,23,41}}, {3,{16,47,54}}, {2,{43,57}} } },
   {4, {{4,{11,24,45,63}}, {3,{14,23,41}}, {3,{15,47,64}}, {2,{43,67}} } },
   {4, {{4,{11,24,46,63}}, {3,{14,23,41}}, {3,{16,47,64}}, {2,{43,67}} } },
   {3, {{5,{11,24,37,48,63}}, {5,{14,23,39,41,67}}, {2,{38,49}} } },
   {3, {{5,{11,24,37,49,63}}, {5,{14,23,38,41,67}}, {2,{39,48}} } },
   {3, {{5,{11,24,37,48,53}}, {5,{14,23,39,41,57}}, {2,{38,49}} } },
   {3, {{5,{11,24,37,49,53}}, {5,{14,23,38,41,57}}, {2,{39,48}} } },
   {3, {{4,{11,24,47,53}}, {5,{14,23,37,41,59}}, {3,{29,34,57}} } },
   {3, {{4,{11,24,47,53}}, {5,{14,23,37,41,58}}, {3,{28,34,57}} } },
   {3, {{4,{11,24,47,63}}, {5,{14,23,37,41,69}}, {3,{29,34,67}} } },
   {3, {{4,{11,24,47,63}}, {5,{14,23,37,41,68}}, {3,{28,34,67}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {3,{17,23,41}}, {2,{21,34}}, {2,{27,31}} } },
   {3, {{4,{11,24,57,65}}, {5,{15,23,41,54,67}}, {3,{21,45,63}} } },
   {3, {{4,{11,24,57,66}}, {5,{16,23,41,54,67}}, {3,{21,46,63}} } },
   {3, {{4,{11,24,67,55}}, {5,{15,23,41,64,57}}, {3,{21,45,53}} } },
   {3, {{4,{11,24,67,56}}, {5,{16,23,41,64,57}}, {3,{21,46,53}} } },
   {3, {{5,{11,24,47,55,63}}, {3,{15,31,54}}, {4,{23,34,41,67}} } },
   {3, {{5,{11,24,47,56,63}}, {3,{16,31,54}}, {4,{23,34,41,67}} } },
   {3, {{5,{11,24,47,65,53}}, {3,{15,31,64}}, {4,{23,34,41,57}} } },
   {3, {{5,{11,24,47,66,53}}, {3,{16,31,64}}, {4,{23,34,41,57}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,69}}, {3,{19,31,67}}, {3,{23,34,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,68}}, {3,{18,31,67}}, {3,{23,34,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,59}}, {3,{19,31,57}}, {3,{23,34,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,37,58}}, {3,{18,31,57}}, {3,{23,34,41}} } },
   {3, {{4,{11,24,53,65}}, {4,{14,23,41,57}}, {4,{15,47,54,61}} } },
   {3, {{4,{11,24,53,66}}, {4,{14,23,41,57}}, {4,{16,47,54,61}} } },
   {3, {{4,{11,24,63,55}}, {4,{14,23,41,67}}, {4,{15,47,64,51}} } },
   {3, {{4,{11,24,63,56}}, {4,{14,23,41,67}}, {4,{16,47,64,51}} } },
   {4, {{3,{11,24,42}}, {4,{14,22,37,43}}, {2,{17,31}}, {3,{23,34,41}} } },
   {3, {{5,{11,24,37,49,63}}, {3,{19,31,67}}, {4,{23,34,41,69}} } },
   {3, {{5,{11,24,37,48,63}}, {3,{18,31,67}}, {4,{23,34,41,68}} } },
   {3, {{5,{11,24,37,49,53}}, {3,{19,31,57}}, {4,{23,34,41,59}} } },
   {3, {{5,{11,24,37,48,53}}, {3,{18,31,57}}, {4,{23,34,41,58}} } },
   {4, {{3,{11,24,43}}, {3,{14,33,61}}, {4,{23,34,41,67}}, {2,{47,63}} } },
   {4, {{3,{11,24,43}}, {3,{14,33,51}}, {4,{23,34,41,57}}, {2,{47,53}} } },
   {3, {{5,{11,24,37,46,62}}, {3,{14,23,41}}, {4,{27,32,43,66}} } },
   {3, {{5,{11,24,37,45,62}}, {3,{14,23,41}}, {4,{27,32,43,65}} } },
   {3, {{5,{11,24,37,46,52}}, {3,{14,23,41}}, {4,{27,32,43,56}} } },
   {3, {{5,{11,24,37,45,52}}, {3,{14,23,41}}, {4,{27,32,43,55}} } },
   {3, {{3,{11,24,63}}, {5,{14,23,37,41,69}}, {4,{29,34,47,61}} } },
   {3, {{3,{11,24,63}}, {5,{14,23,37,41,68}}, {4,{28,34,47,61}} } },
   {3, {{3,{11,24,53}}, {5,{14,23,37,41,59}}, {4,{29,34,47,51}} } },
   {3, {{3,{11,24,53}}, {5,{14,23,37,41,58}}, {4,{28,34,47,51}} } },
   {4, {{4,{11,24,47,62}}, {4,{14,23,41,57}}, {2,{22,63}}, {2,{53,67}} } },
   {4, {{4,{11,24,47,52}}, {4,{14,23,41,67}}, {2,{22,53}}, {2,{63,57}} } },
   {4, {{3,{11,24,43}}, {3,{14,29,67}}, {3,{15,23,41}}, {3,{17,25,69}} } },
   {4, {{3,{11,24,43}}, {3,{14,28,67}}, {3,{15,23,41}}, {3,{17,25,68}} } },
   {4, {{3,{11,24,43}}, {3,{14,29,67}}, {3,{16,23,41}}, {3,{17,26,69}} } },
   {4, {{3,{11,24,43}}, {3,{14,28,67}}, {3,{16,23,41}}, {3,{17,26,68}} } },
   {4, {{3,{11,24,43}}, {3,{14,29,57}}, {3,{15,23,41}}, {3,{17,25,59}} } },
   {4, {{3,{11,24,43}}, {3,{14,28,57}}, {3,{15,23,41}}, {3,{17,25,58}} } },
   {4, {{3,{11,24,43}}, {3,{14,29,57}}, {3,{16,23,41}}, {3,{17,26,59}} } },
   {4, {{3,{11,24,43}}, {3,{14,28,57}}, {3,{16,23,41}}, {3,{17,26,58}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,65}}, {3,{23,37,41}}, {3,{27,35,64}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,66}}, {3,{23,37,41}}, {3,{27,36,64}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,55}}, {3,{23,37,41}}, {3,{27,35,54}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,56}}, {3,{23,37,41}}, {3,{27,36,54}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,27}}, {4,{15,23,41,67}}, {2,{17,25}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,27}}, {4,{16,23,41,67}}, {2,{17,26}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,27}}, {4,{15,23,41,57}}, {2,{17,25}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,27}}, {4,{16,23,41,57}}, {2,{17,26}} } },
   {3, {{4,{11,24,37,49}}, {4,{14,23,41,59}}, {4,{29,31,47,53}} } },
   {3, {{4,{11,24,37,48}}, {4,{14,23,41,58}}, {4,{28,31,47,53}} } },
   {3, {{4,{11,24,37,49}}, {4,{14,23,41,69}}, {4,{29,31,47,63}} } },
   {3, {{4,{11,24,37,48}}, {4,{14,23,41,68}}, {4,{28,31,47,63}} } },
   },
   {
   {3, {{4,{11,24,47,53}}, {3,{14,22,41}}, {3,{23,42,57}} } },
   {3, {{4,{11,24,47,63}}, {3,{14,22,41}}, {3,{23,42,67}} } },
   {3, {{4,{11,24,37,43}}, {3,{17,32,41}}, {3,{23,34,42}} } },
   {4, {{2,{11,24}}, {3,{12,27,43}}, {3,{14,23,42}}, {2,{17,21}} } },
   {3, {{4,{11,24,47,63}}, {2,{12,41}}, {4,{14,23,42,67}} } },
   {3, {{4,{11,24,47,53}}, {2,{12,41}}, {4,{14,23,42,57}} } },
   {3, {{4,{11,24,43,67}}, {3,{12,47,61}}, {3,{14,23,42}} } },
   {3, {{4,{11,24,43,57}}, {3,{12,47,51}}, {3,{14,23,42}} } },
   {4, {{4,{11,24,43,57}}, {3,{12,47,51}}, {2,{14,22}}, {2,{23,42}} } },
   {4, {{4,{11,24,43,67}}, {3,{12,47,61}}, {2,{14,22}}, {2,{23,42}} } },
   {4, {{2,{11,24}}, {3,{12,47,51}}, {3,{14,23,42}}, {3,{21,43,57}} } },
   {4, {{2,{11,24}}, {3,{12,47,61}}, {3,{14,23,42}}, {3,{21,43,67}} } },
   {4, {{3,{11,24,43}}, {2,{14,27}}, {3,{17,32,41}}, {3,{23,37,42}} } },
   {4, {{3,{11,24,43}}, {3,{14,22,37}}, {3,{17,32,41}}, {2,{23,42}} } },
   {4, {{3,{11,24,43}}, {4,{14,23,37,42}}, {2,{17,31}}, {2,{32,41}} } },
   {3, {{3,{11,24,53}}, {4,{14,32,47,51}}, {4,{23,34,42,57}} } },
   {3, {{3,{11,24,63}}, {4,{14,32,47,61}}, {4,{23,34,42,67}} } },
   {4, {{4,{11,24,37,43}}, {2,{12,41}}, {2,{17,32}}, {3,{23,34,42}} } },
   {3, {{5,{11,24,39,43,67}}, {5,{17,22,34,49,61}}, {2,{23,42}} } },
   {3, {{5,{11,24,38,43,67}}, {5,{17,22,34,48,61}}, {2,{23,42}} } },
   {3, {{5,{11,24,39,43,57}}, {5,{17,22,34,49,51}}, {2,{23,42}} } },
   {3, {{5,{11,24,38,43,57}}, {5,{17,22,34,48,51}}, {2,{23,42}} } },
   {3, {{4,{11,24,57,63}}, {4,{14,32,47,51}}, {4,{23,34,42,67}} } },
   {3, {{4,{11,24,67,53}}, {4,{14,32,47,61}}, {4,{23,34,42,57}} } },
   {5, {{2,{11,24}}, {3,{12,47,51}}, {3,{14,23,42}}, {2,{21,53}}, {2,{43,57}} } },
   {5, {{2,{11,24}}, {3,{12,47,61}}, {3,{14,23,42}}, {2,{21,63}}, {2,{43,67}} } },
   {5, {{3,{11,24,43}}, {3,{14,22,51}}, {2,{23,42}}, {2,{41,57}}, {2,{47,52}} } },
   {5, {{3,{11,24,43}}, {3,{14,22,61}}, {2,{23,42}}, {2,{41,67}}, {2,{47,62}} } },
   {5, {{3,{11,24,53}}, {3,{14,22,41}}, {2,{23,42}}, {2,{43,57}}, {2,{47,51}} } },
   {5, {{3,{11,24,63}}, {3,{14,22,41}}, {2,{23,42}}, {2,{43,67}}, {2,{47,61}} } },
   {3, {{5,{11,24,46,53,67}}, {2,{12,41}}, {5,{14,23,42,57,66}} } },
   {3, {{5,{11,24,45,53,67}}, {2,{12,41}}, {5,{14,23,42,57,65}} } },
   {3, {{5,{11,24,46,63,57}}, {2,{12,41}}, {5,{14,23,42,67,56}} } },
   {3, {{5,{11,24,45,63,57}}, {2,{12,41}}, {5,{14,23,42,67,55}} } },
   {3, {{5,{11,24,39,43,57}}, {2,{12,41}}, {5,{17,23,34,42,59}} } },
   {3, {{5,{11,24,38,43,57}}, {2,{12,41}}, {5,{17,23,34,42,58}} } },
   {3, {{5,{11,24,39,43,67}}, {2,{12,41}}, {5,{17,23,34,42,69}} } },
   {3, {{5,{11,24,38,43,67}}, {2,{12,41}}, {5,{17,23,34,42,68}} } },
   {4, {{4,{11,24,39,43}}, {3,{19,32,41}}, {3,{23,37,42}}, {2,{27,34}} } },
   {4, {{4,{11,24,38,43}}, {3,{18,32,41}}, {3,{23,37,42}}, {2,{27,34}} } },
   {5, {{3,{11,24,43}}, {2,{12,41}}, {2,{14,27}}, {3,{15,23,42}}, {2,{17,25}} } },
   {5, {{3,{11,24,43}}, {2,{12,41}}, {2,{14,27}}, {3,{16,23,42}}, {2,{17,26}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,31}}, {4,{23,34,42,57}}, {2,{32,41}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,31}}, {4,{23,34,42,67}}, {2,{32,41}} } },
   {5, {{3,{11,24,43}}, {2,{12,51}}, {3,{14,23,42}}, {2,{41,57}}, {2,{47,52}} } },
   {5, {{3,{11,24,43}}, {2,{12,61}}, {3,{14,23,42}}, {2,{41,67}}, {2,{47,62}} } },
   {5, {{4,{11,24,37,43}}, {2,{12,41}}, {2,{17,32}}, {2,{22,34}}, {2,{23,42}} } },
   {5, {{2,{11,24}}, {2,{12,37}}, {3,{14,23,42}}, {2,{17,21}}, {3,{27,32,43}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,28}}, {3,{18,22,41}}, {3,{23,42,57}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,29}}, {3,{19,22,41}}, {3,{23,42,57}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,28}}, {3,{18,22,41}}, {3,{23,42,67}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,29}}, {3,{19,22,41}}, {3,{23,42,67}} } },
   {5, {{2,{11,24}}, {3,{12,27,43}}, {3,{14,23,42}}, {2,{15,21}}, {2,{17,25}} } },
   {5, {{2,{11,24}}, {3,{12,27,43}}, {3,{14,23,42}}, {2,{16,21}}, {2,{17,26}} } },
   {4, {{3,{11,24,38}}, {3,{12,27,43}}, {2,{17,21}}, {4,{18,23,34,42}} } },
   {4, {{3,{11,24,39}}, {3,{12,27,43}}, {2,{17,21}}, {4,{19,23,34,42}} } },
   {3, {{5,{11,24,37,43,65}}, {4,{17,32,45,61}}, {3,{23,34,42}} } },
   {3, {{5,{11,24,37,43,66}}, {4,{17,32,46,61}}, {3,{23,34,42}} } },
   {3, {{5,{11,24,37,43,55}}, {4,{17,32,45,51}}, {3,{23,34,42}} } },
   {3, {{5,{11,24,37,43,56}}, {4,{17,32,46,51}}, {3,{23,34,42}} } },
   {4, {{3,{11,24,39}}, {4,{14,23,37,42}}, {2,{17,31}}, {3,{29,32,43}} } },
   {4, {{3,{11,24,38}}, {4,{14,23,37,42}}, {2,{17,31}}, {3,{28,32,43}} } },
   {5, {{3,{11,24,43}}, {2,{14,31}}, {3,{23,37,42}}, {2,{27,34}}, {2,{32,41}} } },
   {6, {{2,{11,24}}, {2,{12,43}}, {2,{13,27}}, {2,{14,22}}, {2,{17,21}}, {2,{23,42}} } },
   {4, {{2,{11,24}}, {4,{12,27,36,43}}, {3,{14,23,42}}, {3,{16,21,37}} } },
   {4, {{2,{11,24}}, {4,{12,27,35,43}}, {3,{14,23,42}}, {3,{15,21,37}} } },
   {4, {{3,{11,24,39}}, {3,{14,27,31}}, {3,{23,37,42}}, {3,{29,32,43}} } },
   {4, {{3,{11,24,38}}, {3,{14,27,31}}, {3,{23,37,42}}, {3,{28,32,43}} } },
   {4, {{4,{11,24,43,58}}, {3,{12,48,51}}, {2,{14,27}}, {3,{17,23,42}} } },
   {4, {{4,{11,24,43,59}}, {3,{12,49,51}}, {2,{14,27}}, {3,{17,23,42}} } },
   {4, {{4,{11,24,43,68}}, {3,{12,48,61}}, {2,{14,27}}, {3,{17,23,42}} } },
   {4, {{4,{11,24,43,69}}, {3,{12,49,61}}, {2,{14,27}}, {3,{17,23,42}} } },
   {4, {{4,{11,24,47,53}}, {3,{14,22,41}}, {3,{23,42,54}}, {2,{44,57}} } },
   {4, {{4,{11,24,47,63}}, {3,{14,22,41}}, {3,{23,42,64}}, {2,{44,67}} } },
   {4, {{3,{11,24,43}}, {2,{12,41}}, {3,{14,29,57}}, {4,{17,23,42,59}} } },
   {4, {{3,{11,24,43}}, {2,{12,41}}, {3,{14,28,57}}, {4,{17,23,42,58}} } },
   {4, {{3,{11,24,43}}, {2,{12,41}}, {3,{14,29,67}}, {4,{17,23,42,69}} } },
   {4, {{3,{11,24,43}}, {2,{12,41}}, {3,{14,28,67}}, {4,{17,23,42,68}} } },
   {4, {{3,{11,24,53}}, {4,{14,22,47,51}}, {3,{23,42,54}}, {2,{44,57}} } },
   {4, {{3,{11,24,63}}, {4,{14,22,47,61}}, {3,{23,42,64}}, {2,{44,67}} } },
   {3, {{4,{11,24,38,63}}, {3,{12,47,61}}, {5,{18,23,34,42,67}} } },
   {3, {{4,{11,24,39,63}}, {3,{12,47,61}}, {5,{19,23,34,42,67}} } },
   {3, {{4,{11,24,38,53}}, {3,{12,47,51}}, {5,{18,23,34,42,57}} } },
   {3, {{4,{11,24,39,53}}, {3,{12,47,51}}, {5,{19,23,34,42,57}} } },
   {5, {{3,{11,24,43}}, {3,{14,22,37}}, {2,{17,31}}, {2,{23,42}}, {2,{32,41}} } },
   {4, {{4,{11,24,43,67}}, {3,{12,47,51}}, {3,{14,23,42}}, {2,{57,61}} } },
   {4, {{4,{11,24,43,57}}, {3,{12,47,61}}, {3,{14,23,42}}, {2,{67,51}} } },
   {3, {{5,{11,24,38,43,57}}, {3,{12,47,51}}, {4,{18,23,34,42}} } },
   {3, {{5,{11,24,39,43,57}}, {3,{12,47,51}}, {4,{19,23,34,42}} } },
   {3, {{5,{11,24,38,43,67}}, {3,{12,47,61}}, {4,{18,23,34,42}} } },
   {3, {{5,{11,24,39,43,67}}, {3,{12,47,61}}, {4,{19,23,34,42}} } },
   {4, {{2,{11,24}}, {3,{14,32,41}}, {3,{21,47,63}}, {4,{23,34,42,67}} } },
   {4, {{2,{11,24}}, {3,{14,32,41}}, {3,{21,47,53}}, {4,{23,34,42,57}} } },
   {5, {{2,{11,24}}, {2,{13,27}}, {3,{14,32,43}}, {2,{17,21}}, {3,{23,34,42}} } },
   {4, {{3,{11,24,43}}, {3,{12,37,41}}, {2,{14,31}}, {4,{17,23,34,42}} } },
   {5, {{3,{11,24,43}}, {2,{12,41}}, {2,{14,37}}, {2,{17,32}}, {3,{23,34,42}} } },
   {3, {{5,{11,24,37,43,66}}, {5,{17,22,36,44,61}}, {2,{23,42}} } },
   {3, {{5,{11,24,37,43,65}}, {5,{17,22,35,44,61}}, {2,{23,42}} } },
   {3, {{5,{11,24,37,43,56}}, {5,{17,22,36,44,51}}, {2,{23,42}} } },
   {3, {{5,{11,24,37,43,55}}, {5,{17,22,35,44,51}}, {2,{23,42}} } },
   {3, {{3,{11,24,63}}, {5,{14,22,37,48,61}}, {4,{23,38,42,67}} } },
   {3, {{3,{11,24,63}}, {5,{14,22,37,49,61}}, {4,{23,39,42,67}} } },
   {3, {{3,{11,24,53}}, {5,{14,22,37,48,51}}, {4,{23,38,42,57}} } },
   {3, {{3,{11,24,53}}, {5,{14,22,37,49,51}}, {4,{23,39,42,57}} } },
   {5, {{2,{11,24}}, {3,{12,29,43}}, {2,{14,27}}, {3,{17,23,42}}, {2,{19,21}} } },
   {5, {{2,{11,24}}, {3,{12,28,43}}, {2,{14,27}}, {3,{17,23,42}}, {2,{18,21}} } },
   {4, {{4,{11,24,47,63}}, {2,{12,43}}, {2,{13,41}}, {4,{14,23,42,67}} } },
   {4, {{4,{11,24,47,53}}, {2,{12,43}}, {2,{13,41}}, {4,{14,23,42,57}} } },
   {3, {{4,{11,24,45,63}}, {4,{14,27,32,41}}, {4,{23,37,42,65}} } },
   {3, {{4,{11,24,46,63}}, {4,{14,27,32,41}}, {4,{23,37,42,66}} } },
   {3, {{4,{11,24,45,53}}, {4,{14,27,32,41}}, {4,{23,37,42,55}} } },
   {3, {{4,{11,24,46,53}}, {4,{14,27,32,41}}, {4,{23,37,42,56}} } },
   {5, {{3,{11,24,43}}, {3,{14,27,31}}, {2,{22,37}}, {2,{23,42}}, {2,{32,41}} } },
   {3, {{5,{11,24,43,56,67}}, {4,{12,47,51,66}}, {3,{14,23,42}} } },
   {3, {{5,{11,24,43,55,67}}, {4,{12,47,51,65}}, {3,{14,23,42}} } },
   {3, {{5,{11,24,43,66,57}}, {4,{12,47,61,56}}, {3,{14,23,42}} } },
   {3, {{5,{11,24,43,65,57}}, {4,{12,47,61,55}}, {3,{14,23,42}} } },
   {4, {{4,{11,24,43,57}}, {2,{14,31}}, {3,{23,34,42}}, {3,{32,47,51}} } },
   {4, {{4,{11,24,43,67}}, {2,{14,31}}, {3,{23,34,42}}, {3,{32,47,61}} } },
   {4, {{4,{11,24,37,43}}, {2,{17,21}}, {3,{23,34,42}}, {3,{27,32,41}} } },
   {5, {{3,{11,24,43}}, {2,{14,31}}, {3,{22,37,41}}, {2,{23,42}}, {2,{27,34}} } },
   {5, {{3,{11,24,43}}, {2,{14,27}}, {2,{17,31}}, {3,{23,37,42}}, {2,{32,41}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {2,{17,31}}, {3,{22,34,41}}, {2,{23,42}} } },
   {5, {{3,{11,24,43}}, {2,{14,37}}, {3,{17,22,41}}, {2,{23,42}}, {2,{27,34}} } },
   {4, {{3,{11,24,63}}, {3,{12,57,61}}, {4,{14,23,42,67}}, {2,{47,52}} } },
   {4, {{3,{11,24,53}}, {3,{12,67,51}}, {4,{14,23,42,57}}, {2,{47,62}} } },
   {4, {{4,{11,24,43,57}}, {2,{14,31}}, {4,{22,34,47,51}}, {2,{23,42}} } },
   {4, {{4,{11,24,43,67}}, {2,{14,31}}, {4,{22,34,47,61}}, {2,{23,42}} } },
   {4, {{4,{11,24,57,63}}, {3,{12,47,51}}, {2,{14,22}}, {3,{23,42,67}} } },
   {4, {{4,{11,24,67,53}}, {3,{12,47,61}}, {2,{14,22}}, {3,{23,42,57}} } },
   {5, {{2,{11,24}}, {2,{14,37}}, {3,{17,32,41}}, {2,{21,43}}, {3,{23,34,42}} } },
   {4, {{2,{11,24}}, {4,{14,23,37,42}}, {2,{15,21}}, {4,{17,25,32,43}} } },
   {4, {{2,{11,24}}, {4,{14,23,37,42}}, {2,{16,21}}, {4,{17,26,32,43}} } },
   {5, {{2,{11,24}}, {3,{12,27,43}}, {2,{14,32}}, {2,{17,21}}, {3,{23,34,42}} } },
   {3, {{5,{11,24,37,49,53}}, {3,{14,22,41}}, {4,{23,39,42,57}} } },
   {3, {{5,{11,24,37,48,53}}, {3,{14,22,41}}, {4,{23,38,42,57}} } },
   {3, {{5,{11,24,37,49,63}}, {3,{14,22,41}}, {4,{23,39,42,67}} } },
   {3, {{5,{11,24,37,48,63}}, {3,{14,22,41}}, {4,{23,38,42,67}} } },
   {4, {{4,{11,24,37,43}}, {2,{12,34}}, {3,{14,23,42}}, {3,{17,32,41}} } },
   {4, {{4,{11,24,53,67}}, {3,{14,22,41}}, {3,{23,42,57}}, {2,{47,61}} } },
   {4, {{4,{11,24,63,57}}, {3,{14,22,41}}, {3,{23,42,67}}, {2,{47,51}} } },
   {3, {{4,{11,24,47,53}}, {4,{14,29,32,41}}, {4,{23,39,42,57}} } },
   {3, {{4,{11,24,47,53}}, {4,{14,28,32,41}}, {4,{23,38,42,57}} } },
   {3, {{4,{11,24,47,63}}, {4,{14,29,32,41}}, {4,{23,39,42,67}} } },
   {3, {{4,{11,24,47,63}}, {4,{14,28,32,41}}, {4,{23,38,42,67}} } },
   {4, {{3,{11,24,43}}, {4,{12,27,34,41}}, {2,{14,37}}, {3,{17,23,42}} } },
   {4, {{3,{11,24,53}}, {3,{12,37,43}}, {3,{14,23,42}}, {3,{17,33,51}} } },
   {4, {{3,{11,24,63}}, {3,{12,37,43}}, {3,{14,23,42}}, {3,{17,33,61}} } },
   },
   };

int const cntpat2x3[ 4 ] = {489, 255, 744, 148};

UnavPat const unavpat3x3[ 4 ][ 155 ] = {
   {
   {4, {{3,{11,24,73}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,71}} } },
   {3, {{4,{11,24,77,83}}, {3,{14,23,41}}, {3,{43,71,87}} } },
   {3, {{4,{11,24,77,93}}, {3,{14,23,41}}, {3,{43,71,97}} } },
   {2, {{5,{11,24,37,43,76}}, {5,{17,23,36,41,74}} } },
   {2, {{5,{11,24,37,43,75}}, {5,{17,23,35,41,74}} } },
   {3, {{3,{11,24,73}}, {4,{14,27,43,71}}, {3,{17,23,41}} } },
   {3, {{4,{11,24,39,77}}, {3,{14,23,41}}, {4,{27,31,43,79}} } },
   {3, {{4,{11,24,38,77}}, {3,{14,23,41}}, {4,{27,31,43,78}} } },
   {3, {{4,{11,24,77,82}}, {3,{14,23,41}}, {4,{22,43,71,87}} } },
   {3, {{4,{11,24,77,92}}, {3,{14,23,41}}, {4,{22,43,71,97}} } },
   {5, {{3,{11,24,63}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,71}}, {2,{61,73}} } },
   {5, {{3,{11,24,53}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,71}}, {2,{51,73}} } },
   {2, {{6,{11,24,37,49,52,73}}, {6,{17,23,34,41,59,72}} } },
   {2, {{6,{11,24,37,48,52,73}}, {6,{17,23,34,41,58,72}} } },
   {2, {{6,{11,24,37,49,62,73}}, {6,{17,23,34,41,69,72}} } },
   {2, {{6,{11,24,37,48,62,73}}, {6,{17,23,34,41,68,72}} } },
   {5, {{3,{11,24,72}}, {2,{14,27}}, {3,{17,23,41}}, {2,{22,43}}, {2,{42,71}} } },
   {5, {{3,{11,24,73}}, {3,{14,23,41}}, {2,{43,57}}, {2,{47,51}}, {2,{53,71}} } },
   {5, {{3,{11,24,73}}, {3,{14,23,41}}, {2,{43,67}}, {2,{47,61}}, {2,{63,71}} } },
   {4, {{4,{11,24,42,73}}, {2,{14,27}}, {4,{16,23,41,72}}, {2,{17,26}} } },
   {4, {{4,{11,24,42,73}}, {2,{14,27}}, {4,{15,23,41,72}}, {2,{17,25}} } },
   {4, {{3,{11,24,73}}, {2,{14,29}}, {4,{19,23,41,67}}, {3,{47,63,71}} } },
   {4, {{3,{11,24,73}}, {2,{14,28}}, {4,{18,23,41,67}}, {3,{47,63,71}} } },
   {4, {{3,{11,24,73}}, {2,{14,29}}, {4,{19,23,41,57}}, {3,{47,53,71}} } },
   {4, {{3,{11,24,73}}, {2,{14,28}}, {4,{18,23,41,57}}, {3,{47,53,71}} } },
   {3, {{3,{11,24,43}}, {4,{14,28,69,77}}, {5,{17,23,41,68,79}} } },
   {3, {{3,{11,24,43}}, {4,{14,29,68,77}}, {5,{17,23,41,69,78}} } },
   {3, {{3,{11,24,43}}, {4,{14,28,59,77}}, {5,{17,23,41,58,79}} } },
   {3, {{3,{11,24,43}}, {4,{14,29,58,77}}, {5,{17,23,41,59,78}} } },
   {5, {{3,{11,24,73}}, {2,{14,27}}, {3,{17,23,41}}, {2,{42,71}}, {2,{43,72}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,59}}, {4,{19,23,41,77}}, {2,{57,79}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,58}}, {4,{18,23,41,77}}, {2,{57,78}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,69}}, {4,{19,23,41,77}}, {2,{67,79}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,68}}, {4,{18,23,41,77}}, {2,{67,78}} } },
   {2, {{6,{11,24,37,43,68,79}}, {6,{19,23,34,41,67,78}} } },
   {2, {{6,{11,24,37,43,69,78}}, {6,{18,23,34,41,67,79}} } },
   {2, {{6,{11,24,37,43,58,79}}, {6,{19,23,34,41,57,78}} } },
   {2, {{6,{11,24,37,43,59,78}}, {6,{18,23,34,41,57,79}} } },
   {4, {{3,{11,24,73}}, {2,{14,31}}, {4,{23,34,41,67}}, {3,{47,63,71}} } },
   {4, {{3,{11,24,73}}, {2,{14,31}}, {4,{23,34,41,57}}, {3,{47,53,71}} } },
   {3, {{5,{11,24,49,63,78}}, {5,{14,23,41,68,77}}, {2,{47,79}} } },
   {3, {{5,{11,24,48,63,79}}, {5,{14,23,41,69,77}}, {2,{47,78}} } },
   {3, {{5,{11,24,49,53,78}}, {5,{14,23,41,58,77}}, {2,{47,79}} } },
   {3, {{5,{11,24,48,53,79}}, {5,{14,23,41,59,77}}, {2,{47,78}} } },
   {3, {{5,{11,24,38,47,63}}, {5,{14,23,41,68,77}}, {2,{37,78}} } },
   {3, {{5,{11,24,39,47,63}}, {5,{14,23,41,69,77}}, {2,{37,79}} } },
   {3, {{5,{11,24,38,47,53}}, {5,{14,23,41,58,77}}, {2,{37,78}} } },
   {3, {{5,{11,24,39,47,53}}, {5,{14,23,41,59,77}}, {2,{37,79}} } },
   {3, {{5,{11,24,47,62,73}}, {5,{14,23,41,57,72}}, {2,{52,67}} } },
   {3, {{5,{11,24,47,52,73}}, {5,{14,23,41,67,72}}, {2,{62,57}} } },
   {4, {{2,{11,24}}, {4,{12,27,43,71}}, {4,{14,23,41,72}}, {2,{17,21}} } },
   {4, {{4,{11,24,42,73}}, {2,{14,37}}, {4,{17,23,41,72}}, {2,{27,34}} } },
   {3, {{4,{11,24,42,73}}, {3,{14,27,98}}, {5,{18,23,41,72,97}} } },
   {3, {{4,{11,24,42,73}}, {3,{14,27,99}}, {5,{19,23,41,72,97}} } },
   {3, {{4,{11,24,42,73}}, {3,{14,27,88}}, {5,{18,23,41,72,87}} } },
   {3, {{4,{11,24,42,73}}, {3,{14,27,89}}, {5,{19,23,41,72,87}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,78}}, {4,{18,23,41,77}}, {2,{22,43}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,79}}, {4,{19,23,41,77}}, {2,{22,43}} } },
   {4, {{3,{11,24,73}}, {3,{14,27,79}}, {4,{19,23,41,77}}, {2,{43,71}} } },
   {4, {{3,{11,24,73}}, {3,{14,27,78}}, {4,{18,23,41,77}}, {2,{43,71}} } },
   {3, {{3,{11,24,73}}, {5,{14,28,47,63,71}}, {4,{18,23,41,67}} } },
   {3, {{3,{11,24,73}}, {5,{14,29,47,63,71}}, {4,{19,23,41,67}} } },
   {3, {{3,{11,24,73}}, {5,{14,28,47,53,71}}, {4,{18,23,41,57}} } },
   {3, {{3,{11,24,73}}, {5,{14,29,47,53,71}}, {4,{19,23,41,57}} } },
   {4, {{4,{11,24,37,72}}, {3,{14,23,41}}, {3,{27,32,73}}, {2,{43,71}} } },
   {3, {{3,{11,24,83}}, {5,{14,27,43,78,81}}, {4,{18,23,41,77}} } },
   {3, {{3,{11,24,83}}, {5,{14,27,43,79,81}}, {4,{19,23,41,77}} } },
   {3, {{3,{11,24,93}}, {5,{14,27,43,78,91}}, {4,{18,23,41,77}} } },
   {3, {{3,{11,24,93}}, {5,{14,27,43,79,91}}, {4,{19,23,41,77}} } },
   {3, {{3,{11,24,73}}, {5,{14,27,49,63,71}}, {4,{19,23,41,67}} } },
   {3, {{3,{11,24,73}}, {5,{14,27,48,63,71}}, {4,{18,23,41,67}} } },
   {3, {{3,{11,24,73}}, {5,{14,27,49,53,71}}, {4,{19,23,41,57}} } },
   {3, {{3,{11,24,73}}, {5,{14,27,48,53,71}}, {4,{18,23,41,57}} } },
   {3, {{4,{11,24,42,76}}, {5,{16,23,41,67,74}}, {3,{22,47,63}} } },
   {3, {{4,{11,24,42,75}}, {5,{15,23,41,67,74}}, {3,{22,47,63}} } },
   {3, {{4,{11,24,42,76}}, {5,{16,23,41,57,74}}, {3,{22,47,53}} } },
   {3, {{4,{11,24,42,75}}, {5,{15,23,41,57,74}}, {3,{22,47,53}} } },
   {4, {{3,{11,24,53}}, {4,{14,23,41,67}}, {3,{47,63,71}}, {2,{51,73}} } },
   {4, {{3,{11,24,63}}, {4,{14,23,41,57}}, {3,{47,53,71}}, {2,{61,73}} } },
   {4, {{3,{11,24,43}}, {2,{14,75}}, {4,{15,23,37,41}}, {3,{27,35,74}} } },
   {4, {{3,{11,24,43}}, {2,{14,76}}, {4,{16,23,37,41}}, {3,{27,36,74}} } },
   {3, {{5,{11,24,37,79,93}}, {3,{14,23,41}}, {4,{39,43,71,97}} } },
   {3, {{5,{11,24,37,78,93}}, {3,{14,23,41}}, {4,{38,43,71,97}} } },
   {3, {{5,{11,24,37,79,83}}, {3,{14,23,41}}, {4,{39,43,71,87}} } },
   {3, {{5,{11,24,37,78,83}}, {3,{14,23,41}}, {4,{38,43,71,87}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,75}}, {3,{23,37,41}}, {3,{27,35,74}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,76}}, {3,{23,37,41}}, {3,{27,36,74}} } },
   {4, {{3,{11,24,43}}, {3,{14,22,71}}, {3,{23,37,41}}, {3,{27,31,72}} } },
   {4, {{3,{11,24,63}}, {4,{14,27,61,73}}, {3,{17,23,41}}, {2,{43,71}} } },
   {4, {{3,{11,24,53}}, {4,{14,27,51,73}}, {3,{17,23,41}}, {2,{43,71}} } },
   {4, {{4,{11,24,37,43}}, {4,{15,23,41,74}}, {2,{17,35}}, {2,{34,75}} } },
   {4, {{4,{11,24,37,43}}, {4,{16,23,41,74}}, {2,{17,36}}, {2,{34,76}} } },
   {4, {{3,{11,24,73}}, {4,{14,27,42,71}}, {3,{17,23,41}}, {2,{43,72}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,75}}, {4,{15,23,41,67}}, {2,{25,74}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,76}}, {4,{16,23,41,67}}, {2,{26,74}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,75}}, {4,{15,23,41,57}}, {2,{25,74}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,76}}, {4,{16,23,41,57}}, {2,{26,74}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,31}}, {3,{22,37,71}}, {3,{23,41,72}} } },
   {4, {{4,{11,24,37,43}}, {2,{17,78}}, {3,{18,23,41}}, {3,{28,34,77}} } },
   {4, {{4,{11,24,37,43}}, {2,{17,79}}, {3,{19,23,41}}, {3,{29,34,77}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,78}}, {3,{18,31,77}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,79}}, {3,{19,31,77}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,73}}, {4,{14,23,41,57}}, {2,{42,71}}, {3,{47,53,72}} } },
   {4, {{3,{11,24,73}}, {4,{14,23,41,67}}, {2,{42,71}}, {3,{47,63,72}} } },
   {3, {{4,{11,24,47,53}}, {3,{14,31,75}}, {5,{23,35,41,57,74}} } },
   {3, {{4,{11,24,47,53}}, {3,{14,31,76}}, {5,{23,36,41,57,74}} } },
   {3, {{4,{11,24,47,63}}, {3,{14,31,75}}, {5,{23,35,41,67,74}} } },
   {3, {{4,{11,24,47,63}}, {3,{14,31,76}}, {5,{23,36,41,67,74}} } },
   {3, {{5,{11,24,47,63,76}}, {3,{16,29,74}}, {4,{19,23,41,67}} } },
   {3, {{5,{11,24,47,63,76}}, {3,{16,28,74}}, {4,{18,23,41,67}} } },
   {3, {{5,{11,24,47,63,75}}, {3,{15,29,74}}, {4,{19,23,41,67}} } },
   {3, {{5,{11,24,47,63,75}}, {3,{15,28,74}}, {4,{18,23,41,67}} } },
   {3, {{5,{11,24,47,53,76}}, {3,{16,29,74}}, {4,{19,23,41,57}} } },
   {3, {{5,{11,24,47,53,76}}, {3,{16,28,74}}, {4,{18,23,41,57}} } },
   {3, {{5,{11,24,47,53,75}}, {3,{15,29,74}}, {4,{19,23,41,57}} } },
   {3, {{5,{11,24,47,53,75}}, {3,{15,28,74}}, {4,{18,23,41,57}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,79}}, {3,{15,23,41}}, {3,{19,25,77}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,78}}, {3,{15,23,41}}, {3,{18,25,77}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,79}}, {3,{16,23,41}}, {3,{19,26,77}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,78}}, {3,{16,23,41}}, {3,{18,26,77}} } },
   {3, {{4,{11,24,43,76}}, {4,{16,27,74,88}}, {4,{18,23,41,87}} } },
   {3, {{4,{11,24,43,76}}, {4,{16,27,74,89}}, {4,{19,23,41,87}} } },
   {3, {{4,{11,24,43,75}}, {4,{15,27,74,88}}, {4,{18,23,41,87}} } },
   {3, {{4,{11,24,43,75}}, {4,{15,27,74,89}}, {4,{19,23,41,87}} } },
   {3, {{4,{11,24,43,76}}, {4,{16,27,74,98}}, {4,{18,23,41,97}} } },
   {3, {{4,{11,24,43,76}}, {4,{16,27,74,99}}, {4,{19,23,41,97}} } },
   {3, {{4,{11,24,43,75}}, {4,{15,27,74,98}}, {4,{18,23,41,97}} } },
   {3, {{4,{11,24,43,75}}, {4,{15,27,74,99}}, {4,{19,23,41,97}} } },
   {3, {{5,{11,24,39,43,77}}, {4,{17,23,41,78}}, {3,{28,34,79}} } },
   {3, {{5,{11,24,38,43,77}}, {4,{17,23,41,79}}, {3,{29,34,78}} } },
   {4, {{3,{11,24,73}}, {2,{14,31}}, {3,{23,37,41}}, {4,{27,34,43,71}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{43,69,77}}, {3,{47,61,79}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{43,68,77}}, {3,{47,61,78}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{43,59,77}}, {3,{47,51,79}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{43,58,77}}, {3,{47,51,78}} } },
   {4, {{4,{11,24,43,76}}, {2,{16,37}}, {2,{17,31}}, {4,{23,36,41,74}} } },
   {4, {{4,{11,24,43,75}}, {2,{15,37}}, {2,{17,31}}, {4,{23,35,41,74}} } },
   },
   {
   {5, {{3,{11,24,37}}, {2,{12,71}}, {3,{17,22,34}}, {2,{21,42}}, {2,{41,72}} } },
   {6, {{2,{11,24}}, {2,{12,71}}, {2,{14,27}}, {2,{17,22}}, {2,{21,42}}, {2,{41,72}} } },
   {5, {{2,{11,24}}, {2,{13,71}}, {2,{14,27}}, {3,{17,22,43}}, {3,{21,42,73}} } },
   {2, {{6,{11,24,43,72,87,96}}, {6,{16,21,42,74,83,97}} } },
   {2, {{6,{11,24,43,72,87,95}}, {6,{15,21,42,74,83,97}} } },
   {2, {{6,{11,24,43,72,97,86}}, {6,{16,21,42,74,93,87}} } },
   {2, {{6,{11,24,43,72,97,85}}, {6,{15,21,42,74,93,87}} } },
   {4, {{3,{11,24,37}}, {3,{12,43,71}}, {3,{17,22,34}}, {3,{21,42,73}} } },
   {5, {{2,{11,24}}, {2,{12,71}}, {4,{14,27,41,72}}, {2,{17,22}}, {2,{21,42}} } },
   {4, {{3,{11,24,37}}, {3,{12,43,71}}, {2,{17,32}}, {4,{21,34,42,73}} } },
   {2, {{6,{11,24,37,43,72,96}}, {6,{17,21,36,42,74,93}} } },
   {2, {{6,{11,24,37,43,72,95}}, {6,{17,21,35,42,74,93}} } },
   {2, {{6,{11,24,37,43,72,86}}, {6,{17,21,36,42,74,83}} } },
   {2, {{6,{11,24,37,43,72,85}}, {6,{17,21,35,42,74,83}} } },
   {4, {{4,{11,24,37,76}}, {2,{12,41}}, {4,{17,22,36,74}}, {2,{21,42}} } },
   {4, {{4,{11,24,37,75}}, {2,{12,41}}, {4,{17,22,35,74}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {3,{13,27,72}}, {3,{14,22,43}}, {4,{17,21,42,73}} } },
   {4, {{2,{11,24}}, {4,{14,22,37,43}}, {3,{17,33,71}}, {3,{21,42,73}} } },
   },
   {
   {4, {{3,{11,24,73}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,71}} } },
   {3, {{4,{11,24,77,83}}, {3,{14,23,41}}, {3,{43,71,87}} } },
   {3, {{4,{11,24,77,93}}, {3,{14,23,41}}, {3,{43,71,97}} } },
   {2, {{5,{11,24,37,43,76}}, {5,{17,23,36,41,74}} } },
   {2, {{5,{11,24,37,43,75}}, {5,{17,23,35,41,74}} } },
   {3, {{3,{11,24,73}}, {4,{14,27,43,71}}, {3,{17,23,41}} } },
   {3, {{4,{11,24,39,77}}, {3,{14,23,41}}, {4,{27,31,43,79}} } },
   {3, {{4,{11,24,38,77}}, {3,{14,23,41}}, {4,{27,31,43,78}} } },
   {3, {{4,{11,24,77,82}}, {3,{14,23,41}}, {4,{22,43,71,87}} } },
   {3, {{4,{11,24,77,92}}, {3,{14,23,41}}, {4,{22,43,71,97}} } },
   {5, {{3,{11,24,63}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,71}}, {2,{61,73}} } },
   {5, {{3,{11,24,53}}, {2,{14,27}}, {3,{17,23,41}}, {2,{43,71}}, {2,{51,73}} } },
   {2, {{6,{11,24,37,49,52,73}}, {6,{17,23,34,41,59,72}} } },
   {2, {{6,{11,24,37,48,52,73}}, {6,{17,23,34,41,58,72}} } },
   {2, {{6,{11,24,37,49,62,73}}, {6,{17,23,34,41,69,72}} } },
   {2, {{6,{11,24,37,48,62,73}}, {6,{17,23,34,41,68,72}} } },
   {5, {{3,{11,24,72}}, {2,{14,27}}, {3,{17,23,41}}, {2,{22,43}}, {2,{42,71}} } },
   {5, {{3,{11,24,73}}, {3,{14,23,41}}, {2,{43,57}}, {2,{47,51}}, {2,{53,71}} } },
   {5, {{3,{11,24,73}}, {3,{14,23,41}}, {2,{43,67}}, {2,{47,61}}, {2,{63,71}} } },
   {4, {{4,{11,24,42,73}}, {2,{14,27}}, {4,{16,23,41,72}}, {2,{17,26}} } },
   {4, {{4,{11,24,42,73}}, {2,{14,27}}, {4,{15,23,41,72}}, {2,{17,25}} } },
   {4, {{3,{11,24,73}}, {2,{14,29}}, {4,{19,23,41,67}}, {3,{47,63,71}} } },
   {4, {{3,{11,24,73}}, {2,{14,28}}, {4,{18,23,41,67}}, {3,{47,63,71}} } },
   {4, {{3,{11,24,73}}, {2,{14,29}}, {4,{19,23,41,57}}, {3,{47,53,71}} } },
   {4, {{3,{11,24,73}}, {2,{14,28}}, {4,{18,23,41,57}}, {3,{47,53,71}} } },
   {3, {{3,{11,24,43}}, {4,{14,28,69,77}}, {5,{17,23,41,68,79}} } },
   {3, {{3,{11,24,43}}, {4,{14,29,68,77}}, {5,{17,23,41,69,78}} } },
   {3, {{3,{11,24,43}}, {4,{14,28,59,77}}, {5,{17,23,41,58,79}} } },
   {3, {{3,{11,24,43}}, {4,{14,29,58,77}}, {5,{17,23,41,59,78}} } },
   {5, {{3,{11,24,73}}, {2,{14,27}}, {3,{17,23,41}}, {2,{42,71}}, {2,{43,72}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,59}}, {4,{19,23,41,77}}, {2,{57,79}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,58}}, {4,{18,23,41,77}}, {2,{57,78}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,69}}, {4,{19,23,41,77}}, {2,{67,79}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,68}}, {4,{18,23,41,77}}, {2,{67,78}} } },
   {2, {{6,{11,24,37,43,68,79}}, {6,{19,23,34,41,67,78}} } },
   {2, {{6,{11,24,37,43,69,78}}, {6,{18,23,34,41,67,79}} } },
   {2, {{6,{11,24,37,43,58,79}}, {6,{19,23,34,41,57,78}} } },
   {2, {{6,{11,24,37,43,59,78}}, {6,{18,23,34,41,57,79}} } },
   {4, {{3,{11,24,73}}, {2,{14,31}}, {4,{23,34,41,67}}, {3,{47,63,71}} } },
   {4, {{3,{11,24,73}}, {2,{14,31}}, {4,{23,34,41,57}}, {3,{47,53,71}} } },
   {3, {{5,{11,24,49,63,78}}, {5,{14,23,41,68,77}}, {2,{47,79}} } },
   {3, {{5,{11,24,48,63,79}}, {5,{14,23,41,69,77}}, {2,{47,78}} } },
   {3, {{5,{11,24,49,53,78}}, {5,{14,23,41,58,77}}, {2,{47,79}} } },
   {3, {{5,{11,24,48,53,79}}, {5,{14,23,41,59,77}}, {2,{47,78}} } },
   {3, {{5,{11,24,38,47,63}}, {5,{14,23,41,68,77}}, {2,{37,78}} } },
   {3, {{5,{11,24,39,47,63}}, {5,{14,23,41,69,77}}, {2,{37,79}} } },
   {3, {{5,{11,24,38,47,53}}, {5,{14,23,41,58,77}}, {2,{37,78}} } },
   {3, {{5,{11,24,39,47,53}}, {5,{14,23,41,59,77}}, {2,{37,79}} } },
   {3, {{5,{11,24,47,62,73}}, {5,{14,23,41,57,72}}, {2,{52,67}} } },
   {3, {{5,{11,24,47,52,73}}, {5,{14,23,41,67,72}}, {2,{62,57}} } },
   {4, {{2,{11,24}}, {4,{12,27,43,71}}, {4,{14,23,41,72}}, {2,{17,21}} } },
   {4, {{4,{11,24,42,73}}, {2,{14,37}}, {4,{17,23,41,72}}, {2,{27,34}} } },
   {3, {{4,{11,24,42,73}}, {3,{14,27,98}}, {5,{18,23,41,72,97}} } },
   {3, {{4,{11,24,42,73}}, {3,{14,27,99}}, {5,{19,23,41,72,97}} } },
   {3, {{4,{11,24,42,73}}, {3,{14,27,88}}, {5,{18,23,41,72,87}} } },
   {3, {{4,{11,24,42,73}}, {3,{14,27,89}}, {5,{19,23,41,72,87}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,78}}, {4,{18,23,41,77}}, {2,{22,43}} } },
   {4, {{3,{11,24,42}}, {3,{14,27,79}}, {4,{19,23,41,77}}, {2,{22,43}} } },
   {4, {{3,{11,24,73}}, {3,{14,27,79}}, {4,{19,23,41,77}}, {2,{43,71}} } },
   {4, {{3,{11,24,73}}, {3,{14,27,78}}, {4,{18,23,41,77}}, {2,{43,71}} } },
   {3, {{3,{11,24,73}}, {5,{14,28,47,63,71}}, {4,{18,23,41,67}} } },
   {3, {{3,{11,24,73}}, {5,{14,29,47,63,71}}, {4,{19,23,41,67}} } },
   {3, {{3,{11,24,73}}, {5,{14,28,47,53,71}}, {4,{18,23,41,57}} } },
   {3, {{3,{11,24,73}}, {5,{14,29,47,53,71}}, {4,{19,23,41,57}} } },
   {4, {{4,{11,24,37,72}}, {3,{14,23,41}}, {3,{27,32,73}}, {2,{43,71}} } },
   {3, {{3,{11,24,83}}, {5,{14,27,43,78,81}}, {4,{18,23,41,77}} } },
   {3, {{3,{11,24,83}}, {5,{14,27,43,79,81}}, {4,{19,23,41,77}} } },
   {3, {{3,{11,24,93}}, {5,{14,27,43,78,91}}, {4,{18,23,41,77}} } },
   {3, {{3,{11,24,93}}, {5,{14,27,43,79,91}}, {4,{19,23,41,77}} } },
   {3, {{3,{11,24,73}}, {5,{14,27,49,63,71}}, {4,{19,23,41,67}} } },
   {3, {{3,{11,24,73}}, {5,{14,27,48,63,71}}, {4,{18,23,41,67}} } },
   {3, {{3,{11,24,73}}, {5,{14,27,49,53,71}}, {4,{19,23,41,57}} } },
   {3, {{3,{11,24,73}}, {5,{14,27,48,53,71}}, {4,{18,23,41,57}} } },
   {3, {{4,{11,24,42,76}}, {5,{16,23,41,67,74}}, {3,{22,47,63}} } },
   {3, {{4,{11,24,42,75}}, {5,{15,23,41,67,74}}, {3,{22,47,63}} } },
   {3, {{4,{11,24,42,76}}, {5,{16,23,41,57,74}}, {3,{22,47,53}} } },
   {3, {{4,{11,24,42,75}}, {5,{15,23,41,57,74}}, {3,{22,47,53}} } },
   {4, {{3,{11,24,53}}, {4,{14,23,41,67}}, {3,{47,63,71}}, {2,{51,73}} } },
   {4, {{3,{11,24,63}}, {4,{14,23,41,57}}, {3,{47,53,71}}, {2,{61,73}} } },
   {4, {{3,{11,24,43}}, {2,{14,75}}, {4,{15,23,37,41}}, {3,{27,35,74}} } },
   {4, {{3,{11,24,43}}, {2,{14,76}}, {4,{16,23,37,41}}, {3,{27,36,74}} } },
   {3, {{5,{11,24,37,79,93}}, {3,{14,23,41}}, {4,{39,43,71,97}} } },
   {3, {{5,{11,24,37,78,93}}, {3,{14,23,41}}, {4,{38,43,71,97}} } },
   {3, {{5,{11,24,37,79,83}}, {3,{14,23,41}}, {4,{39,43,71,87}} } },
   {3, {{5,{11,24,37,78,83}}, {3,{14,23,41}}, {4,{38,43,71,87}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,75}}, {3,{23,37,41}}, {3,{27,35,74}} } },
   {4, {{3,{11,24,43}}, {3,{14,31,76}}, {3,{23,37,41}}, {3,{27,36,74}} } },
   {4, {{3,{11,24,43}}, {3,{14,22,71}}, {3,{23,37,41}}, {3,{27,31,72}} } },
   {4, {{3,{11,24,63}}, {4,{14,27,61,73}}, {3,{17,23,41}}, {2,{43,71}} } },
   {4, {{3,{11,24,53}}, {4,{14,27,51,73}}, {3,{17,23,41}}, {2,{43,71}} } },
   {4, {{4,{11,24,37,43}}, {4,{15,23,41,74}}, {2,{17,35}}, {2,{34,75}} } },
   {4, {{4,{11,24,37,43}}, {4,{16,23,41,74}}, {2,{17,36}}, {2,{34,76}} } },
   {4, {{3,{11,24,73}}, {4,{14,27,42,71}}, {3,{17,23,41}}, {2,{43,72}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,75}}, {4,{15,23,41,67}}, {2,{25,74}} } },
   {4, {{4,{11,24,47,63}}, {2,{14,76}}, {4,{16,23,41,67}}, {2,{26,74}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,75}}, {4,{15,23,41,57}}, {2,{25,74}} } },
   {4, {{4,{11,24,47,53}}, {2,{14,76}}, {4,{16,23,41,57}}, {2,{26,74}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,31}}, {3,{22,37,71}}, {3,{23,41,72}} } },
   {4, {{4,{11,24,37,43}}, {2,{17,78}}, {3,{18,23,41}}, {3,{28,34,77}} } },
   {4, {{4,{11,24,37,43}}, {2,{17,79}}, {3,{19,23,41}}, {3,{29,34,77}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,78}}, {3,{18,31,77}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,79}}, {3,{19,31,77}}, {3,{23,37,41}} } },
   {4, {{3,{11,24,73}}, {4,{14,23,41,57}}, {2,{42,71}}, {3,{47,53,72}} } },
   {4, {{3,{11,24,73}}, {4,{14,23,41,67}}, {2,{42,71}}, {3,{47,63,72}} } },
   {3, {{4,{11,24,47,53}}, {3,{14,31,75}}, {5,{23,35,41,57,74}} } },
   {3, {{4,{11,24,47,53}}, {3,{14,31,76}}, {5,{23,36,41,57,74}} } },
   {3, {{4,{11,24,47,63}}, {3,{14,31,75}}, {5,{23,35,41,67,74}} } },
   {3, {{4,{11,24,47,63}}, {3,{14,31,76}}, {5,{23,36,41,67,74}} } },
   {3, {{5,{11,24,47,63,76}}, {3,{16,29,74}}, {4,{19,23,41,67}} } },
   {3, {{5,{11,24,47,63,76}}, {3,{16,28,74}}, {4,{18,23,41,67}} } },
   {3, {{5,{11,24,47,63,75}}, {3,{15,29,74}}, {4,{19,23,41,67}} } },
   {3, {{5,{11,24,47,63,75}}, {3,{15,28,74}}, {4,{18,23,41,67}} } },
   {3, {{5,{11,24,47,53,76}}, {3,{16,29,74}}, {4,{19,23,41,57}} } },
   {3, {{5,{11,24,47,53,76}}, {3,{16,28,74}}, {4,{18,23,41,57}} } },
   {3, {{5,{11,24,47,53,75}}, {3,{15,29,74}}, {4,{19,23,41,57}} } },
   {3, {{5,{11,24,47,53,75}}, {3,{15,28,74}}, {4,{18,23,41,57}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,79}}, {3,{15,23,41}}, {3,{19,25,77}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,78}}, {3,{15,23,41}}, {3,{18,25,77}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,79}}, {3,{16,23,41}}, {3,{19,26,77}} } },
   {4, {{3,{11,24,43}}, {3,{14,27,78}}, {3,{16,23,41}}, {3,{18,26,77}} } },
   {3, {{4,{11,24,43,76}}, {4,{16,27,74,88}}, {4,{18,23,41,87}} } },
   {3, {{4,{11,24,43,76}}, {4,{16,27,74,89}}, {4,{19,23,41,87}} } },
   {3, {{4,{11,24,43,75}}, {4,{15,27,74,88}}, {4,{18,23,41,87}} } },
   {3, {{4,{11,24,43,75}}, {4,{15,27,74,89}}, {4,{19,23,41,87}} } },
   {3, {{4,{11,24,43,76}}, {4,{16,27,74,98}}, {4,{18,23,41,97}} } },
   {3, {{4,{11,24,43,76}}, {4,{16,27,74,99}}, {4,{19,23,41,97}} } },
   {3, {{4,{11,24,43,75}}, {4,{15,27,74,98}}, {4,{18,23,41,97}} } },
   {3, {{4,{11,24,43,75}}, {4,{15,27,74,99}}, {4,{19,23,41,97}} } },
   {3, {{5,{11,24,39,43,77}}, {4,{17,23,41,78}}, {3,{28,34,79}} } },
   {3, {{5,{11,24,38,43,77}}, {4,{17,23,41,79}}, {3,{29,34,78}} } },
   {4, {{3,{11,24,73}}, {2,{14,31}}, {3,{23,37,41}}, {4,{27,34,43,71}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{43,69,77}}, {3,{47,61,79}} } },
   {4, {{3,{11,24,63}}, {3,{14,23,41}}, {3,{43,68,77}}, {3,{47,61,78}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{43,59,77}}, {3,{47,51,79}} } },
   {4, {{3,{11,24,53}}, {3,{14,23,41}}, {3,{43,58,77}}, {3,{47,51,78}} } },
   {4, {{4,{11,24,43,76}}, {2,{16,37}}, {2,{17,31}}, {4,{23,36,41,74}} } },
   {4, {{4,{11,24,43,75}}, {2,{15,37}}, {2,{17,31}}, {4,{23,35,41,74}} } },
   {5, {{3,{11,24,37}}, {2,{12,71}}, {3,{17,22,34}}, {2,{21,42}}, {2,{41,72}} } },
   {6, {{2,{11,24}}, {2,{12,71}}, {2,{14,27}}, {2,{17,22}}, {2,{21,42}}, {2,{41,72}} } },
   {5, {{2,{11,24}}, {2,{13,71}}, {2,{14,27}}, {3,{17,22,43}}, {3,{21,42,73}} } },
   {2, {{6,{11,24,43,72,87,96}}, {6,{16,21,42,74,83,97}} } },
   {2, {{6,{11,24,43,72,87,95}}, {6,{15,21,42,74,83,97}} } },
   {2, {{6,{11,24,43,72,97,86}}, {6,{16,21,42,74,93,87}} } },
   {2, {{6,{11,24,43,72,97,85}}, {6,{15,21,42,74,93,87}} } },
   {4, {{3,{11,24,37}}, {3,{12,43,71}}, {3,{17,22,34}}, {3,{21,42,73}} } },
   {5, {{2,{11,24}}, {2,{12,71}}, {4,{14,27,41,72}}, {2,{17,22}}, {2,{21,42}} } },
   {4, {{3,{11,24,37}}, {3,{12,43,71}}, {2,{17,32}}, {4,{21,34,42,73}} } },
   {2, {{6,{11,24,37,43,72,96}}, {6,{17,21,36,42,74,93}} } },
   {2, {{6,{11,24,37,43,72,95}}, {6,{17,21,35,42,74,93}} } },
   {2, {{6,{11,24,37,43,72,86}}, {6,{17,21,36,42,74,83}} } },
   {2, {{6,{11,24,37,43,72,85}}, {6,{17,21,35,42,74,83}} } },
   {4, {{4,{11,24,37,76}}, {2,{12,41}}, {4,{17,22,36,74}}, {2,{21,42}} } },
   {4, {{4,{11,24,37,75}}, {2,{12,41}}, {4,{17,22,35,74}}, {2,{21,42}} } },
   {4, {{2,{11,24}}, {3,{13,27,72}}, {3,{14,22,43}}, {4,{17,21,42,73}} } },
   {4, {{2,{11,24}}, {4,{14,22,37,43}}, {3,{17,33,71}}, {3,{21,42,73}} } },
   },
   {
   {3, {{4,{11,24,43,72}}, {2,{14,27}}, {4,{17,23,42,71}} } },
   {2, {{5,{11,24,37,43,72}}, {5,{17,23,34,42,71}} } },
   {4, {{3,{11,24,73}}, {3,{12,43,71}}, {2,{14,27}}, {3,{17,23,42}} } },
   {3, {{4,{11,24,43,72}}, {4,{14,23,37,42}}, {3,{17,32,71}} } },
   {3, {{3,{11,24,37}}, {4,{14,23,42,71}}, {4,{27,31,43,72}} } },
   {5, {{2,{11,24}}, {3,{12,43,71}}, {2,{14,27}}, {3,{17,23,42}}, {2,{21,73}} } },
   {4, {{3,{11,24,37}}, {3,{12,43,71}}, {4,{17,23,34,42}}, {2,{21,73}} } },
   {4, {{3,{11,24,73}}, {2,{12,41}}, {3,{14,27,32}}, {4,{23,37,42,71}} } },
   {3, {{5,{11,24,49,53,72}}, {2,{14,27}}, {5,{17,23,42,59,71}} } },
   {3, {{5,{11,24,48,53,72}}, {2,{14,27}}, {5,{17,23,42,58,71}} } },
   {3, {{5,{11,24,49,63,72}}, {2,{14,27}}, {5,{17,23,42,69,71}} } },
   {3, {{5,{11,24,48,63,72}}, {2,{14,27}}, {5,{17,23,42,68,71}} } },
   {4, {{4,{11,24,43,72}}, {2,{14,27}}, {3,{17,32,71}}, {3,{23,37,42}} } },
   {3, {{5,{11,24,43,77,82}}, {2,{14,29}}, {5,{19,23,42,71,87}} } },
   {3, {{5,{11,24,43,77,82}}, {2,{14,28}}, {5,{18,23,42,71,87}} } },
   {3, {{5,{11,24,43,77,92}}, {2,{14,29}}, {5,{19,23,42,71,97}} } },
   {3, {{5,{11,24,43,77,92}}, {2,{14,28}}, {5,{18,23,42,71,97}} } },
   {5, {{2,{11,24}}, {3,{12,27,73}}, {3,{14,23,42}}, {2,{17,21}}, {2,{43,72}} } },
   {2, {{6,{11,24,43,75,82,97}}, {6,{15,23,42,77,84,91}} } },
   {2, {{6,{11,24,43,76,82,97}}, {6,{16,23,42,77,84,91}} } },
   {2, {{6,{11,24,43,75,92,87}}, {6,{15,23,42,77,94,81}} } },
   {2, {{6,{11,24,43,76,92,87}}, {6,{16,23,42,77,94,81}} } },
   {3, {{4,{11,24,77,83}}, {5,{14,23,42,71,97}}, {3,{43,87,92}} } },
   {3, {{4,{11,24,77,93}}, {5,{14,23,42,71,87}}, {3,{43,97,82}} } },
   {5, {{3,{11,24,73}}, {2,{12,41}}, {2,{14,27}}, {3,{17,23,42}}, {2,{43,71}} } },
   {4, {{3,{11,24,43}}, {2,{12,71}}, {4,{14,27,41,72}}, {3,{17,23,42}} } },
   {4, {{3,{11,24,37}}, {3,{12,43,71}}, {3,{14,23,42}}, {3,{27,31,73}} } },
   {5, {{3,{11,24,73}}, {2,{14,27}}, {3,{17,22,71}}, {2,{23,42}}, {2,{43,72}} } },
   {4, {{4,{11,24,37,73}}, {4,{17,32,43,71}}, {2,{22,34}}, {2,{23,42}} } },
   {2, {{6,{11,24,43,76,87,92}}, {6,{16,23,42,71,84,97}} } },
   {2, {{6,{11,24,43,75,87,92}}, {6,{15,23,42,71,84,97}} } },
   {2, {{6,{11,24,43,76,97,82}}, {6,{16,23,42,71,94,87}} } },
   {2, {{6,{11,24,43,75,97,82}}, {6,{15,23,42,71,94,87}} } },
   {4, {{4,{11,24,77,83}}, {3,{12,71,87}}, {3,{14,23,42}}, {2,{43,82}} } },
   {4, {{4,{11,24,77,93}}, {3,{12,71,97}}, {3,{14,23,42}}, {2,{43,92}} } },
   {4, {{3,{11,24,43}}, {4,{14,27,41,72}}, {3,{17,22,71}}, {2,{23,42}} } },
   {4, {{4,{11,24,43,72}}, {2,{14,37}}, {2,{17,31}}, {4,{23,34,42,71}} } },
   {4, {{3,{11,24,43}}, {3,{12,71,97}}, {3,{14,23,42}}, {3,{41,77,92}} } },
   {4, {{3,{11,24,43}}, {3,{12,71,87}}, {3,{14,23,42}}, {3,{41,77,82}} } },
   {3, {{5,{11,24,43,77,92}}, {2,{14,31}}, {5,{23,34,42,71,97}} } },
   {3, {{5,{11,24,43,77,82}}, {2,{14,31}}, {5,{23,34,42,71,87}} } },
   {4, {{4,{11,24,43,72}}, {2,{14,27}}, {2,{17,31}}, {4,{23,37,42,71}} } },
   {4, {{3,{11,24,73}}, {2,{12,71}}, {4,{14,23,42,57}}, {3,{47,53,72}} } },
   {4, {{3,{11,24,73}}, {2,{12,71}}, {4,{14,23,42,67}}, {3,{47,63,72}} } },
   {4, {{4,{11,24,43,72}}, {2,{14,31}}, {4,{23,37,42,71}}, {2,{27,34}} } },
   {4, {{4,{11,24,73,87}}, {4,{12,43,77,81}}, {2,{14,22}}, {2,{23,42}} } },
   {4, {{4,{11,24,73,97}}, {4,{12,43,77,91}}, {2,{14,22}}, {2,{23,42}} } },
   {4, {{3,{11,24,63}}, {2,{12,71}}, {4,{14,23,42,67}}, {3,{47,61,72}} } },
   {4, {{3,{11,24,53}}, {2,{12,71}}, {4,{14,23,42,57}}, {3,{47,51,72}} } },
   {4, {{3,{11,24,73}}, {2,{14,27}}, {4,{17,32,43,71}}, {3,{23,37,42}} } },
   {4, {{4,{11,24,43,67}}, {3,{14,22,71}}, {2,{23,42}}, {3,{47,61,72}} } },
   {4, {{4,{11,24,43,57}}, {3,{14,22,71}}, {2,{23,42}}, {3,{47,51,72}} } },
   }
};

int const cntpat3x3[ 4 ] = {137, 18, 155, 52};

#endif /* BLUEPRINTS_H_ */
